/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016-2017 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "usb_pd_protocol.h"
#include "tcpm.h"
#include "tusb422_common.h"
#include "usb_pd.h"
#include "usb_pd_policy_engine.h"
#ifdef CONFIG_TUSB422
	#include <linux/string.h>
#else
	#include <string.h>
#endif


/* PD Timeout Values */
#define T_CHUNK_SENDER_RESPONSE_MS    25   /*  24 - 30 ms */
#define T_CHUNK_SENDER_REQUEST_MS     25   /*  24 - 30 ms */

#define MSG_ID_CLEARED 0xCE

extern usb_pd_port_t pd[NUM_TCPC_DEVICES];
extern usb_pd_port_config_t* usb_pd_pm_get_config(unsigned int port);

#if DEBUG_LEVEL >= 3

static const char * const ctrlmsg2string[]=
{
	"",                       /* 0x00 */
	"GOOD_CRC",               /* 0x01 */
	"GOTO_MIN",               /* 0x02 */
	"ACCEPT",                 /* 0x03 */
	"REJECT",                 /* 0x04 */
	"PING",                   /* 0x05 */
	"PS_RDY",                 /* 0x06 */
	"GET_SRC_CAP",            /* 0x07 */
	"GET_SNK_CAP",            /* 0x08 */
	"DR_SWAP",                /* 0x09 */
	"PR_SWAP",                /* 0x0A */
	"VCONN_SWAP",             /* 0x0B */
	"WAIT",                   /* 0x0C */
	"SOFT_RESET",             /* 0x0D */
	"",                       /* 0x0E */
	"",                       /* 0x0F */
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	"NOT_SUPPORTED",          /* 0x10 */
	"GET_SRC_CAP_EXT",        /* 0x11 */
	"GET_STATUS",             /* 0x12 */
	"FR_SWAP",                /* 0x13 */
	"GET_PSS_STAT",           /* 0x14 */
	"GET_COUNTRY_CODES"       /* 0x15 */
#endif
};

static const char * const datamsg2string[]=
{
	"",                   /* 0x00 */
	"SRC_CAPS",           /* 0x01 */
	"REQUEST",            /* 0x02 */
	"BIST",               /* 0x03 */
	"SNK_CAPS",           /* 0x04 */
	"BATT_STATUS",        /* 0x05 */
	"ALERT",              /* 0x06 */
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	"GET_COUNTRY_INFO",   /* 0x07 */
#else
	"",
#endif
	"", "", "", "", "", "", "",  /* 0x08 - 0x0E */
	"VENDOR"              /* 0x0F */
};

#ifdef ENABLE_USB_PD_REV3_SUPPORT
static const char * const extmsg2string[]=
{
	"",                  /* 0x00 */
	"SRC_CAPS_EXT",      /* 0x01 */   /* SOP only */
	"STATUS",            /* 0x02 */   /* SOP only */
	"GET_BATT_CAP",      /* 0x03 */   /* SOP only */
	"GET_BATT_STAT",     /* 0x04 */   /* SOP only */
	"BATT_CAP",          /* 0x05 */   /* SOP only */
	"GET_MANUF_INFO",    /* 0x06 */
	"MANUF_INFO",        /* 0x07 */
	"SECURITY_REQ",      /* 0x08 */
	"SECURITY_RESP",     /* 0x09 */
	"FW_UPDATE_REQ",     /* 0x0A */
	"FW_UPDATE_RESP",    /* 0x0B */
};
#endif
#endif /* DEBUG_LEVEL */

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
#if DEBUG_LEVEL >= 1
const char * const rchstate2string[NUM_RCH_STATES] =
{
	"WAIT_FOR_MSG",
	"REQUESTING_CHUNK",
	"WAITING_CHUNK",
};

const char * const tchstate2string[NUM_TCH_STATES] =
{
	"WAIT_FOR_MSG_REQ",
	"CONSTRUCT_MSG",
	"SENDING_MSG",
	"WAIT_CHUNK_REQ",
};
#endif /* DEBUG_LEVEL */

static void prl_set_rch_state(usb_pd_port_t *dev, rch_state_t new_state)
{
	CRIT("RCH_%s\n", rchstate2string[new_state]);
	dev->rch_state = new_state;
	dev->chunked_state_change = true;
}

static void prl_set_tch_state(usb_pd_port_t *dev, tch_state_t new_state)
{
	CRIT("TCH_%s\n", tchstate2string[new_state]);
	dev->tch_state = new_state;
	dev->chunked_state_change = true;
}
#endif


void usb_pd_prl_reset(unsigned int port)
{
	unsigned int sop_type;
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	// Reset all msg IDs.
	for (sop_type = 0; sop_type < MAX_SOP_NUM; sop_type++)
	{
		dev->msg_id[sop_type] = 0;
		dev->stored_msg_id[sop_type] = MSG_ID_CLEARED;
	}

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	dev->abort_flag = false;
	prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);
	prl_set_tch_state(dev, TCH_WAIT_FOR_MSG_REQ);
#endif

	return;
}

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT

#define MAX_EXT_MSG_CHUNK_LEN  26 /* bytes */
#define USB_PD_EXT_HDR_GET_CHUNKED(header)       ((header) >> 15)
#define USB_PD_EXT_HDR_GET_CHUNK_NUM(header)     (((header) >> 11) & 0x0F)
#define USB_PD_EXT_HDR_GET_REQUEST_CHUNK(header) (((header) >> 10) & 0x01)
#define USB_PD_EXT_HDR_GET_DATA_SIZE(header)     ((header) & 0x1FF)

static void timeout_chunk_sender_response(unsigned int port)
{
	prl_set_rch_state(&pd[port], RCH_WAIT_FOR_MSG);
	return;
}

static void timeout_chunk_sender_request(unsigned int port)
{
	usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_SUCCESS);
	prl_set_tch_state(&pd[port], TCH_WAIT_FOR_MSG_REQ);
	return;
}
#endif

static void usb_pd_prl_transmit_alert_handler(unsigned int port, tx_status_t tx_status)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	if (tx_status != TX_STATUS_DISCARDED)
	{
		// Increment message ID.
		dev->msg_id[dev->tx_sop_type]++;
	}

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	if (tx_status == TX_STATUS_SUCCESS)
	{
		if (dev->rch_state == RCH_REQUESTING_CHUNK)
		{
			prl_set_rch_state(dev, RCH_WAITING_CHUNK);

			// Start chunck sender response timer.
			timer_start(&dev->timer, T_CHUNK_SENDER_RESPONSE_MS, timeout_chunk_sender_response);	
		}
		else if (dev->tch_state == TCH_SENDING_MSG)
		{
			dev->chunk_num_to_send++;
			if ((dev->chunk_num_to_send * MAX_EXT_MSG_CHUNK_LEN) >= dev->extended_msg_byte_cnt)
			{
				// Last chunk.
				prl_set_tch_state(dev, TCH_WAIT_FOR_MSG_REQ);
			}
			else
			{
				prl_set_tch_state(dev, TCH_WAIT_CHUNK_REQ);

				// Start chunk sender request timer.
				timer_start(&dev->timer, T_CHUNK_SENDER_REQUEST_MS, timeout_chunk_sender_request);	
			}
		}
		else
		{
			// GoodCRC received.
			usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_SUCCESS);
		}
	}
	else
	{
		// Report error to policy engine. 

		if (dev->rch_state != RCH_WAIT_FOR_MSG)
		{
			prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);
		}

		if (dev->tch_state != TCH_WAIT_FOR_MSG_REQ)
		{
			prl_set_tch_state(dev, TCH_WAIT_FOR_MSG_REQ);
		}

		if (tx_status == TX_STATUS_DISCARDED)
		{
			// Either we issued a Hard Reset or received a msg.
			usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_DISCARDED);
		}
		else if (tx_status == TX_STATUS_FAILED)
		{
			// No valid GoodCRC received.  
			usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_FAILED);
		}
	}

#else

	if (tx_status == TX_STATUS_SUCCESS)
	{
		// GoodCRC received.
		usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_SUCCESS);
	}
	else if (tx_status == TX_STATUS_DISCARDED)
	{
		// Either we issued a Hard Reset or received a msg.
		usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_DISCARDED);
	}
	else if (tx_status == TX_STATUS_FAILED)
	{
		// No valid GoodCRC received.  
		usb_pd_pe_notify(port, PRL_ALERT_MSG_TX_FAILED);
	}

#endif

	return;
}

static void usb_pd_prl_hard_reset_alert_handler(unsigned int port)
{
	usb_pd_pe_notify(port, PRL_ALERT_HARD_RESET_RECEIVED);
	return;
}


static void usb_pd_prl_tx_msg(unsigned int port, uint8_t *buf, tcpc_transmit_t sop_type)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	// Cable Plugs, nRetryCount = 0. (retry is not allowed)
	// USB-PD rev2, nRetryCount = 3.
	// USB-PD rev3, nRetryCount = 2.
#ifdef CABLE_PLUG
	tcpm_transmit(port, buf, sop_type, 0);
#else
	if (sop_type == TCPC_TX_SOP)
	{
		tcpm_transmit(port, buf, TCPC_TX_SOP, (dev->p2p_spec_rev > PD_REV20) ? 2 : 3);
	}
	else /* SOP' or SOP" */
	{
		tcpm_transmit(port, buf, sop_type, (dev->p2c_spec_rev > PD_REV20) ? 2 : 3);
	}
#endif
	// Save the SOP type.
	dev->tx_sop_type = sop_type;

	return;
}

static void usb_pd_prl_handle_soft_reset(usb_pd_port_t *dev, uint8_t sop_type)
{
	// Reset message ID counter.
	dev->msg_id[sop_type] = 0;
	dev->stored_msg_id[sop_type] = MSG_ID_CLEARED;

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	dev->abort_flag = false;
	prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);
	prl_set_tch_state(dev, TCH_WAIT_FOR_MSG_REQ);
#endif

	return;
}


void usb_pd_prl_tx_ctrl_msg(unsigned int port, msg_hdr_ctrl_msg_type_t msg_type, tcpc_transmit_t sop_type)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	DEBUG("Tx msg_type: %u (%s), sop: %u\n", msg_type, ctrlmsg2string[msg_type], sop_type);

	if (msg_type == CTRL_MSG_TYPE_SOFT_RESET)
	{
		usb_pd_prl_handle_soft_reset(dev, sop_type);
	}

	dev->msg_buf[0] = 2;	/* Tx byte cnt */

	if (sop_type == TCPC_TX_SOP)
	{
		dev->msg_buf[1] = USB_PD_HDR_GEN_BYTE0(dev->p2p_spec_rev, dev->data_role, msg_type);
	}
	else /* SOP' or SOP" */
	{
		dev->msg_buf[1] = USB_PD_HDR_GEN_BYTE0(dev->p2c_spec_rev, 0, msg_type);
	}

	dev->msg_buf[2] = USB_PD_HDR_GEN_BYTE1(0, 0, dev->msg_id[sop_type], dev->power_role);

	INFO("buf[0-2]: 0x%02x %02x %02x\n", dev->msg_buf[0], dev->msg_buf[1], dev->msg_buf[2]);

	usb_pd_prl_tx_msg(port, dev->msg_buf, sop_type);

	return;
}

static void tx_data_msg(unsigned int port, uint8_t msg_type, tcpc_transmit_t sop_type, unsigned int ndo, uint8_t extended_bit)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	DEBUG("Tx %s msg_type: %u (%s), sop: %u, ndo: %u\n", ((extended_bit) ? "ext" : "data"), msg_type, 
		  ((extended_bit) ? extmsg2string[msg_type] : datamsg2string[msg_type]), sop_type, ndo);

	dev->msg_buf[0] = (ndo << 2) + 2; // Each data object is 4-bytes plus 2-byte header.

	if (sop_type == TCPC_TX_SOP)
	{
		dev->msg_buf[1] = USB_PD_HDR_GEN_BYTE0(dev->p2p_spec_rev, dev->data_role, msg_type);
	}
	else /* SOP' or SOP" */
	{
		dev->msg_buf[1] = USB_PD_HDR_GEN_BYTE0(dev->p2c_spec_rev, 0, msg_type);
	}
#ifdef CABLE_PLUG
	// Set Cable Plug bit in msg header.
	dev->msg_buf[2] = USB_PD_HDR_GEN_BYTE1(extended_bit, ndo, dev->msg_id[sop_type], 1);
#else
	dev->msg_buf[2] = USB_PD_HDR_GEN_BYTE1(extended_bit, ndo, dev->msg_id[sop_type], (sop_type == TCPC_TX_SOP) ? dev->power_role : 0);
#endif
	INFO("buf[0-2]: 0x%02x %02x %02x\n", dev->msg_buf[0], dev->msg_buf[1], dev->msg_buf[2]);

	usb_pd_prl_tx_msg(port, dev->msg_buf, sop_type);

	return;
}


void usb_pd_prl_tx_data_msg(unsigned int port, msg_hdr_data_msg_type_t msg_type, tcpc_transmit_t sop_type, unsigned int ndo)
{
	tx_data_msg(port, msg_type, sop_type, ndo, 0);
	return;
}

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT


static void usb_pd_prl_rch_request_chunk(usb_pd_port_t *dev)
{
	uint16_t ext_hdr;

	// Create ext header with Chunked and Request Chunk bits set and data size of zero.
	ext_hdr = USB_PD_EXT_HDR_GEN(1, dev->expected_chunk_num, 1, 0);
	dev->msg_buf[3] = ext_hdr & 0xFF;
	dev->msg_buf[4] = ext_hdr >> 8;
	dev->msg_buf[5] = 0;
	dev->msg_buf[6] = 0;

	tx_data_msg(dev->port, dev->rx_msg_type, (tcpc_transmit_t)dev->rx_sop, 1, 1);

	return;
}


static void usb_pd_prl_tch_send_chunked_msg(usb_pd_port_t *dev)
{
	tx_data_msg(dev->port, dev->chunked_msg_type, (tcpc_transmit_t)dev->rx_sop, dev->ndos_to_send, 1);
	return;
}

void usb_pd_prl_chunking_state_machine(unsigned int port)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	if (dev->chunked_state_change)
	{
		dev->chunked_state_change = false;

		if (dev->rch_state == RCH_REQUESTING_CHUNK)
		{
			usb_pd_prl_rch_request_chunk(dev);
		}
		else if (dev->tch_state == TCH_SENDING_MSG)
		{
			usb_pd_prl_tch_send_chunked_msg(dev);
		}
	}

	return;
}


static void usb_pd_prl_construct_chunked_msg(usb_pd_port_t *dev)
{
	uint16_t ext_hdr;
	uint16_t copy_len;

	memset(dev->msg_buf, 0, sizeof(dev->msg_buf));

	ext_hdr = USB_PD_EXT_HDR_GEN(1, dev->chunk_num_to_send, 0, dev->extended_msg_byte_cnt);
	dev->msg_buf[3] = ext_hdr & 0xFF;
	dev->msg_buf[4] = ext_hdr >> 8;

	copy_len = MIN(MAX_EXT_MSG_CHUNK_LEN, (dev->extended_msg_byte_cnt - (dev->chunk_num_to_send * MAX_EXT_MSG_CHUNK_LEN))); 

	memcpy(&dev->msg_buf[5], &dev->extended_msg_buf[dev->chunk_num_to_send * MAX_EXT_MSG_CHUNK_LEN], copy_len);

	// Add 2 for ext msg header and add 3 to pad to next data object.
	dev->ndos_to_send = (copy_len + 2 + 3) >> 2;

	prl_set_tch_state(dev, TCH_SENDING_MSG);

	return;
}


void usb_pd_prl_tx_data_msg_ext(unsigned int port, ext_msg_hdr_msg_type_t msg_type, tcpc_transmit_t sop_type, uint16_t len)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);

	DEBUG("Tx ext msg_type: %u (%s), sop: %u, len: %u\n", msg_type, extmsg2string[msg_type], sop_type, len);

	dev->chunked_msg_type = msg_type;
	dev->chunk_num_to_send = 0;
	dev->extended_msg_byte_cnt = len;

	// Check extended header to see if this is a chunked message.
	// For chunked messages, data must be padded to the next NDO.
	// For unchunked messages, NDO field is set to zero.
	// 
	// TUSB422 does not support unchucked extended messages due to Rx buffer limitation.

	usb_pd_prl_construct_chunked_msg(dev);

	return;
}


#ifndef ENABLE_EXTENDED_MSG_SUPPORT
static void timeout_chunking_not_supported(unsigned int port)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);
	
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_NOT_SUPPORTED, TCPC_TX_SOP);
	return;
}
#endif

static void usb_pd_prl_chunked_rx_handler(usb_pd_port_t *dev)
{
	uint16_t ext_hdr;
	uint8_t chunk_num;
	uint8_t chunk_len;
	uint16_t data_size;
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	INFO("%s\n", __func__);

	ext_hdr = (dev->msg_buf[1] << 8) | dev->msg_buf[0];

	if (USB_PD_EXT_HDR_GET_CHUNKED(ext_hdr) && 
		!USB_PD_EXT_HDR_GET_REQUEST_CHUNK(ext_hdr))
	{
#ifdef ENABLE_EXTENDED_MSG_SUPPORT
		if (dev->abort_flag)
		{
			dev->abort_flag = false;
			prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);
			return;
		}

		chunk_num = USB_PD_EXT_HDR_GET_CHUNK_NUM(ext_hdr);
		data_size = USB_PD_EXT_HDR_GET_DATA_SIZE(ext_hdr);
		chunk_len = MIN((dev->rx_msg_data_len - 2), MAX_EXT_MSG_CHUNK_LEN);

		INFO("chunk_num = %u, data_size = %u bytes\n", chunk_num, data_size);

		if (chunk_num == 0)
		{
			dev->expected_chunk_num = 0;
		}

		if (chunk_num == dev->expected_chunk_num)
		{
			// Handle received chunk. (26-bytes max)
			memcpy(&dev->extended_msg_buf[chunk_num * MAX_EXT_MSG_CHUNK_LEN], &dev->msg_buf[2], chunk_len);

			if (((chunk_num * MAX_EXT_MSG_CHUNK_LEN) + chunk_len) < data_size)
			{
				// Request next chunk.
				dev->expected_chunk_num++;

				prl_set_rch_state(dev, RCH_REQUESTING_CHUNK);
			}
			else /* Message complete */
			{
				dev->extended_msg_byte_cnt = data_size;
	
				usb_pd_pe_notify(dev->port, PRL_ALERT_MSG_RECEIVED);
			}
		}
		else
		{
			// Report error to policy engine.

			prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);
		}
#else
		// Delay before sending NOT SUPPORTED response.
		timer_start(&dev->timer, T_CHUNKING_NOT_SUPPORTED_MS, timeout_chunking_not_supported);	
#endif
	}
	else /* Unchunked */
	{
		if (config->unchunked_msg_support)
		{
			// Handle unchunked extended message.
			// TUSB422 does not support unchucked messages so this should never occur.
		}
		else /* Chunked != Chunking */
		{
			// Report error to policy engine.

			prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);

			usb_pd_pe_notify(dev->port, PRL_ALERT_MSG_RECEIVED);
		}
	}

	return;
}


static void usb_pd_prl_chunked_msg_router(usb_pd_port_t *dev)
{
	uint16_t ext_hdr;
	uint8_t chunk_num;

	if ((dev->rx_msg_data_len == 0) &&
		(dev->rx_msg_type == CTRL_MSG_TYPE_PING) &&
		!dev->rx_msg_extended)
	{
		usb_pd_pe_notify(dev->port, PRL_ALERT_MSG_RECEIVED);
	}
	else if (dev->tch_state == TCH_WAIT_CHUNK_REQ)
	{
		// The following is part of the Chunked Tx state machine.

		ext_hdr = (dev->msg_buf[1] << 8) | dev->msg_buf[0];

		chunk_num = USB_PD_EXT_HDR_GET_CHUNK_NUM(ext_hdr);

		// Check if this is a chunk request and that the 
		// chunk num requested matches the chunk number to send.
		if (USB_PD_EXT_HDR_GET_REQUEST_CHUNK(ext_hdr) &&
			(chunk_num == dev->chunk_num_to_send))
		{
			usb_pd_prl_construct_chunked_msg(dev);
		}
		else
		{
			prl_set_tch_state(dev, TCH_WAIT_FOR_MSG_REQ);
			usb_pd_prl_chunked_rx_handler(dev);
		}
	}
	else
	{
		usb_pd_prl_chunked_rx_handler(dev);
	}

	return;
}

#endif /* ENABLE_USB_PD_REV3_SUPPORT */




static void usb_pd_prl_receive_alert_handler(unsigned int port)
{
	usb_pd_port_t *dev = usb_pd_pe_get_device(port);
	uint8_t sop;
	uint8_t msg_id;
	uint16_t hdr;

	if (*dev->current_state == PE_BIST_TEST_MODE)
	{
		return;
	}

	// Read Rx header and SOP type.
	tcpm_get_msg_header_type(port, &sop, &hdr);
	dev->rx_sop = sop;
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	dev->rx_msg_extended = USB_PD_HDR_GET_EXT_MSG_TYPE(hdr);
#endif
	dev->rx_msg_type = USB_PD_HDR_GET_MSG_TYPE(hdr);
	dev->rx_msg_data_len = USB_PD_HDR_GET_DATA_LEN(hdr);
	dev->rx_msg_spec_rev = USB_PD_HDR_GET_SPEC_REV(hdr);

	if (sop == 0)
	{
		// Check port data role field for SOP pkts.
		if (dev->data_role == USB_PD_HDR_GET_DATA_ROLE(hdr))
		{
			PRINT("Rx hdr data role %sFP invalid! msg_type = 0x%x, len = %u.\n", 
				  (USB_PD_HDR_GET_DATA_ROLE(hdr) == PD_DATA_ROLE_UFP) ? "U" : "D", 
				  dev->rx_msg_type, dev->rx_msg_data_len);

			// Perform Type-C Error Recovery.
			tcpm_execute_error_recovery(port);
			return;
		}
	}

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	if (dev->rx_msg_extended)
	{
		DEBUG("Rx ext msg len: %u-bytes, type: %u (%s)\n", dev->rx_msg_data_len, 
			  dev->rx_msg_type, (dev->rx_msg_type <= 0x0B) ? extmsg2string[dev->rx_msg_type] : "?");
	}
	else
	{
		DEBUG("Rx msg len: %u-bytes, type: %u (%s)\n", dev->rx_msg_data_len, 
			  dev->rx_msg_type, (dev->rx_msg_data_len) ? 
			  datamsg2string[dev->rx_msg_type & 0x0F] : 
			  (dev->rx_msg_type <= 0x15) ? ctrlmsg2string[dev->rx_msg_type] : "?");
	}
#else /* PD 2.0 */
	DEBUG("Rx msg len: %u-bytes, type: %u (%s)\n", dev->rx_msg_data_len, 
		  dev->rx_msg_type, (dev->rx_msg_data_len) ? 
		  datamsg2string[dev->rx_msg_type & 0x0F] : ctrlmsg2string[dev->rx_msg_type & 0x0F]);
#endif


	// Check for Soft Reset.
	if ((dev->rx_msg_data_len == 0) &&
		(dev->rx_msg_type == CTRL_MSG_TYPE_SOFT_RESET)
#ifdef ENABLE_USB_PD_REV3_SUPPORT
		 && !dev->rx_msg_extended)
#else
		)
#endif
	{
		CRIT("Soft Reset Rx'd. SOP=%u\n", sop);

		usb_pd_prl_handle_soft_reset(dev, sop);

		usb_pd_pe_notify(port, PRL_ALERT_MSG_RECEIVED);
	}
	else
	{
		msg_id = USB_PD_HDR_GET_MSG_ID(hdr);
		INFO("MsgID: %u\n", msg_id);

		// Verify this is not a retry msg. 
		if (msg_id != dev->stored_msg_id[sop])
		{
			// Store message ID.
			dev->stored_msg_id[sop] = msg_id;

			tcpm_read_message(port, dev->msg_buf, dev->rx_msg_data_len);

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
			if (dev->rx_msg_extended)
			{
				// When Extended Messages are supported, chunking shall be supported.
				usb_pd_prl_chunked_msg_router(dev);
			}
			else /* Non-extended msg Rx'd */
			{
				usb_pd_pe_notify(port, PRL_ALERT_MSG_RECEIVED);

				if (dev->rch_state == RCH_WAITING_CHUNK)
				{
					prl_set_rch_state(dev, RCH_WAIT_FOR_MSG);
				}
				else if (dev->tch_state == TCH_WAIT_CHUNK_REQ)
				{
					prl_set_tch_state(dev, TCH_WAIT_FOR_MSG_REQ);
				}
			}

#else /* USB-PD r3.0 support not enabled */

			usb_pd_pe_notify(port, PRL_ALERT_MSG_RECEIVED);
#endif
		}
		else
		{
			CRIT("MsgID: %u matches stored[%u]!\n", msg_id, sop);
		}
	}

	return;
}


static const tcpm_callbacks_t tcpm_callbacks = {
	.conn_state_change_cbk = usb_pd_pe_connection_state_change_handler,
	.current_change_cbk = usb_pd_pe_current_change_handler,
	.pd_hard_reset_cbk = usb_pd_prl_hard_reset_alert_handler,
	.pd_receive_cbk = usb_pd_prl_receive_alert_handler,
	.pd_transmit_cbk = usb_pd_prl_transmit_alert_handler,
	.volt_alarm_cbk = usb_pd_pe_voltage_alarm_handler,
};

void usb_pd_prl_init(void)
{
	tcpm_register_callbacks(&tcpm_callbacks);

	return;
}
