#include "smbus.h"
#include "pb.h"
#include "bq25713_reg.h"

extern SMBus sSMBMaster[];

#define SMBUS_TIMEOUT   10000

typedef enum
{
    SMBUS_MASTER0 = 0,
    TOTAL_NUM_SMBUS_MASTERS
} smbus_interface_t;

struct bq25713 {
    smbus_interface_t  intf;
    uint8_t            slave_addr;

};

struct bq25713_reg {
    uint8_t addr;
    uint16_t val;
};

struct bq25713_reg bq25713_reg_val[30];

const struct bq25713 s_bq = {
     .intf = SMBUS_MASTER0,
     .slave_addr = 0xD6 >> 1,
};

static const struct bq25713 *bq = &s_bq;

enum {
    BQ_I2C_SUCCESS = 0,
    BQ_I2C_PARA_ERROR = -1,
    BQ_I2C_ERROR = -2,
};

static int8_t _bq25713_read(const struct bq25713 *bq, uint8_t reg,
                         uint8_t *buf,
                         uint8_t len)
{
    int8_t ret;
    SMBus *smbus_intf = &sSMBMaster[bq->intf];

    ret = SMBus_masterReadByteWord32(smbus_intf,
                                     bq->slave_addr,
                                     reg,
                                     buf,
                                     len);

    if (ret == SMBUS_RET_OK)
    {
        if ((SMBus_masterWaitUntilDone(smbus_intf, SMBUS_TIMEOUT) != SMBUS_RET_OK) ||
            (SMBus_getState(smbus_intf) != SMBus_State_OK))
        {
            SMBus_masterReset(smbus_intf); // Force a reset, so we can retry later
            return BQ_I2C_ERROR;
        }
        return BQ_I2C_SUCCESS;

    }


    return BQ_I2C_ERROR;

}

static int8_t bq25713_read_reg(const struct bq25713 *bq, uint8_t reg, uint16_t *data)
{
    uint8_t tmp_buf[2];
    int ret;
    ret = _bq25713_read(bq, reg, tmp_buf, 2);
    if (!ret)
        *data = tmp_buf[0] | tmp_buf[1] << 8;

    return ret;
}

static int8_t _bq25713_write(const struct bq25713 *bq,
                          uint8_t reg,
                          uint8_t *buf,
                          uint8_t len)
{
    int8_t ret;
    SMBus *smbus_intf = &sSMBMaster[bq->intf];

    ret =  SMBus_masterWriteByteWord32(smbus_intf,
                                       bq->slave_addr,
                                       reg,
                                       buf,
                                       len);

    if (ret == SMBUS_RET_OK)
    {
        if ((SMBus_masterWaitUntilDone(smbus_intf, SMBUS_TIMEOUT) != SMBUS_RET_OK) ||
            (SMBus_getState(smbus_intf) != SMBus_State_OK))
        {
            SMBus_masterReset(smbus_intf); // Force a reset, so we can retry later
            return BQ_I2C_ERROR;
        }
        return BQ_I2C_SUCCESS;
    }

    return BQ_I2C_ERROR;
}

static int8_t bq25713_write_reg(const struct bq25713 *bq,
                   uint8_t reg,
                   uint16_t data)
{
    return _bq25713_write(bq, reg, (uint8_t *)&data, 2);
}


static int8_t bq25713_update_bits(const struct bq25713 *bq,
                                  uint8_t reg,
                                  uint16_t mask,
                                  uint16_t value
                                  )
{
    int8_t ret;
    uint16_t tmp;

    ret = bq25713_read_reg(bq, reg, &tmp);
    if (ret)
        return ret;
    tmp &= ~mask;
    tmp |= value & mask;

    ret = bq25713_write_reg(bq, reg, tmp);

    return ret;
}

static int bq25713_set_lwpwr_mode(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == true)
		reg_val = BQ25713_LWPWR_MODE_ENABLE;
	else
		reg_val = BQ25713_LWPWR_MODE_DISABLE;

	reg_val <<= BQ25713_LWPWR_MODE_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_LWPWR_MODE_CTRL_MASK, reg_val);

	return ret;
}


static int bq25713_set_watchdog_timer(const struct bq25713 *bq, int time)
{
	int ret;
	uint16_t reg_val;

	if (time == 0)
		reg_val = BQ25713_WDTMR_DISABLE;
	else if (time == 5)
		reg_val = BQ25713_WDTMR_5S;
	else if (time == 88)
		reg_val = BQ25713_WDTMR_88S;
	else
		reg_val = BQ25713_WDTMR_175S;

	reg_val <<= BQ25713_WDTMR_ADJ_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_WDTMR_ADJ_MASK, reg_val);

	return ret;
}

#if 0
static int bq25713_set_idpm_auto_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == true)
		reg_val = BQ25713_IDPM_AUTO_ENABLE;
	else
		reg_val = BQ25713_IDPM_AUTO_DISABLE;

	reg_val <<= BQ25713_IDPM_AUTO_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_IDPM_AUTO_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_otg_on_chrgok(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_OTG_ON_CHRGOK_DISABLE;
	else
		reg_val = BQ25713_OTG_ON_CHRGOK_ENABLE;

	reg_val <<= BQ25713_OTG_ON_CHRGOK_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_OTG_ON_CHRGOK_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_pwm_freq(const struct bq25713 *bq, int freq)
{
	int ret;
	uint16_t reg_val;

	if (freq == 1200)
		reg_val = BQ25713_PWM_FREQ_1200KHZ;
	else
		reg_val = BQ25713_PWM_FREQ_800KHZ;

	reg_val <<= BQ25713_PWM_FREQ_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_PWM_FREQ_MASK, reg_val);

	return ret;
}

static int bq25713_set_learn_mode(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_LEARN_MODE_DISABLE;
	else
		reg_val = BQ25713_LEARN_MODE_ENABLE;

	reg_val <<= BQ25713_LEARN_MODE_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_LEARN_MODE_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_iadpt_gain(const struct bq25713 *bq, int gain)
{
	int ret;
	uint16_t reg_val;

	if (gain == 20)
		reg_val = BQ25713_IADPT_GAIN_20X;
	else
		reg_val = BQ25713_IADPT_GAIN_40X;

	reg_val <<= BQ25713_IADPT_GAIN_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_IADPT_GAIN_MASK, reg_val);

	return ret;
}
#endif

static int bq25713_set_ibat_gain(const struct bq25713 *bq, int gain)
{
	int ret;
	uint16_t reg_val;

	if (gain == 8)
		reg_val = BQ25713_IBAT_GAIN_8X;
	else
		reg_val = BQ25713_IBAT_GAIN_16X;

	reg_val <<= BQ25713_IBAT_GAIN_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_IBAT_GAIN_MASK, reg_val);

	return ret;
}

#if 0
static int bq25713_set_ldo_mode(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_LDO_MODE_DISABLE;
	else
		reg_val = BQ25713_LDO_MODE_ENABLE;

	reg_val <<= BQ25713_LDO_MODE_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_LDO_MODE_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_idpm_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_IDPM_DISABLE;
	else
		reg_val = BQ25713_IDPM_ENABLE;

	reg_val <<= BQ25713_IDPM_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_IDPM_CTRL_MASK, reg_val);

	return ret;
}
#endif

static int bq25713_charge_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == true)
		reg_val = BQ25713_CHRG_ENABLE;
	else
		reg_val = BQ25713_CHRG_DISABLE;

	reg_val <<= BQ25713_CHRG_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION0,
				BQ25713_CHRG_CTRL_MASK, reg_val);

	return ret;
}

#if 0
static int bq25713_set_psys_output(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_PSYS_DISABLE;
	else
		reg_val = BQ25713_PSYS_ENABLE;

	reg_val <<= BQ25713_PSYS_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION1,
				BQ25713_PSYS_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_power_path_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == true)
		reg_val = BQ25713_POWER_PATH_ON;
	else
		reg_val = BQ25713_POWER_PATH_OFF;

	reg_val <<= BQ25713_POWER_PATH_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION1,
				BQ25713_POWER_PATH_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_acoc_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_ACOC_DISABLE;
	else
		reg_val = BQ25713_ACOC_ENABLE;

	reg_val <<= BQ25713_ACOC_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION2,
				BQ25713_ACOC_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_acoc_volt_pct(const struct bq25713 *bq, int volt_pct)
{
	int ret;
	uint16_t reg_val;

	if (volt_pct == 125)
		reg_val = BQ25713_ACOC_VTH_133PCT;
	else
		reg_val = BQ25713_ACOC_VTH_200PCT;

	reg_val <<= BQ25713_ACOC_VTH_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION2,
				BQ25713_ACOC_VTH_MASK, reg_val);

	return ret;
}

static int bq25713_set_batoc_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_BATOC_DISABLE;
	else
		reg_val = BQ25713_BATOC_ENABLE;

	reg_val <<= BQ25713_BATOC_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION2,
				BQ25713_BATOC_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_batoc_volt_pct(const struct bq25713 *bq, int volt_pct)
{
	int ret;
	uint16_t reg_val;

	if (volt_pct == 150)
		reg_val = BQ25713_BATOC_VTH_150PCT;
	else
		reg_val = BQ25713_BATOC_VTH_200PCT;

	reg_val <<= BQ25713_BATOC_VTH_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION2,
				BQ25713_BATOC_VTH_MASK, reg_val);

	return ret;
}

static int bq25713_set_hiz_mode(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_HIZ_MODE_DISABLE;
	else
		reg_val = BQ25713_HIZ_MODE_ENABLE;

	reg_val <<= BQ25713_HIZ_MODE_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_HIZ_MODE_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_reset_reg(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_REG_RESET_IDLE;
	else
		reg_val = BQ25713_REG_RESET;

	reg_val <<= BQ25713_REG_RESET_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_REG_RESET_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_reset_vindpm(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_VINDPM_RESET_IDLE;
	else
		reg_val = BQ25713_VINDPM_RESET;

	reg_val <<= BQ25713_VINDPM_RESET_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_VINDPM_RESET_CTRL_MASK, reg_val);

	return ret;
}

#endif

static int bq25713_set_otg_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_OTG_DISABLE;
	else
		reg_val = BQ25713_OTG_ENABLE;

	reg_val <<= BQ25713_OTG_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_OTG_CTRL_MASK, reg_val);

	return ret;
}

#if 0
static int bq25713_set_ico_enable(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_ICO_DISABLE;
	else
		reg_val = BQ25713_ICO_ENABLE;

	reg_val <<= BQ25713_ICO_MODE_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_ICO_MODE_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_otg_vap_mode(const struct bq25713 *bq, bool otg_mode)
{
	int ret;
	uint16_t reg_val;

	if (otg_mode == false)
		reg_val = BQ25713_OTG_VAP_MODE_VAP;
	else
		reg_val = BQ25713_OTG_VAP_MODE_OTG;

	reg_val <<= BQ25713_OTG_VAP_MODE_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_OTG_VAP_MODE_MASK, reg_val);

	return ret;
}

static int bq25713_set_indcutor_clamp_current(const struct bq25713 *bq, int clamp_curr)
{
	int ret;
	uint16_t reg_val;

	if (clamp_curr == 6)
		reg_val = BQ25713_INDUCTOR_CLAMP_CURRENT_6A;
	else if (clamp_curr == 8)
		reg_val = BQ25713_INDUCTOR_CLAMP_CURRENT_8A;
	else if (clamp_curr == 10)
		reg_val = BQ25713_INDUCTOR_CLAMP_CURRENT_10A;
	else
		reg_val = BQ25713_INDUCTOR_CLAMP_CURRENT_15A;

	reg_val <<= BQ25713_INDUCTOR_CLAMP_CURRENT_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_INDUCTOR_CLAMP_CURRENT_MASK, reg_val);

	return ret;
}

static int bq25713_set_otg_range_low(const struct bq25713 *bq, bool range_low)
{
	int ret;
	uint16_t reg_val;

	if (range_low == false)
		reg_val = BQ25713_OTG_RANGE_4P28V_20P8V;
	else
		reg_val = BQ25713_OTG_RANGE_3V_19P52V;

	reg_val <<= BQ25713_OTG_RANGE_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_OTG_RANGE_MASK, reg_val);

	return ret;
}

static int bq25713_set_batfet_in_hiz(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == true)
		reg_val = BQ25713_HIZ_BATFET_ON;
	else
		reg_val = BQ25713_HIZ_BATFET_OFF;

	reg_val <<= BQ25713_HIZ_BATFET_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION3,
				BQ25713_HIZ_BATFET_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_get_ilim2_vth(const struct bq25713 *bq, int *vth)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_PROHOT_OPTION0, &reg_val);
	if (ret)
		return ret;

	*vth = reg_val & BQ25713_ILIM2_VTH_MASK;
	*vth >>= BQ25713_ILIM2_VTH_SHIFT;
	*vth *= BQ25713_ILIM2_VTH_LSB;
	*vth += BQ25713_ILIM2_VTH_BASE;

	return 0;
}

static int bq25713_set_icrit_deglitch(const struct bq25713 *bq, int deg)
{
	int ret;
	uint16_t reg_val;

	if (deg == 15)
		reg_val = BQ25713_ICRIT_DEG_15US;
	else if (deg == 100)
		reg_val = BQ25713_ICRIT_DEG_100US;
	else if (deg == 400)
		reg_val = BQ25713_ICRIT_DEG_400US;
	else
		reg_val = BQ25713_ICRIT_DEG_800US;

	reg_val <<= BQ25713_ICRIT_DEG_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_PROHOT_OPTION0,
				BQ25713_ICRIT_DEG_MASK, reg_val);

	return ret;
}

static int bq25713_set_prochot_vdpm_pct(const struct bq25713 *bq, int deg)
{
	int ret;
	uint16_t reg_val;

	if (deg == 80)
		reg_val = BQ25713_PROCHOT_VDPM_80PCT;
	else
		reg_val = BQ25713_PROCHOT_VDPM_90PCT;

	reg_val <<= BQ25713_PROCHOT_VDPM_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_PROHOT_OPTION0,
				BQ25713_PROCHOT_VDPM_MASK, reg_val);

	return ret;
}

static int bq25713_set_vsys_th(const struct bq25713 *bq, int th)
{
	int ret;
	uint16_t reg_val;

	if (th == 5750)
		reg_val = BQ25713_VSYS_VTH_5P75V_2P85V;
	else if (th == 6000)
		reg_val = BQ25713_VSYS_VTH_6P0V_3P1V;
	else if (th == 6250)
		reg_val = BQ25713_VSYS_VTH_6P25V_3P35V;
	else
		reg_val = BQ25713_VSYS_VTH_6P5V_3P6V;

	reg_val <<= BQ25713_VSYS_VTH_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_PROHOT_OPTION0,
				BQ25713_VSYS_VTH_MASK, reg_val);

	return ret;
}

static int bq25713_set_dsg_th(const struct bq25713 *bq, int vth)
{
	int ret;
	uint16_t reg_val;

	if (vth < BQ25713_DSG_VTH_BASE)
		vth = BQ25713_DSG_VTH_BASE;

	vth -= BQ25713_DSG_VTH_BASE;
	reg_val = vth / BQ25713_DSG_VTH_LSB;
	reg_val <<= BQ25713_DSG_VTH_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_PROHOT_OPTION1,
				BQ25713_DSG_VTH_MASK, reg_val);

	return ret;
}
#endif


static int bq25713_set_adc_scan_mode(const struct bq25713 *bq, bool oneshot)
{
	int ret;
	uint16_t reg_val;

	if (oneshot == true)
		reg_val = BQ25713_ADC_CONV_ONESHOT;
	else
		reg_val = BQ25713_ADC_CONV_CONT;

	reg_val <<= BQ25713_ADC_CONV_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_ADC_OPTION,
				BQ25713_ADC_CONV_MASK, reg_val);

	return ret;
}

static int bq25713_adc_start(const struct bq25713 *bq, bool start)
{
	int ret;
	uint16_t reg_val;

	if (start == false)
		reg_val = BQ25713_ADC_START_IDLE;
	else
		reg_val = BQ25713_ADC_START;

	reg_val <<= BQ25713_ADC_START_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_ADC_OPTION,
				BQ25713_ADC_START_CTRL_MASK, reg_val);

	return ret;
}
#if 0
static int bq25713_set_adc_full_scale(const struct bq25713 *bq, int scale)
{
	int ret;
	uint16_t reg_val;

	if (scale == 2040)
		reg_val = BQ25713_ADC_FULLSCALE_2P04V;
	else
		reg_val = BQ25713_ADC_FULLSCALE_3P06V;

	reg_val <<= BQ25713_ADC_FULLSCALE_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_ADC_OPTION,
				BQ25713_ADC_FULLSCALE_MASK, reg_val);

	return ret;
}
#endif

static int bq25713_set_adc_scan_channel(const struct bq25713 *bq, uint16_t mask)
{
    int ret;
    uint16_t reg_val;

    reg_val = mask;

    ret = bq25713_update_bits(bq, BQ25713_REG_ADC_OPTION,
                0x00FF, reg_val);

    return ret;
}

#if 0
static int bq25713_set_prochot_extension(const struct bq25713 *bq, bool enable)
{
	int ret;
	uint16_t reg_val;

	if (enable == false)
		reg_val = BQ25713_PROCHOT_EXT_DISABLE;
	else
		reg_val = BQ25713_PROCHOT_EXT_ENABLE;

	reg_val <<= BQ25713_PROCHOT_EXT_CTRL_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_PROCHOT_STATUS,
				BQ25713_PROCHOT_EXT_CTRL_MASK, reg_val);

	return ret;
}

static int bq25713_set_prochot_pulse_width(const struct bq25713 *bq, long width_us)
{
	int ret;
	uint16_t reg_val;


	reg_val <<= BQ25713_PROCHOT_PULSE_WIDTH_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_PROCHOT_STATUS,
				BQ25713_PROCHOT_PULSE_WIDTH_MASK, reg_val);

	return ret;
}
#endif

static int bq25713_set_charge_current(const struct bq25713 *bq, int curr)
{
	int ret;
	uint16_t reg_val;

	if (curr < BQ25713_CHARGE_CURRENT_BASE)
		curr = BQ25713_CHARGE_CURRENT_BASE;

	curr -= BQ25713_CHARGE_CURRENT_BASE;
	reg_val = curr / BQ25713_CHARGE_CURRENT_LSB;
	reg_val <<= BQ25713_CHARGE_CURRENT_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_CURRENT,
				BQ25713_CHARGE_CURRENT_MASK, reg_val);

	return ret;
}

static int bq25713_set_charge_voltage(const struct bq25713 *bq, int volt)
{
	int ret;
	uint16_t reg_val;

	if (volt < BQ25713_MAX_CHG_VOLT_BASE)
		volt = BQ25713_MAX_CHG_VOLT_BASE;

	volt -= BQ25713_MAX_CHG_VOLT_BASE;
	reg_val = volt / BQ25713_MAX_CHG_VOLT_LSB;
	reg_val <<= BQ25713_MAX_CHG_VOLT_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_MAX_CHG_VOLT,
				BQ25713_MAX_CHG_VOLT_MASK, reg_val);

	return ret;
}

#if 0
static int bq25713_set_min_sys_volt(const struct bq25713 *bq, int volt)
{
	int ret;
	uint16_t reg_val;

	if (volt < BQ25713_MIN_SYS_VOLT_BASE)
		volt = BQ25713_MIN_SYS_VOLT_BASE;

	volt -= BQ25713_MIN_SYS_VOLT_BASE;
	reg_val = volt / BQ25713_MIN_SYS_VOLT_LSB;
	reg_val <<= BQ25713_MIN_SYS_VOLT_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_MIN_SYS_VOLT,
				BQ25713_MIN_SYS_VOLT_MASK, reg_val);

	return ret;
}
#endif

static int bq25713_set_input_current_limit(const struct bq25713 *bq, int curr)
{
	int ret;
	uint16_t reg_val;

	if (curr < BQ25713_INLIM_HOST_BASE)
		curr = BQ25713_INLIM_HOST_BASE;

	curr -= BQ25713_INLIM_HOST_BASE;
	reg_val = curr / BQ25713_INLIM_HOST_LSB;
	reg_val <<= BQ25713_INLIM_HOST_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_IIN_HOST,
				BQ25713_INLIM_HOST_MASK, reg_val);

	return ret;
}

#if 0
static int bq25713_get_real_input_current_limit(const struct bq25713 *bq, int curr)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_IIN_DPM, &reg_val);
	if (ret)
		return ret;

	curr = reg_val & BQ25713_IIN_DPM_MASK;
	curr >>= BQ25713_IIN_DPM_SHIFT;
	curr *= BQ25713_IIN_DPM_LSB;
	curr += BQ25713_IIN_DPM_BASE;

	return 0;
}
#endif

static int bq25713_set_input_voltage_limit(const struct bq25713 *bq, int volt)
{
	int ret;
	uint16_t reg_val;

	if (volt < BQ25713_VIN_DPM_BASE)
		volt = BQ25713_VIN_DPM_BASE;

	volt -= BQ25713_VIN_DPM_BASE;
	reg_val = volt / BQ25713_VIN_DPM_LSB;
	reg_val <<= BQ25713_VIN_DPM_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_VIN_DPM,
				BQ25713_VIN_DPM_MASK, reg_val);

	return ret;
}

static int bq25713_set_otg_voltage(const struct bq25713 *bq, int volt)
{
	int ret;
	uint16_t reg_val;
	int base = 0;

	ret = bq25713_read_reg(bq, BQ25713_REG_CHARGE_OPTION3, &reg_val);
	if (!ret) {
	    if (reg_val & BQ25713_OTG_RANGE_MASK)
	        base = BQ25713_OTG_VOLT_BASE;
	    else
	        base = BQ25713_OTG_VOLT_BASE_1280;
	}

	if (volt < base)
		volt = base;

	volt -= base;
	reg_val = (volt + (BQ25713_OTG_VOLT_LSB >> 1)) / BQ25713_OTG_VOLT_LSB;
	reg_val <<= BQ25713_OTG_VOLT_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_OTG_VOLT,
				BQ25713_OTG_VOLT_MASK, reg_val);

	return ret;
}

static int bq25713_set_otg_current(const struct bq25713 *bq, int curr)
{
	int ret;
	uint16_t reg_val;

	if (curr < BQ25713_OTG_CURRENT_BASE)
		curr = BQ25713_OTG_CURRENT_BASE;

	curr -= BQ25713_OTG_CURRENT_BASE;
	reg_val = (curr + (BQ25713_OTG_CURRENT_LSB >> 1)) / BQ25713_OTG_CURRENT_LSB;
	reg_val <<= BQ25713_OTG_CURRENT_SHIFT;

	ret = bq25713_update_bits(bq, BQ25713_REG_OTG_CURRENT,
				BQ25713_OTG_CURRENT_MASK, reg_val);

	return ret;
}

static int bq25713_set_adc_current_sel(const struct bq25713 *bq, bool as_chg)
{
    int ret;
    uint16_t reg_val;

    if (as_chg)
        reg_val = BQ25713_IBAT_PIN_CHG;
    else
        reg_val = BQ25713_IBAT_PIN_DSG;

    reg_val <<= BQ25713_IBAT_PIN_CTRL_SHIFT;

    ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION2,
                BQ25713_IBAT_PIN_CTRL_MASK, reg_val);

    return ret;
}

static int bq25713_read_vbus_volt(const struct bq25713 *bq, int *volt)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_VBUS_PSYS, &reg_val);
	if (ret)
		return ret;

	*volt = reg_val & BQ25713_ADC_VBUS_VOLT_MASK;
	*volt >>= BQ25713_ADC_VBUS_VOLT_SHIFT;
	*volt *= BQ25713_ADC_VBUS_VOLT_LSB;
	*volt += BQ25713_ADC_VBUS_VOLT_BASE;

	return 0;
}

#if 0
static int bq25713_read_psys_volt(const struct bq25713 *bq, int *volt)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_VBUS_PSYS, &reg_val);
	if (ret)
		return ret;

	*volt = reg_val & BQ25713_ADC_PSYS_VOLT_MASK;
	*volt >>= BQ25713_ADC_PSYS_VOLT_SHIFT;
	*volt *= BQ25713_ADC_PSYS_VOLT_LSB;
	*volt += BQ25713_ADC_PSYS_VOLT_BASE;

	return 0;
}
#endif

static int bq25713_read_ibat_chg_current(const struct bq25713 *bq, int *curr)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_IBAT, &reg_val);
	if (ret)
		return ret;

	*curr = reg_val & BQ25713_ADC_CHG_CURRENT_MASK;
	*curr >>= BQ25713_ADC_CHG_CURRENT_SHIFT;
	*curr *= BQ25713_ADC_CHG_CURRENT_LSB;
	*curr += BQ25713_ADC_CHG_CURRENT_BASE;

	return 0;
}

#if 0

static int bq25713_read_ibat_dsg_current(const struct bq25713 *bq, int *curr)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_IBAT, &reg_val);
	if (ret)
		return ret;

	*curr = reg_val & BQ25713_ADC_DSG_CURRENT_MASK;
	*curr >>= BQ25713_ADC_DSG_CURRENT_SHIFT;
	*curr *= BQ25713_ADC_DSG_CURRENT_LSB;
	*curr += BQ25713_ADC_DSG_CURRENT_BASE;

	return 0;
}

static int bq25713_read_ibus_current(const struct bq25713 *bq, int *curr)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_IIN_CMPIN, &reg_val);
	if (ret)
		return ret;

	*curr = reg_val & BQ25713_ADC_INPUT_CURRENT_MASK;
	*curr >>= BQ25713_ADC_INPUT_CURRENT_SHIFT;
	*curr *= BQ25713_ADC_INPUT_CURRENT_LSB;
	*curr += BQ25713_ADC_INPUT_CURRENT_BASE;

	return 0;
}


static int bq25713_read_cmpin_volt(const struct bq25713 *bq, int *volt)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_IIN_CMPIN, &reg_val);
	if (ret)
		return ret;

	*volt = reg_val & BQ25713_ADC_CMPIN_VOLT_MASK;
	*volt >>= BQ25713_ADC_CMPIN_VOLT_SHIFT;
	*volt *= BQ25713_ADC_CMPIN_VOLT_LSB;
	*volt += BQ25713_ADC_CMPIN_VOLT_BASE;

	return 0;
}


static int bq25713_read_vsys_volt(const struct bq25713 *bq, int *volt)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_VSYS_VBAT, &reg_val);
	if (ret)
		return ret;

	*volt = reg_val & BQ25713_ADC_SYS_VOLT_MASK;
	*volt >>= BQ25713_ADC_SYS_VOLT_SHIFT;
	*volt *= BQ25713_ADC_SYS_VOLT_LSB;
	*volt += BQ25713_ADC_SYS_VOLT_BASE;

	return 0;
}

#endif

static int bq25713_read_vbat_volt(const struct bq25713 *bq, int *volt)
{
	int ret;
	uint16_t reg_val;

	ret = bq25713_read_reg(bq, BQ25713_REG_ADC_VSYS_VBAT, &reg_val);
	if (ret)
		return ret;

	*volt = reg_val & BQ25713_ADC_BAT_VOLT_MASK;
	*volt >>= BQ25713_ADC_BAT_VOLT_SHIFT;
	*volt *= BQ25713_ADC_BAT_VOLT_LSB;
	*volt += BQ25713_ADC_BAT_VOLT_BASE;

	return 0;
}


static int bq25713_get_charging_status(const struct bq25713 *bq, bool *is_charging)
{
    int ret;
    uint16_t reg_val;

    ret = bq25713_read_reg(bq, BQ25713_REG_CHARGER_STATUS, &reg_val);
    if (ret)
        *is_charging = false;
    else
        *is_charging = !!(reg_val & (BQ25713_FAST_CHG_STAT_MASK | BQ25713_PRE_CHG_STAT_MASK));

    return ret;
}

static int bq25713_get_ac_present(const struct bq25713 *bq, bool *present)
{
    int ret;
    uint16_t reg_val;

    ret = bq25713_read_reg(bq, BQ25713_REG_CHARGER_STATUS, &reg_val);
    if (ret)
        return ret;

    *present = !!(reg_val & BQ25713_AC_STAT_MASK );

    return 0;
}


static int bq25713_get_fault_status(const struct bq25713 *bq, uint8_t* fault)
{
    int ret;
    uint16_t reg_val;

    ret = bq25713_read_reg(bq, BQ25713_REG_CHARGER_STATUS, &reg_val);
    if (ret)
        *fault = 0;
    else
        *fault = reg_val & 0xFF;

    return ret;
}

static int bq25713_get_otg_mode(const struct bq25713 *bq, bool *otg_active)
{
    int ret;
    uint16_t reg_val;

    ret = bq25713_read_reg(bq, BQ25713_REG_CHARGER_STATUS, &reg_val);
    if (ret)
        *otg_active = false;
    else
        *otg_active = !!(reg_val & BQ25713_OTG_STAT_MASK);

    return ret;
}

#ifdef USE_MSP430_ADC_IBAT_IBUS

static int bq25713_get_ibus_ibat_gain(const struct bq25713 *bq, uint8_t *bus, uint8_t *bat)
{
    int ret;
    uint16_t reg_val;

    ret = bq25713_read_reg(bq, BQ25713_REG_CHARGE_OPTION0, &reg_val);
    if (ret)
        return ret;

    if (reg_val & BQ25713_IADPT_GAIN_MASK)
        *bus = 40;
    else
        *bus = 20;

    if (reg_val & BQ25713_IBAT_GAIN_MASK)
        *bat = 16;
    else
        *bat = 8;


    return 0;
}


static int bq25713_set_ibat_output_buffer(const struct bq25713 *bq, bool enable)
{
    int ret;
    uint16_t reg_val;

    if(enable)
        reg_val = BQ25713_IBAT_OUTPUT_BUF_ENABLE;
    else
        reg_val = BQ25713_IBAT_OUTPUT_BUF_DISABLE;

    reg_val <<= BQ25713_IBAT_OUTPUT_BUF_CTRL_SHIFT;

    ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION1,
                          BQ25713_IBAT_OUTPUT_BUF_CTRL_MASK, reg_val);
    return ret;
}

static int bq25713_set_ibat_output_mode(const struct bq25713 *bq, bool asChg)
{
    int ret;
    uint16_t reg_val;

    if(asChg)
        reg_val = BQ25713_IBAT_PIN_CHG;
    else
        reg_val = BQ25713_IBAT_PIN_DSG;

    reg_val <<= BQ25713_IBAT_PIN_CTRL_SHIFT;

    ret = bq25713_update_bits(bq, BQ25713_REG_CHARGE_OPTION2,
                              BQ25713_IBAT_PIN_CTRL_MASK, reg_val);
    return ret;

}
#endif

int bq2571x_enable_charge()
{
    return bq25713_charge_enable(bq, true);
}

int bq2571x_disable_charge()
{
    return bq25713_charge_enable(bq, false);
}

int bq2571x_set_charge_current(int curr)
{
    return bq25713_set_charge_current(bq, curr);
}

int bq2571x_set_charge_voltage(int volt)
{
    return bq25713_set_charge_voltage(bq, volt);
}

int bq2571x_set_input_voltage_limit(int volt)
{
    return bq25713_set_input_voltage_limit(bq, volt);
}

int bq2571x_set_input_current_limit(int curr)
{
    return bq25713_set_input_current_limit(bq, curr);
}

int bq2571x_enable_otg()
{
    return bq25713_set_otg_enable(bq, true);
}

int bq2571x_disable_otg()
{
    return bq25713_set_otg_enable(bq, false);
}

int bq2571x_set_otg_voltage(int volt)
{
    return bq25713_set_otg_voltage(bq, volt);
}

int bq2571x_set_otg_current(int curr)
{
    curr += 300; //calibration
    return bq25713_set_otg_current(bq, curr);
}

int bq2571x_read_vbat_volt(int *volt)
{
    return bq25713_read_vbat_volt(bq, volt);
}

int bq2571x_read_vbus_volt(int *volt)
{
    return bq25713_read_vbus_volt(bq, volt);
}

int bq2571x_read_chg_current(int *curr)
{
    return bq25713_read_ibat_chg_current(bq, curr);
}

int bq2571x_get_charging_status(bool *is_charging)
{
    return bq25713_get_charging_status(bq, is_charging);
}

int bq2571x_get_ac_present(bool *pres)
{
    return bq25713_get_ac_present(bq, pres);

}

int bq2571x_get_otg_mode(bool *otg_active)
{
    return bq25713_get_otg_mode(bq, otg_active);
}

int bq2571x_get_fault_status(uint8_t *fault)
{
    return bq25713_get_fault_status(bq, fault);
}

int bq2571x_set_lwpwr_mode(bool enable)
{
    return bq25713_set_lwpwr_mode(bq, enable);
}

int bq2571x_set_adc_current_sel(bool as_chg)
{
    return bq25713_set_adc_current_sel(bq, as_chg);
}

#ifdef USE_MSP430_ADC_IBAT_IBUS
int bq2571x_get_ibus_ibat_gain(uint8_t *bus_gain, uint8_t *bat_gain)
{
    return bq25713_get_ibus_ibat_gain(bq, bus_gain, bat_gain);
}
#endif


const uint8_t regs[]= {0x00,0x02,0x04,0x06,0x08,0x0A,0x0C,0x0E,0x20,0x22,0x24,0x26,0x28,0x2A,0x2C,0x30,0x32,0x34,0x36,0x38,0x3A};

void bq2571x_reg_dump()
{
   uint8_t i = 0;
   uint16_t reg_val;
   int ret;

   for (i = 0; i < sizeof(regs); i++){
       ret = bq25713_read_reg(bq, regs[i], &reg_val);
       bq25713_reg_val[i].addr = regs[i];
       if (!ret)
           bq25713_reg_val[i].val = reg_val;
       else
           bq25713_reg_val[i].val = 0xEE;
   }
}


int bq2571x_init()
{

    bq25713_set_watchdog_timer(bq,0);

    bq25713_set_ibat_gain(bq,16);

#ifdef USE_MSP430_ADC_IBAT_IBUS
    bq25713_set_ibat_output_buffer(bq, true);
    bq25713_set_ibat_output_mode(bq, true);
#endif

#ifndef USE_MSP430_ADC_IBAT_IBUS
    bq25713_set_lwpwr_mode(bq, false);
    bq25713_set_adc_scan_mode(bq, false);
    bq25713_set_adc_scan_channel(bq, 0xFF/*BQ25713_ADC_VBAT_CTRL_MASK | BQ25713_ADC_VSYS_CTRL_MASK | BQ25713_ADC_CHG_CTRL_MASK | BQ25713_ADC_DSG_CTRL_MASK*/);
    bq25713_adc_start(bq, true);
#endif
    return 0;
}
