/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "tcpm_hal.h"
#include "driverlib.h"
#include "tusb422_common.h"
#include <stdint.h>

unsigned int g_alert_event = 0;

//
// Local functions
//

extern bool tcpm_alert_event(unsigned int port);
extern bool HAL_I2C_Event(void);
extern void usb_pd_pe_hpd_in_event(unsigned int port, uint8_t val);

//*****************************************************************************
//
//! Clock Initialization
//!
//! This demo sets SMCLK/MCLK = DCO @ MCLK_MHZ (defined in project settings)
//!                ACLK = VLO
//!
//! \return None
//
// *****************************************************************************
static void mcu_clock_init(void)
{
    // MSP430 USB requires a Vcore setting of at least 2.  2 is high enough
	// for 20MHz MCLK.
    PMM_setVCore(PMM_CORE_LEVEL_2);

    // Assign the REFO (32768Hz) as the FLL reference clock
	UCS_initClockSignal(UCS_FLLREF, UCS_REFOCLK_SELECT, UCS_CLOCK_DIVIDER_1);

	// Initialize the DCO for target MCLK and SMCLK freq.
    UCS_initFLLSettle(MCLK_MHZ*1000, MCLK_MHZ*1000000/32768);

	// Assign the REFO as the source for ACLK.
	UCS_initClockSignal(UCS_ACLK, UCS_REFOCLK_SELECT,UCS_CLOCK_DIVIDER_1);

	// Configure P5.2 for XT2IN.
	P5SEL |= GPIO_PIN2;

	// Initialize XT2 oscillator for 4MHz xtal.
	UCS_turnOnXT2(UCS_XT2_DRIVE_4MHZ_8MHZ);

    return;
}

//*****************************************************************************
//
//! MCU initialization
//!
//! Initializes the MSP430
//!  Sets GPIOs to default state
//!  Configures clocks
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_init(void)
{
    // Initialize the MSP430 Clocks
    mcu_clock_init();

    // Turn off Temp sensor to save power.
    REFCTL0 |= REFTCOFF;

    // Set I2C pins for SCL/SDA functionality.
    GPIO_setAsPeripheralModuleFunctionOutputPin(I2C_PORT, I2C_SDA_PIN + I2C_SCL_PIN);

    // DDC_EN. (high to enable)
    GPIO_setAsOutputPin(DDC_EN_PORT, DDC_EN_PIN);
    GPIO_setOutputHighOnPin(DDC_EN_PORT, DDC_EN_PIN);

    // Configure input pins for interrupts.
    // P6.1 is ALERTN.
    GPIO_setAsInputPinWithPullUpResistor(ALERTN_PORT, ALERTN_PIN);
    GPIO_selectInterruptEdge(ALERTN_PORT, ALERTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);

	if (GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN))
	{
        GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
	}
    else
    {
        GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_LOW_TO_HIGH_TRANSITION);
    }

    // Clear all PxIFGs to avoid erroneous port interrupts.
    GPIO_clearInterrupt(ALERTN_PORT, ALERTN_PIN);
    GPIO_clearInterrupt(HPD_IN_PORT, HPD_IN_PIN);

    // Enable port interrupts.
    GPIO_enableInterrupt(ALERTN_PORT, ALERTN_PIN);
    GPIO_enableInterrupt(HPD_IN_PORT, HPD_IN_PIN);

    // Enable WDT.
//    WDT_A_start(WDT_A_BASE);

    return;
}


//*****************************************************************************
//
//! Get Base Address of I2C used for SMBus
//!
//! The base address is used by smbuslib PHY layer for FR5xx eUSCI.
//! The simpler implementation from 2xx doesn't use base address.
//!
//! \param intf Interface number (0 for first interface)
//!
//! \return Base address of I2C module, NULL if no base address used
//
// *****************************************************************************
uint32_t tcpm_hal_get_i2c_base_addr(uint8_t intf)
{
    // Note: Only one interface is supported in this HAL
    // FR5xx implementation uses Base address
    return(USCI_B0_BASE);
}

//*****************************************************************************
//
//! Turn on a LED
//!
//! \param led LED_RED, LED2, or LED_BOTH
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_led_high(led_sel_t sel)
{
    if (sel & LED_RED)
    {
        GPIO_setOutputHighOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_setOutputHighOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }

    return;
}

//*****************************************************************************
//
//! Turn off a LED
//!
//! \param led LED_RED, LED2, or LED_BOTH
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_led_low(led_sel_t sel)
{
    if (sel & LED_RED)
    {
        GPIO_setOutputLowOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_setOutputLowOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }

    return;
}


void tcpm_hal_led_toggle(led_sel_t sel)
{
    if (sel & LED_RED)
    {
        GPIO_toggleOutputOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_toggleOutputOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }

    return;
}

void tcpm_hal_set_mux_polarity(uint8_t val)
{
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_POL_PORT, MUX_POL_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_POL_PORT, MUX_POL_PIN);
	}
	return;
}

void tcpm_hal_set_mux_am_sel(uint8_t val)
{
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	}
	return;
}

void tcpm_hal_set_mux_enable(uint8_t val)
{
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_EN_PORT, MUX_EN_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_EN_PORT, MUX_EN_PIN);
	}
	return;
}


void tcpm_hal_hpd_out_control(uint8_t port, uint8_t val)
{
	if (val)
	{
		GPIO_setOutputHighOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
	}
	return;
}

uint8_t tcpm_hal_get_hpd_in(uint8_t port)
{
    uint8_t val;

	// Only a single Type-C port...
	val = GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN);

    // Make sure interrupt edge is correct.
    if (val)
    {
        GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    }
    else
    {
        GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_LOW_TO_HIGH_TRANSITION);
    }

    return val;
}

void tcpm_hal_vbus_enable(uint8_t port, vbus_select_t sel)
{
    if (port == 0)
    {
        if (sel == VBUS_SRC_5V)
        {
            // Disable high voltage.
            GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);

            // Enable 5V.
            GPIO_setOutputHighOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);

            // Enable SRC switch.
            GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        }
        else if (sel == VBUS_SRC_HI_VOLT)
        {
            // Disable 5V.
            GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);

            // Enable high voltage.
            GPIO_setOutputHighOnPin(HV_EN_PORT, HV_EN_PIN);

            // Enable SRC switch.
            GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        }
        else if (sel == VBUS_SNK)
        {
            // Enable SNK switch.
            GPIO_setAsOutputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
        }
    }
    return;
}


void tcpm_hal_vbus_disable(uint8_t port, vbus_select_t sel)
{
    if (port == 0)
    {
        if (sel == VBUS_SRC_5V)
        {
            // Disable SRC switch.
            GPIO_setAsInputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);

            // Disable 5V.
            GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
        }
        else if (sel == VBUS_SRC_HI_VOLT)
        {
            // Disable high voltage.
            GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);
        }
        else if (sel == VBUS_SNK)
        {
            // Disable SNK switch.
            GPIO_setAsInputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
        }
    }
    return;
}


bool tcpm_is_alert_asserted(unsigned int port)
{
    if (port == 0)
    {
        return (GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN) == GPIO_INPUT_PIN_LOW);
    }
    else
    {
        return false;
    }
}


//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! I2C Interrupt Service Routine
//!
//! ISR for I2C module
//!   This ISR generates an event which should be declared in application
//!
//! \return None
//
// *****************************************************************************
#pragma vector=USCI_B0_VECTOR
__interrupt void I2C_ISR(void)
{
    // This event should be declared in Application
    if (HAL_I2C_Event() == true)
    {
        LPM3_EXIT;
    }
}

//*****************************************************************************
//
//! Port1 Interrupt Service Routine
//!
//! ISR for P1
//!
//! \return None
//
// *****************************************************************************
#pragma vector=PORT1_VECTOR
__interrupt void P1_ISR(void)
{
    // Reading P1IV clears the highest priority interrupt.  
    switch (__even_in_range(P1IV,16))
    {
        case 0x00:  break;  // No interrupt pending
        case 0x02:  
#ifdef HDMI_DONGLE
            // Notify upper layers of HPD_IN change.
            usb_pd_pe_hpd_in_event(0, GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN));

            if (GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN))
            {
                GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
            }
            else
            {
                GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_LOW_TO_HIGH_TRANSITION);
            }

            // HPD will be debounced so no need to wake up here or set dummy alert.
//            // Set dummy alert event so PD state machine will get run.
//            g_alert_event |= (1 << 0);
//            LPM3_EXIT;
#endif
			break;  // P1.0 ISR
        case 0x04:  break;  // P1.1 ISR
        case 0x06:  break;  // P1.2 ISR
                             
		case 0x08:          // P1.3 ISR
            // Port 0 ALERT# event.
            g_alert_event |= (1 << 0);
            LPM3_EXIT;
            break;

        case 0x0A:          // P1.4 ISR
            break;

        case 0x0C:          // P1.5 ISR
            break;

        case 0x0E:  break;  // P1.6 ISR
        case 0x10:  break;  // P1.7 ISR
        default:  break;
    }

    return;
}



//*****************************************************************************
//
//! Low level initialization
//!
//! Disables the watchdog after reset
//!
//! \return 1: Initialize all variables
//
// *****************************************************************************
#ifdef __IAR_SYSTEMS_ICC__
int __low_level_init(void)
#elif defined (__TI_COMPILER_VERSION__)
int _system_pre_init(void)
#endif
{
    WDT_A_hold(WDT_A_BASE);             //Stop WDT
    return(1);
}

//*****************************************************************************
//
//! Dummy ISR
//!
//! Dummy ISR in case an interrupt is enabled unintentionally
//!
//! \return none
//
// *****************************************************************************
#pragma vector=RTC_VECTOR,TIMER0_A1_VECTOR,TIMER2_A0_VECTOR,TIMER2_A1_VECTOR, \
    USCI_A1_VECTOR,TIMER1_A1_VECTOR, \
    SYSNMI_VECTOR,  \
    TIMER1_A0_VECTOR, UNMI_VECTOR
__interrupt void Dummy_ISR(void)
{
    while (1);
}


// RTC_VECTOR,       
// PORT2_VECTOR,      
// TIMER2_A1_VECTOR, 
// TIMER2_A0_VECTOR,  
// USCI_B1_VECTOR,    
// USCI_A1_VECTOR,    
// PORT1_VECTOR,      
// TIMER1_A1_VECTOR,  
// TIMER1_A0_VECTOR,  
// DMA_VECTOR,     
// USB_UBM_VECTOR,    
// TIMER0_A1_VECTOR,  
// TIMER0_A0_VECTOR,  
// ADC12_VECTOR,      
// USCI_B0_VECTOR,    
// USCI_A0_VECTOR,    
// WDT_VECTOR,      
// TIMER0_B1_VECTOR,  
// TIMER0_B0_VECTOR,  
// COMP_B_VECTOR,     
// UNMI_VECTOR,       
// SYSNMI_VECTOR,     
// RESET_VECTOR,      
