/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "tcpm_hal.h"
#include "driverlib.h"
#include "smbus.h"
#include "tusb422_common.h"
#include <stdint.h>
#include "uart.h"  // for debug
#include "bq25713.h"
#include "pb.h"

unsigned int g_alert_event = 0;
extern power_bank_t pb;

//
// Local functions
//

extern bool tcpm_vbus_snk_fault_event(unsigned int port);
extern bool tcpm_vbus_src_fault_event(unsigned int port);
extern bool tcpm_alert_event(unsigned int port);
extern bool HAL_I2C_Event(void);
extern void usb_pd_pr_swap(unsigned int port);

void en_out_disable();

//*****************************************************************************
//
//! Clock Initialization
//!
//! This demo sets SMCLK/MCLK = DCO @ MCLK_MHZ (defined in project settings)
//!                ACLK = VLO
//!
//! \return None
//
// *****************************************************************************
static void mcu_clock_init(void)
{
    // Enable PJ[4:5] for crystal
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_PJ,
                                               GPIO_PIN4 | GPIO_PIN5,
                                               GPIO_PRIMARY_MODULE_FUNCTION);

#if (MCLK_MHZ == 1)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_initClockSignal(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_initClockSignal(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
#elif (MCLK_MHZ == 8)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_initClockSignal(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_initClockSignal(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#elif (MCLK_MHZ == 16)
    //set wait state to 1 in FRAM control register
    FRAMCtl_configureWaitStateControl(FRAMCTL_ACCESS_TIME_CYCLES_1);
    //Set DCO Frequency to 16MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_4);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_initClockSignal(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_initClockSignal(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);

#else
#error "Frequency not supported, change project settings"
#endif

    // Set LF external clock frequency to 32.768 KHz
//    CS_setExternalClockSource(32768, 0);

    // Set ACLK=LFXT
//    CS_initClockSignal(CS_ACLK, CS_LFXTCLK_SELECT, CS_CLOCK_DIVIDER_1);

    // Start XT1 with no time out and lowest drive strength.
//   CS_turnOnLFXT(CS_LFXT_DRIVE_0);

    // HF Crystal is not used.
    CS_turnOffHFXT();

    return;
}

//*****************************************************************************
//
//! MCU initialization
//!
//! Initializes the MSP430
//!  Sets GPIOs to default state
//!  Configures clocks
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_init(void)
{
    // Initialize the MSP430 Clocks
    mcu_clock_init();

    // Turn off Temp sensor
    REFCTL0 |= REFTCOFF;
    REFCTL0 &= ~REFON;

    // Set I2C pins for SCL/SDA functionality.
    GPIO_setAsPeripheralModuleFunctionOutputPin(I2C_PORT, I2C_SDA_PIN + I2C_SCL_PIN,
                                                GPIO_SECONDARY_MODULE_FUNCTION);

    // P1.2 is SNK VBUS_EN#. (open drain, active low)
    GPIO_setAsInputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
    GPIO_setOutputLowOnPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);

    // P3.0 is SRC VBUS_EN#. (open drain, active low)
    GPIO_setAsInputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
    GPIO_setOutputLowOnPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
#if 0
    // P3.5 is 5V_EN. (high to enable, low to disable)
    GPIO_setAsOutputPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
    GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
    
    // P2.2 is HV_EN. (high to enable, low to disable)
    GPIO_setAsOutputPin(HV_EN_PORT, HV_EN_PIN);
    GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);

    // P3.4 is VCONN_EN. (high to enable)
    GPIO_setAsOutputPin(VCONN_EN_PORT, VCONN_EN_PIN);
    GPIO_setOutputHighOnPin(VCONN_EN_PORT, VCONN_EN_PIN);
    
    // P4.6 is LED1 output.
    // P1.0 is LED2 output.
    GPIO_setOutputLowOnPin(RED_LED_PORT, RED_LED_PIN);
    GPIO_setAsOutputPin(RED_LED_PORT, RED_LED_PIN);
    GPIO_setOutputLowOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    GPIO_setAsOutputPin(GREEN_LED_PORT, GREEN_LED_PIN);

	// Mux control. (Set polarity, Alt-mode select, and enable to low -> Mux power down)
    GPIO_setOutputLowOnPin(MUX_POL_PORT, MUX_POL_PIN);
    GPIO_setAsOutputPin(MUX_POL_PORT, MUX_POL_PIN);
	GPIO_setOutputLowOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
    GPIO_setAsOutputPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	GPIO_setOutputLowOnPin(MUX_EN_PORT, MUX_EN_PIN);
    GPIO_setAsOutputPin(MUX_EN_PORT, MUX_EN_PIN);

	// Set HPD_OUT low.
	GPIO_setOutputLowOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
    GPIO_setAsOutputPin(HPD_OUT_PORT, HPD_OUT_PIN);
#endif

    // Set USB A pin
    GPIO_setOutputLowOnPin(USBA_EN_PORT, USBA_EN_PIN);
    GPIO_setAsOutputPin(USBA_EN_PORT, USBA_EN_PIN);
    // OTG Enable pin
    GPIO_setOutputLowOnPin(OTG_EN_PORT, OTG_EN_PIN);
    GPIO_setAsOutputPin(OTG_EN_PORT, OTG_EN_PIN);
    // Bus Discharge pin
    GPIO_setOutputLowOnPin(BUS_DSG_PORT, BUS_DSG_PIN);
    GPIO_setAsOutputPin(BUS_DSG_PORT, BUS_DSG_PIN);
    //CHRG ok pin
    GPIO_setAsInputPin(CHRG_OK_PORT, CHRG_OK_PIN);


    // Configure input pins for interrupts.
    // P1.3 is ALERTN.
    // P1.4 is SRC FAULT#.
    // P1.5 is SNK FAULT#.
    GPIO_setAsInputPinWithPullUpResistor(ALERTN_PORT, ALERTN_PIN);
#if 0
    GPIO_setAsInputPinWithPullUpResistor(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    GPIO_setAsInputPinWithPullUpResistor(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
#endif
    GPIO_setAsInputPinWithPullUpResistor(KEY_BTN_PORT, KEY_BTN_PIN);

    GPIO_selectInterruptEdge(ALERTN_PORT, ALERTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_selectInterruptEdge(KEY_BTN_PORT, KEY_BTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
#if 0
    GPIO_selectInterruptEdge(SRC_FAULTN_PORT, SRC_FAULTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_selectInterruptEdge(SNK_FAULTN_PORT, SNK_FAULTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
#endif
    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings.
    PMM_unlockLPM5();

    // Clear all PxIFGs to avoid erroneous port interrupts.
    GPIO_clearInterrupt(ALERTN_PORT, ALERTN_PIN);
    GPIO_clearInterrupt(KEY_BTN_PORT, KEY_BTN_PIN);

#if 0
    GPIO_clearInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN); 
    GPIO_clearInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN); 
#endif
    // Enable port interrupts.
    GPIO_enableInterrupt(ALERTN_PORT, ALERTN_PIN);
    GPIO_enableInterrupt(KEY_BTN_PORT, KEY_BTN_PIN);

#if 0
    GPIO_enableInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    GPIO_enableInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);    
#endif

    en_out_disable();
    // Enable WDT.
//    WDT_A_start(WDT_A_BASE);

    //other gpio set as output pin
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN3);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN4);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN6);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN7);

    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN5);

    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN2);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN3);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN4);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN5);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN0);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN1);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN4);

    GPIO_setAsOutputPin(GPIO_PORT_PJ, GPIO_PIN5);


    return;
}


//*****************************************************************************
//
//! Get Base Address of I2C used for SMBus
//!
//! The base address is used by smbuslib PHY layer for FR5xx eUSCI.
//! The simpler implementation from 2xx doesn't use base address.
//!
//! \param intf Interface number (0 for first interface)
//!
//! \return Base address of I2C module, NULL if no base address used
//
// *****************************************************************************
uint32_t tcpm_hal_get_i2c_base_addr(uint8_t intf)
{
    // Note: Only one interface is supported in this HAL
    // FR5xx implementation uses Base address
    return(EUSCI_B0_BASE);
}

//*****************************************************************************
//
//! Turn on a LED
//!
//! \param led LED_RED, LED2, or LED_BOTH
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_led_high(led_sel_t sel)
{
#if 0
    if (sel & LED_RED)
    {
        GPIO_setOutputHighOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_setOutputHighOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }
#endif
    return;
}

//*****************************************************************************
//
//! Turn off a LED
//!
//! \param led LED_RED, LED2, or LED_BOTH
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_led_low(led_sel_t sel)
{
#if 0
    if (sel & LED_RED)
    {
        GPIO_setOutputLowOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_setOutputLowOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }
#endif
    return;
}


void tcpm_hal_led_toggle(led_sel_t sel)
{
#if 0
    if (sel & LED_RED)
    {
        GPIO_toggleOutputOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_toggleOutputOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }
#endif
    return;
}

void tcpm_hal_set_mux_polarity(uint8_t val)
{
#if 0
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_POL_PORT, MUX_POL_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_POL_PORT, MUX_POL_PIN);
	}
	return;
#endif
}

void tcpm_hal_set_mux_am_sel(uint8_t val)
{
#if 0
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	}
	return;
#endif
}

void tcpm_hal_set_mux_enable(uint8_t val)
{
#if 0
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_EN_PORT, MUX_EN_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_EN_PORT, MUX_EN_PIN);
	}
	return;
#endif
}


void tcpm_hal_hpd_out_control(uint8_t port, uint8_t val)
{
#if 0
	if (val)
	{
		GPIO_setOutputHighOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
	}
	return;
#endif
}

uint8_t tcpm_hal_get_hpd_in(uint8_t port)
{
	// Only a single Type-C port...
	return GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN);
}

void tcpm_hal_vbus_enable(uint8_t port, vbus_select_t sel)
{
    if (port == 0)
    {
#if 0
        if (sel == VBUS_SRC_5V)
        {
            // Disable high voltage.
            GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);

            // Enable 5V.
            GPIO_setOutputHighOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);

            // Enable SRC switch.
            GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        }
        else if (sel == VBUS_SRC_HI_VOLT)
        {
            // Disable 5V.
            GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);

            // Enable high voltage.
            GPIO_setOutputHighOnPin(HV_EN_PORT, HV_EN_PIN);

            // Enable SRC switch.
            GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        }
        else if (sel == VBUS_SNK)
        {
            // Enable SNK switch.
            GPIO_setAsOutputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
        }
#endif

        GPIO_setOutputHighOnPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);


    }
    return;
}


void tcpm_hal_vbus_disable(uint8_t port, vbus_select_t sel)
{
    if (port == 0)
    {
#if 0
        if (sel == VBUS_SRC_5V)
        {
            // Disable SRC switch.
            GPIO_setAsInputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);

            // Disable 5V.
            GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
        }
        else if (sel == VBUS_SRC_HI_VOLT)
        {
            // Disable high voltage.
            GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);
        }
        else if (sel == VBUS_SNK)
        {
            // Disable SNK switch.
            GPIO_setAsInputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
        }
#endif

        GPIO_setOutputLowOnPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);

    }
    return;
}



//*****************************************************************************
//
//!  Clears GPIO Alerts
//! 
//! \return None
//
// *****************************************************************************
void tcpm_hal_clear_interrupt(uint16_t flags)
{
    if (flags & INT_ALERTN)
    {
        GPIO_clearInterrupt(ALERTN_PORT, ALERTN_PIN);
    }
#if 0
    if (flags & INT_FAULT_SRC_P0)
    {
        GPIO_clearInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    }

    if (flags & INT_FAULT_SNK_P0)
    {
        GPIO_clearInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
    }
#endif
    return;
}

//*****************************************************************************
//
//!  Gets state of the GPIO Pins which will indicate whether an alert has been
//!  asserted or not.  Alerts are asserted when the voltage on the pin is low
//! 
//! \return None
//
// *****************************************************************************
uint16_t tcpm_hal_get_interrupts_asserted(void)
{
    uint16_t ret = 0;

    if (GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN) == GPIO_INPUT_PIN_LOW)
    {
        ret |= INT_ALERTN;   // AlertN is asserted if P1.3 is low
    }
#if 0
    if (GPIO_getInputPinValue(SRC_FAULTN_PORT, SRC_FAULTN_PIN) == GPIO_INPUT_PIN_LOW)
    {
        ret |= INT_FAULT_SRC_P0;   // Source fault is asserted if P1.4 is low
    }

    if (GPIO_getInputPinValue(SNK_FAULTN_PORT, SNK_FAULTN_PIN) == GPIO_INPUT_PIN_LOW)
    {
        ret |= INT_FAULT_SNK_P0;   // Sink fault is asserted if P1.5 is low
    }
#endif
    return ret;
}

bool tcpm_is_alert_asserted(unsigned int port)
{
    if (port == 0)
    {
        return (GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN) == GPIO_INPUT_PIN_LOW);
    }
    else
    {
        return false;
    }
}

//*****************************************************************************
//
//! Enables GPIO and timer interrupts
//! 
//! \return None
//
// *****************************************************************************

void tcpm_hal_enable_interrupt(uint16_t flags)
{
    uint16_t timer_index;

    for (timer_index = 0; timer_index < 7; timer_index++)
    {
        if (flags & (1 << timer_index))
        {
            Timer_B_enableCaptureCompareInterrupt(TIMER_B0_BASE, 
                                                  TIMER_B_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1));
        }
    }

    if (flags & INT_ALERTN)
    {
        GPIO_enableInterrupt(ALERTN_PORT, ALERTN_PIN);

        if (!GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN))
        {
            GPIO_setInterrupt(ALERTN_PORT, ALERTN_PIN);
        }
    }
#if 0
    if (flags & INT_FAULT_SRC_P0)
    {
        GPIO_enableInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);

        if (!GPIO_getInputPinValue(SRC_FAULTN_PORT, SRC_FAULTN_PIN))
        {
            GPIO_setInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
        }
    }

    if (flags & INT_FAULT_SNK_P0)
    {
        GPIO_enableInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);

        if (!GPIO_getInputPinValue(SNK_FAULTN_PORT, SNK_FAULTN_PIN))
        {
            GPIO_setInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
        }
    }
#endif
    return;
}

void tcpm_hal_disable_interrupt(uint16_t flags)
{
    uint16_t timer_index;

    for (timer_index = 0; timer_index < 7; timer_index++)
    {
        if (flags & (1 << timer_index))
        {
            Timer_B_disableCaptureCompareInterrupt(TIMER_B0_BASE, 
                                                   TIMER_B_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1));
        }
    }

    if (flags & INT_ALERTN)
    {
        GPIO_disableInterrupt(ALERTN_PORT, ALERTN_PIN);
    }
#if 0
    if (flags & INT_FAULT_SRC_P0)
    {
        GPIO_disableInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    }

    if (flags & INT_FAULT_SNK_P0)
    {
        GPIO_disableInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
    }
#endif
    return;
}


uint16_t tcpm_hal_get_interrupt_enable(void)
{
    uint16_t timer_index;
    uint16_t ret = 0;

    for (timer_index = 0; timer_index < 7; timer_index++)
    {
        if (HWREG16(TIMER_B0_BASE + TIMER_B_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1)) & CCIE)
        {
            ret |= (1 << timer_index);
        }
    }
      
    if (P2IE & BIT0)
    {
        ret |= INT_ALERTN;   // P2.0 for AlertHi0
    }
#if 0
    if (P1IE & BIT4)
    {
        ret |= INT_FAULT_SRC_P0;   // P1.4 for SRC Fault
    }

    if (P1IE & BIT5)
    {
        ret |= INT_FAULT_SNK_P0;   // P1.5 for SNK Fault
    }
#endif
    return ret;
}



void en_out_enable()
{
    GPIO_setOutputHighOnPin(USBA_EN_PORT, USBA_EN_PIN);
}

void en_out_disable()
{
    GPIO_setOutputLowOnPin(USBA_EN_PORT, USBA_EN_PIN);
}

extern void periodic_timer_start();

static void key_pressed_handler()
{
    // check if mode is SNK, switch to SRC if so
//    usb_pd_pr_swap(0);
    periodic_timer_start();
    pb.key_pressed = true;
    pb.loadA_pres = true; // assume load is present

    // Enable USB A port output.
    GPIO_setOutputHighOnPin(USBA_EN_PORT, USBA_EN_PIN);

}

//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! I2C Interrupt Service Routine
//!
//! ISR for I2C module
//!   This ISR generates an event which should be declared in application
//!
//! \return None
//
// *****************************************************************************
#pragma vector=USCI_B0_VECTOR
__interrupt void I2C_ISR(void)
{
    // This event should be declared in Application
    if (HAL_I2C_Event() == true)
    {
        LPM3_EXIT;
    }
}

//*****************************************************************************
//
//! Port1 Interrupt Service Routine
//!
//! ISR for P1
//!
//! \return None
//
// *****************************************************************************
#pragma vector=PORT1_VECTOR
__interrupt void P1_ISR(void)
{
    // Reading P1IV clears the highest priority interrupt.  
    switch (__even_in_range(P1IV,16))
    {
        case 0x00:  break;  // No interrupt pending
        case 0x02:  break;  // P1.0 ISR
        case 0x04:  break;  // P1.1 ISR
        case 0x06:  break;  // P1.2 ISR
                             
        case 0x08:          // P1.3 ISR
#if 0
            // Port 0 ALERT# event.
            g_alert_event |= (1 << 0);
            LPM3_EXIT;
#endif
            break;

        case 0x0A:          // P1.4 ISR
#if 0
            if (tcpm_vbus_src_fault_event(0) == true)
            {
                LPM3_EXIT;
            }
#endif
            break;

        case 0x0C:          // P1.5 ISR
#if 0
            if (tcpm_vbus_snk_fault_event(0) == true)
            {
                LPM3_EXIT;
            }
#endif
            break;

        case 0x0E:  break;  // P1.6 ISR
        case 0x10:  break;  // P1.7 ISR
        default:  break;
    }

    return;
}

//*****************************************************************************
//
//! Port1 Interrupt Service Routine
//!
//! ISR for P1
//!
//! \return None
//
// *****************************************************************************
#pragma vector=PORT2_VECTOR
__interrupt void P2_ISR(void)
{
    // Reading P2IV clears the highest priority interrupt.
    switch (__even_in_range(P2IV,16))
    {
        case 0x00:  break;  // No interrupt pending
        case 0x02:  // P2.0 ISR
            //ALERT# event.
            g_alert_event |= (1 << 0);
            LPM3_EXIT;
            break;
        case 0x04:  break;  // P2.1 ISR
        case 0x06:  break;  // P2.2 ISR

        case 0x08:  break;  // P2.3

        case 0x0A:  break;  // P2.4
        case 0x0C:
            key_pressed_handler();
            LPM3_EXIT;
            break;  // P2.5

        case 0x0E:  break;  // P2.6 ISR
        case 0x10:  break;  // P2.7 ISR
        default:  break;
    }

    return;
}

//*****************************************************************************
//
//! Low level initialization
//!
//! Disables the watchdog after reset
//!
//! \return 1: Initialize all variables
//
// *****************************************************************************
#ifdef __IAR_SYSTEMS_ICC__
int __low_level_init(void)
#elif defined (__TI_COMPILER_VERSION__)
int _system_pre_init(void)
#endif
{
    WDT_A_hold(WDT_A_BASE);             //Stop WDT
    return(1);
}

//*****************************************************************************
//
//! Dummy ISR
//!
//! Dummy ISR in case an interrupt is enabled unintentionally
//!
//! \return none
//
// *****************************************************************************
#pragma vector=RTC_VECTOR,TIMER0_A1_VECTOR,TIMER2_A0_VECTOR,TIMER2_A1_VECTOR, \
    TIMER3_A0_VECTOR,TIMER3_A1_VECTOR, \
    USCI_A1_VECTOR,PORT3_VECTOR,TIMER1_A1_VECTOR, \
    SYSNMI_VECTOR,AES256_VECTOR,  \
    TIMER1_A0_VECTOR, COMP_E_VECTOR, UNMI_VECTOR
__interrupt void Dummy_ISR(void)
{
    while (1);
}
