/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __TCPM_HAL_H__
#define __TCPM_HAL_H__

//
// Include files
//
#include <stdint.h>
#include <stdbool.h>
#include "msp430.h"

// Type-C Port-0 Pin Definition.
#define SNK_VBUS_EN_PORT  GPIO_PORT_P3
#define SNK_VBUS_EN_PIN   GPIO_PIN0

#define SRC_VBUS_EN_PORT  GPIO_PORT_P3
#define SRC_VBUS_EN_PIN   GPIO_PIN0

#define VBUS_5V_EN_PORT   GPIO_PORT_P3
#define VBUS_5V_EN_PIN    GPIO_PIN5

#define HV_EN_PORT        GPIO_PORT_P2
#define HV_EN_PIN         GPIO_PIN2

#define VCONN_EN_PORT     GPIO_PORT_P3
#define VCONN_EN_PIN      GPIO_PIN4

#define ALERTN_PORT       GPIO_PORT_P2
#define ALERTN_PIN        GPIO_PIN0

#define SRC_FAULTN_PORT   GPIO_PORT_P1
#define SRC_FAULTN_PIN    GPIO_PIN4

#define SNK_FAULTN_PORT   GPIO_PORT_P1
#define SNK_FAULTN_PIN    GPIO_PIN5

// General Pin Definition.
#define RED_LED_PORT      GPIO_PORT_P4
#define RED_LED_PIN       GPIO_PIN6
                          
#define GREEN_LED_PORT    GPIO_PORT_P1
#define GREEN_LED_PIN     GPIO_PIN0

// I2C Master Pin Definition.
#define I2C_PORT          GPIO_PORT_P1
#define I2C_SDA_PIN       GPIO_PIN6
#define I2C_SCL_PIN       GPIO_PIN7

// Mux control Pin Definition.
#define MUX_POL_PORT      GPIO_PORT_P3
#define MUX_POL_PIN       GPIO_PIN2

#define MUX_AM_SEL_PORT   GPIO_PORT_P2
#define MUX_AM_SEL_PIN    GPIO_PIN5

#define MUX_EN_PORT       GPIO_PORT_P4
#define MUX_EN_PIN        GPIO_PIN3

// DisplayPort HPD Pin Definition.
#define HPD_OUT_PORT      GPIO_PORT_P4
#define HPD_OUT_PIN       GPIO_PIN2

#define HPD_IN_PORT       GPIO_PORT_P4
#define HPD_IN_PIN        GPIO_PIN2

#define USBA_EN_PORT      GPIO_PORT_P3
#define USBA_EN_PIN       GPIO_PIN7

#define OTG_EN_PORT       GPIO_PORT_P3
#define OTG_EN_PIN        GPIO_PIN6

#define BUS_DSG_PORT      GPIO_PORT_P3
#define BUS_DSG_PIN       GPIO_PIN1

#define CHRG_OK_PORT      GPIO_PORT_P2
#define CHRG_OK_PIN       GPIO_PIN2

#define KEY_BTN_PORT      GPIO_PORT_P2
#define KEY_BTN_PIN       GPIO_PIN5

/*! Definition of launchpad LEDs */
typedef enum
{
    LED_RED   = (1 << 0),
    LED_GREEN = (1 << 1),
    LED_BOTH  = (LED_RED | LED_GREEN)
} led_sel_t;


typedef enum
{
    // Timer Interrupts.
    INT_TIMER0        = (1 << 0),
    INT_TIMER1        = (1 << 1),
    INT_TIMER2        = (1 << 2),
    INT_TIMER3        = (1 << 3),
    INT_TIMER4        = (1 << 4),
    INT_TIMER5        = (1 << 5),
    INT_TIMER6        = (1 << 6),
    INT_TIMER_MASK    = 0x7F,

    // Port 0 Interrupts.
    INT_ALERTN     = (1 << 7),
    INT_FAULT_SRC_P0  = (1 << 8),
    INT_FAULT_SNK_P0  = (1 << 9),
    INT_MASK_P0       = 0x380,

    INT_MASK_ALL      = 0x3FF

} tcpm_interrupt_flag_t;

typedef enum
{
	VBUS_SRC_5V       = (1 << 0),
	VBUS_SRC_HI_VOLT  = (1 << 1),
	VBUS_SNK          = (1 << 2)
} vbus_select_t;


#if DEBUG_LEVEL >= 1
//#define LED_ON(sel)   tcpm_hal_led_high(sel)
//#define LED_OFF(sel)  tcpm_hal_led_low(sel)
#else
//#define LED_ON(sel) {}
//#define LED_OFF(sel) {}
#endif


//
// Function prototypes
//

void tcpm_hal_clear_interrupt(uint16_t flags);
void tcpm_hal_disable_interrupt(uint16_t flags);
void tcpm_hal_enable_interrupt(uint16_t flags);
uint16_t tcpm_hal_get_interrupt_enable(void);
uint16_t tcpm_hal_get_interrupts_asserted(void);

uint32_t tcpm_hal_get_i2c_base_addr(uint8_t intf);
bool tcpm_is_alert_asserted(unsigned int port);
void tcpm_hal_init(void);

void tcpm_hal_led_high(led_sel_t sel);
void tcpm_hal_led_low(led_sel_t sel);
void tcpm_hal_led_toggle(led_sel_t sel);

void tcpm_hal_set_mux_polarity(uint8_t val);
void tcpm_hal_set_mux_am_sel(uint8_t val);
void tcpm_hal_set_mux_enable(uint8_t val);

void tcpm_hal_vbus_enable(uint8_t port, vbus_select_t sel);
void tcpm_hal_vbus_disable(uint8_t port, vbus_select_t sel);
void tcpm_hal_hpd_out_control(uint8_t port, uint8_t val);
uint8_t tcpm_hal_get_hpd_in(uint8_t port);

void en_out_enable();
void en_out_disable();
//
// Events (called by HAL, declared in Application)
//


#endif //__TCPM_HAL_H__
