/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "tcpm_hal.h"
#include "driverlib.h"
#include "tusb422_common.h"
#include <stdint.h>

unsigned int g_alert_event = 0;

//
// Local functions
//

extern void usb_pd_pe_hpd_in_event(unsigned int port, uint8_t val);
extern bool tcpm_vbus_snk_fault_event(unsigned int port);
extern bool tcpm_vbus_src_fault_event(unsigned int port);
extern bool tcpm_alert_event(unsigned int port);
extern bool HAL_I2C_Event(void);


//*****************************************************************************
//
//! Clock Initialization
//!
//! This demo sets SMCLK/MCLK = DCO @ MCLK_MHZ (defined in project settings)
//!                ACLK = VLO
//!
//! \return None
//
// *****************************************************************************
void SystemInit(void)
{
    WDT_A_holdTimer();

    // Enable PJ[0:1] for crystal
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_PJ,
                                               GPIO_PIN0 | GPIO_PIN1,
                                               GPIO_PRIMARY_MODULE_FUNCTION);

    SYSCTL->SRAM_BANKEN = SYSCTL_SRAM_BANKEN_BNK7_EN;   // Enable all SRAM banks

#if (MCLK_MHZ == 1)
    //Set DCO Frequency to 8MHz
    CS_setDCOFrequency(8000000);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_8);
    //Initialize HSMCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_initClockSignal(CS_HSMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_8);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_8);

#elif (MCLK_MHZ == 8)
    //Set DCO Frequency to 8MHz
    CS_setDCOFrequency(8000000);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    //Initialize HSMCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_initClockSignal(CS_HSMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

#elif (MCLK_MHZ == 16)

    // 1 flash wait state (BANK0 VCORE0 max is 12 MHz)
    FLCTL->BANK0_RDCTL &= ~FLCTL_BANK0_RDCTL_WAIT_MASK | FLCTL_BANK0_RDCTL_WAIT_1;
    FLCTL->BANK1_RDCTL &= ~FLCTL_BANK0_RDCTL_WAIT_MASK | FLCTL_BANK0_RDCTL_WAIT_1;

    // Enabling FPU for DCO Frequency calculation.
    FPU_enableModule();
    //Set DCO Frequency to 16MHz
    CS_setDCOFrequency(16000000);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    //Initialize HSMCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_initClockSignal(CS_HSMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

#else
#error "Frequency not supported, change project setttings"
#endif

    // Set Flash Bank read buffering
    FLCTL->BANK0_RDCTL |= (FLCTL_BANK0_RDCTL_BUFD | FLCTL_BANK0_RDCTL_BUFI);
    FLCTL->BANK1_RDCTL &= ~(FLCTL_BANK0_RDCTL_BUFD | FLCTL_BANK0_RDCTL_BUFI);

    return;
}

//*****************************************************************************
//
//! MCU initialization
//!
//! Initializes the MSP430
//!  Sets GPIOs to default state
//!  Configures clocks
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_init(void)
{
    /* Halting the Watchdog */
    WDT_A_holdTimer();

    /* Configuring pins for peripheral/crystal usage */
    GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_PJ,
                                                GPIO_PIN0 | GPIO_PIN1, 
                                                GPIO_PRIMARY_MODULE_FUNCTION);

    // Set LF external clock frequency to 32.768 KHz
    CS_setExternalClockSourceFrequency(32768, 48000000);

    // Start XT1 with no time out and lowest drive strength.
    CS_startLFXT(CS_LFXT_DRIVE0);

    // Set ACLK=LFXT
    CS_initClockSignal(CS_ACLK, CS_LFXTCLK_SELECT, CS_CLOCK_DIVIDER_1);

    // Turn off Temp sensor
    REFCTL0 |= REFTCOFF;
    REFCTL0 &= ~REFON;

    // Set I2C pins for SCL/SDA functionality.
    GPIO_setAsPeripheralModuleFunctionOutputPin(I2C_PORT, I2C_SDA_PIN + I2C_SCL_PIN,
                                                GPIO_PRIMARY_MODULE_FUNCTION);

    // SNK VBUS_EN#. (open drain, active low)
    GPIO_setAsInputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
    GPIO_setOutputLowOnPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);

    // SRC VBUS_EN#. (open drain, active low)
    GPIO_setAsInputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
    GPIO_setOutputLowOnPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);

    // 5V_EN. (high to enable, low to disable)
    GPIO_setAsOutputPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
    GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
    
    // HV_EN. (high to enable, low to disable)
    GPIO_setAsOutputPin(HV_EN_PORT, HV_EN_PIN);
    GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);

    // P3.4 is VCONN_EN. (high to enable)
    GPIO_setAsOutputPin(VCONN_EN_PORT, VCONN_EN_PIN);
    GPIO_setOutputHighOnPin(VCONN_EN_PORT, VCONN_EN_PIN);
    
    // LED1 output.
    // LED2 output.
    GPIO_setOutputLowOnPin(RED_LED_PORT, RED_LED_PIN);
    GPIO_setAsOutputPin(RED_LED_PORT, RED_LED_PIN);
    GPIO_setOutputLowOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    GPIO_setAsOutputPin(GREEN_LED_PORT, GREEN_LED_PIN);

	// Mux control. (Set polarity, Alt-mode select, and enable to low -> Mux power down)
    GPIO_setOutputLowOnPin(MUX_POL_PORT, MUX_POL_PIN);
    GPIO_setAsOutputPin(MUX_POL_PORT, MUX_POL_PIN);
	GPIO_setOutputLowOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
    GPIO_setAsOutputPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	GPIO_setOutputLowOnPin(MUX_EN_PORT, MUX_EN_PIN);
    GPIO_setAsOutputPin(MUX_EN_PORT, MUX_EN_PIN);

    // Set HPD_OUT low.
    GPIO_setOutputLowOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
    GPIO_setAsOutputPin(HPD_OUT_PORT, HPD_OUT_PIN);

    // Set HPD_IN as input.
    GPIO_setAsInputPinWithPullDownResistor(HPD_IN_PORT, HPD_IN_PIN);
    if (GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN))
    {
        GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    }
    else /* HPD_IN is low */
    {
        GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_LOW_TO_HIGH_TRANSITION);
    }

    // Configure input pins for interrupts.
    // ALERTN.
    // SRC FAULT#.
    // SNK FAULT#.
    GPIO_setAsInputPinWithPullUpResistor(ALERTN_PORT, ALERTN_PIN);
    GPIO_setAsInputPinWithPullUpResistor(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    GPIO_setAsInputPinWithPullUpResistor(SNK_FAULTN_PORT, SNK_FAULTN_PIN);

    GPIO_selectInterruptEdge(ALERTN_PORT, ALERTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_selectInterruptEdge(SRC_FAULTN_PORT, SRC_FAULTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_selectInterruptEdge(SNK_FAULTN_PORT, SNK_FAULTN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);

    // Clear all PxIFGs to avoid erroneous port interrupts.
    GPIO_clearInterrupt(ALERTN_PORT, ALERTN_PIN);
    GPIO_clearInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    GPIO_clearInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);

    // Enable port interrupts.
    GPIO_enableInterrupt(ALERTN_PORT, ALERTN_PIN);
    GPIO_enableInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    GPIO_enableInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
    GPIO_enableInterrupt(HPD_IN_PORT, HPD_IN_PIN);

    // Enable NVIC interrupts.
    Interrupt_enableInterrupt(INT_PORT3);
    Interrupt_enableInterrupt(INT_PORT5);
    Interrupt_enableInterrupt(INT_PORT6);

    // Enable WDT.
//    WDT_A_start(WDT_A_BASE);

    return;
}


//*****************************************************************************
//
//! Get Base Address of I2C used for SMBus
//!
//! The base address is used by smbuslib PHY layer for FR5xx eUSCI.
//! The simpler implementation from 2xx doesn't use base address.
//!
//! \param intf Interface number (0 for first interface)
//!
//! \return Base address of I2C module, NULL if no base address used
//
// *****************************************************************************
uint32_t tcpm_hal_get_i2c_base_addr(uint8_t intf)
{
    // Note: Only one interface is supported in this HAL
    return(EUSCI_B0_BASE);
}

//*****************************************************************************
//
//! Turn on a LED
//!
//! \param led LED_RED, LED2, or LED_BOTH
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_led_high(led_sel_t sel)
{
    if (sel & LED_RED)
    {
        GPIO_setOutputHighOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_setOutputHighOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }

    return;
}

//*****************************************************************************
//
//! Turn off a LED
//!
//! \param led LED_RED, LED2, or LED_BOTH
//!
//! \return None
//
// *****************************************************************************
void tcpm_hal_led_low(led_sel_t sel)
{
    if (sel & LED_RED)
    {
        GPIO_setOutputLowOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_setOutputLowOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }

    return;
}


void tcpm_hal_led_toggle(led_sel_t sel)
{
    if (sel & LED_RED)
    {
        GPIO_toggleOutputOnPin(RED_LED_PORT, RED_LED_PIN);
    }

    if (sel & LED_GREEN)
    {
        GPIO_toggleOutputOnPin(GREEN_LED_PORT, GREEN_LED_PIN);
    }

    return;
}

void tcpm_hal_set_mux_polarity(uint8_t val)
{
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_POL_PORT, MUX_POL_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_POL_PORT, MUX_POL_PIN);
	}
	return;
}

void tcpm_hal_set_mux_am_sel(uint8_t val)
{
	if (val)
	{
		GPIO_setOutputHighOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	}
	else
	{
		GPIO_setOutputLowOnPin(MUX_AM_SEL_PORT, MUX_AM_SEL_PIN);
	}
	return;
}

void tcpm_hal_set_mux_enable(uint8_t val)
{
    if (val)
    {
        GPIO_setOutputHighOnPin(MUX_EN_PORT, MUX_EN_PIN);
    }
    else
    {
        GPIO_setOutputLowOnPin(MUX_EN_PORT, MUX_EN_PIN);
    }
    return;
}


void tcpm_hal_hpd_out_control(uint8_t port, uint8_t val)
{
    // Only a single Type-C port.
    if (val)
    {
        GPIO_setOutputHighOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
    }
    else
    {
        GPIO_setOutputLowOnPin(HPD_OUT_PORT, HPD_OUT_PIN);
    }
    return;
}

uint8_t tcpm_hal_get_hpd_in(uint8_t port)
{
    // Only a single Type-C port.
    return GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN);
}

void tcpm_hal_vbus_enable(uint8_t port, vbus_select_t sel)
{
    if (port == 0)
    {
        if (sel == VBUS_SRC_5V)
        {
            // Disable high voltage.
            GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);

            // Enable 5V.
            GPIO_setOutputHighOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);

            // Enable SRC switch.
            GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        }
        else if (sel == VBUS_SRC_HI_VOLT)
        {
            // Disable 5V.
            GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);

            // Enable high voltage.
            GPIO_setOutputHighOnPin(HV_EN_PORT, HV_EN_PIN);

            // Enable SRC switch.
            GPIO_setAsOutputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);
        }
        else if (sel == VBUS_SNK)
        {
            // Enable SNK switch.
            GPIO_setAsOutputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
        }
    }
    return;
}


void tcpm_hal_vbus_disable(uint8_t port, vbus_select_t sel)
{
    if (port == 0)
    {
        if (sel == VBUS_SRC_5V)
        {
            // Disable SRC switch.
            GPIO_setAsInputPin(SRC_VBUS_EN_PORT, SRC_VBUS_EN_PIN);

            // Disable 5V.
            GPIO_setOutputLowOnPin(VBUS_5V_EN_PORT, VBUS_5V_EN_PIN);
        }
        else if (sel == VBUS_SRC_HI_VOLT)
        {
            // Disable high voltage.
            GPIO_setOutputLowOnPin(HV_EN_PORT, HV_EN_PIN);
        }
        else if (sel == VBUS_SNK)
        {
            // Disable SNK switch.
            GPIO_setAsInputPin(SNK_VBUS_EN_PORT, SNK_VBUS_EN_PIN);
        }
    }
    return;
}



//*****************************************************************************
//
//!  Clears GPIO Alerts
//! 
//! \return None
//
// *****************************************************************************
void tcpm_hal_clear_interrupt(uint16_t flags)
{
    if (flags & INT_ALERTN_P0)
    {
        GPIO_clearInterrupt(ALERTN_PORT, ALERTN_PIN);
    }

    if (flags & INT_FAULT_SRC_P0)
    {
        GPIO_clearInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    }

    if (flags & INT_FAULT_SNK_P0)
    {
        GPIO_clearInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
    }

    return;
}

//*****************************************************************************
//
//!  Gets state of the GPIO Pins which will indicate whether an alert has been
//!  asserted or not.  Alerts are asserted when the voltage on the pin is low
//! 
//! \return None
//
// *****************************************************************************
uint16_t tcpm_hal_get_interrupts_asserted(void)
{
    uint16_t ret = 0;

    if (GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN) == GPIO_INPUT_PIN_LOW)
    {
        ret |= INT_ALERTN_P0;   // AlertN is asserted if low
    }

    if (GPIO_getInputPinValue(SRC_FAULTN_PORT, SRC_FAULTN_PIN) == GPIO_INPUT_PIN_LOW)
    {
        ret |= INT_FAULT_SRC_P0;   // Source fault is asserted if low
    }

    if (GPIO_getInputPinValue(SNK_FAULTN_PORT, SNK_FAULTN_PIN) == GPIO_INPUT_PIN_LOW)
    {
        ret |= INT_FAULT_SNK_P0;   // Sink fault is asserted if low
    }

    return ret;
}

bool tcpm_is_alert_asserted(unsigned int port)
{
    if (port == 0)
    {
        return (GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN) == GPIO_INPUT_PIN_LOW);
    }
    else
    {
        return false;
    }
}

//*****************************************************************************
//
//! Enables GPIO and timer interrupts
//! 
//! \return None
//
// *****************************************************************************

void tcpm_hal_enable_interrupt(uint16_t flags)
{
    uint16_t timer_index;

    for (timer_index = 0; timer_index < 7; timer_index++)
    {
        if (flags & (1 << timer_index))
        {
            Timer_A_enableCaptureCompareInterrupt(TIMER_A0_BASE, 
                                                  TIMER_A_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1));
        }
    }

    if (flags & INT_ALERTN_P0)
    {
        GPIO_enableInterrupt(ALERTN_PORT, ALERTN_PIN);

        if (!GPIO_getInputPinValue(ALERTN_PORT, ALERTN_PIN))
        {
            GPIO_setInterruptFlag(ALERTN_PORT, ALERTN_PIN);
        }
    }

    if (flags & INT_FAULT_SRC_P0)
    {
        GPIO_enableInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);

        if (!GPIO_getInputPinValue(SRC_FAULTN_PORT, SRC_FAULTN_PIN))
        {
            GPIO_setInterruptFlag(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
        }
    }

    if (flags & INT_FAULT_SNK_P0)
    {
        GPIO_enableInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);

        if (!GPIO_getInputPinValue(SNK_FAULTN_PORT, SNK_FAULTN_PIN))
        {
            GPIO_setInterruptFlag(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
        }
    }

    return;
}

void tcpm_hal_disable_interrupt(uint16_t flags)
{
    uint16_t timer_index;

    for (timer_index = 0; timer_index < 7; timer_index++)
    {
        if (flags & (1 << timer_index))
        {
            Timer_A_disableCaptureCompareInterrupt(TIMER_A0_BASE, 
                                                   TIMER_A_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1));
        }
    }

    if (flags & INT_ALERTN_P0)
    {
        GPIO_disableInterrupt(ALERTN_PORT, ALERTN_PIN);
    }

    if (flags & INT_FAULT_SRC_P0)
    {
        GPIO_disableInterrupt(SRC_FAULTN_PORT, SRC_FAULTN_PIN);
    }

    if (flags & INT_FAULT_SNK_P0)
    {
        GPIO_disableInterrupt(SNK_FAULTN_PORT, SNK_FAULTN_PIN);
    }

    return;
}


uint16_t tcpm_hal_get_interrupt_enable(void)
{
    uint16_t timer_index;
    uint16_t ret = 0;

    for (timer_index = 0; timer_index < 7; timer_index++)
    {
        if (HWREG16(TIMER_A0_BASE + TIMER_A_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1)) & CCIE)
        {
            ret |= (1 << timer_index);
        }
    }
      
    if (P3IE & BIT6)
    {
        ret |= INT_ALERTN_P0;   // P3.6 for AlertHi0
    }

    if (P5IE & BIT2)
    {
        ret |= INT_FAULT_SRC_P0;   // P5.2 for SRC Fault
    }

    if (P5IE & BIT0)
    {
        ret |= INT_FAULT_SNK_P0;   // P5.0 for SNK Fault
    }

    return ret;
}



//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! I2C Interrupt Service Routine
//!
//! ISR for I2C module
//!   This ISR generates an event which should be declared in application
//!
//! \return None
//
// *****************************************************************************
void I2C_ISR(void)
{
    // This event should be declared in Application
    if (HAL_I2C_Event() == false)
    {
//        __sleep();
    }
}

//*****************************************************************************
//
//! Port3 Interrupt Service Routine
//!
//! ISR for P3
//!
//! \return None
//
// *****************************************************************************
void P3_ISR(void)
{
    uint32_t status;

    status = GPIO_getEnabledInterruptStatus(GPIO_PORT_P3);

    if (status & GPIO_PIN6)
    {
		g_alert_event |= (1 << 0);
    }

    GPIO_clearInterruptFlag(GPIO_PORT_P3, status);

    return;
}


//*****************************************************************************
//
//! Port5 Interrupt Service Routine
//!
//! ISR for P5
//!
//! \return None
//
// *****************************************************************************
void P5_ISR(void)
{
    uint32_t status;

    status = GPIO_getEnabledInterruptStatus(GPIO_PORT_P5);

    if (status & GPIO_PIN0)
    {
        if (tcpm_vbus_snk_fault_event(0) == false)
        {
//            __sleep();
        }
    }

    if (status & GPIO_PIN2)
    {
        if (tcpm_vbus_src_fault_event(0) == false)
        {
//            __sleep();
        }
    }

    GPIO_clearInterruptFlag(GPIO_PORT_P5, status);

    return;
}

//*****************************************************************************
//
//! Port5 Interrupt Service Routine
//!
//! ISR for P6
//!
//! \return None
//
// *****************************************************************************
void P6_ISR(void)
{
	uint8_t hpd_in;
    uint32_t status;

    status = GPIO_getEnabledInterruptStatus(GPIO_PORT_P6);

    if (status & HPD_IN_PIN)
    {
        hpd_in = GPIO_getInputPinValue(HPD_IN_PORT, HPD_IN_PIN);

        // Notify upper layers of HPD_IN change.
        usb_pd_pe_hpd_in_event(0, hpd_in);

        if (hpd_in)
        {
            GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
        }
        else /* HPD_IN is low */
        {
            GPIO_selectInterruptEdge(HPD_IN_PORT, HPD_IN_PIN, GPIO_LOW_TO_HIGH_TRANSITION);
        }

        // Set dummy alert event so PD state machine will get run.
        g_alert_event |= (1 << 0);
    }

    GPIO_clearInterruptFlag(GPIO_PORT_P6, status);

    return;
}

//*****************************************************************************
//
//! Low level initialization
//!
//! Disables the watchdog after reset
//!
//! \return 1: Initialize all variables
//
// *****************************************************************************
#ifdef __IAR_SYSTEMS_ICC__
int __low_level_init(void)
#elif defined (__TI_COMPILER_VERSION__)
int _system_pre_init(void)
#endif
{
//    WDT_A_hold(WDT_A_BASE);    //Stop WDT
    return(1);
}


