/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 *
 *
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "timer.h"
#include "driverlib.h"
#include "tcpm.h"
#include "tcpm_hal.h"
#include "tusb422_common.h"
#include "uart.h"
#include <stddef.h>  /* for NULL def */
#include <stdbool.h>
#include <stdint.h>

#define MAX_TIMERS 7


static uint32_t timer_cnt_hi = 0;

struct tusb422_timer_t * timer_list[MAX_TIMERS] = {0};

unsigned int g_timer_expired = 0;

timer_callback_t g_timer_callback[NUM_TCPC_DEVICES];

#define TIMER_TICKS_PER_MS  11  // 11 x 0.091ms = ~1ms.

#define MILLSEC_TO_TICKS(x)  ((uint32_t)(x) * TIMER_TICKS_PER_MS)


static uint32_t get_time(void)
{
    return ((timer_cnt_hi << 16) | Timer_A_getCounterValue(TIMER_A0_BASE));
}


static void _start_timer(unsigned int timer_index)
{
    struct tusb422_timer_t *timer = timer_list[timer_index];

     // Check if timer is expiring within the current timer period.  
     if (timer_cnt_hi == (timer->expires >> 16))
     {
         timer->queued = false;

         Timer_A_setCompareValue(TIMER_A0_BASE,
                                 TIMER_A_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1),
                                 (uint16_t)timer->expires);

         Timer_A_clearCaptureCompareInterrupt(TIMER_A0_BASE, 
                                              TIMER_A_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1));

         Timer_A_enableCaptureCompareInterrupt(TIMER_A0_BASE, 
                                               TIMER_A_CAPTURECOMPARE_REGISTER_0 + (timer_index << 1));


         Timer_A_startCounter(TIMER_A0_BASE,
                              TIMER_A_CONTINUOUS_MODE);
     }
     else
     {
         timer->queued = true;

         if (0 == (timer->expires & 0xFFFF))
         {
             timer->expires += 1;
         }
     }

     return;
}



int timer_start(struct tusb422_timer_t *timer, unsigned int timeout_ms, void (*function)(unsigned int))   
{
    unsigned int i;

    timer_cancel(timer);

    for (i = 0; i < MAX_TIMERS; i++)
    {
        if (timer_list[i] == NULL)
        {
            timer->expires = get_time() + MILLSEC_TO_TICKS(timeout_ms);
            timer->function = function;
            
            timer_list[i] = timer;

            _start_timer(i);

            // Exit for loop.
            break;
        }
    } 

    if (i <= MAX_TIMERS)
    {
        return i;
    }
    else
    {
        CRIT("Error: No more timers!\n");
        return -1;
    }

}


void timer_cancel(struct tusb422_timer_t *timer)
{
    unsigned int i;

    for (i = 0; i < MAX_TIMERS; i++)
    {
        if (timer_list[i] == timer)
        {
            Timer_A_disableCaptureCompareInterrupt(TIMER_A0_BASE,
                                                   TIMER_A_CAPTURECOMPARE_REGISTER_0 + (i << 1));

            timer_list[i] = NULL;
            break;
        }
    }

	// Clear global here in case timer expired but callback has not been executed yet.
	g_timer_expired &= ~(1 << timer->data);

    return;
}


void timer_init(void)
{
    Timer_A_ContinuousModeConfig param = {0};

    // Clock source = ACLK/3 = 32768Hz / 3 ==> 0.091 ms per tick
    param.clockSource = TIMER_A_CLOCKSOURCE_ACLK;
    param.clockSourceDivider = TIMER_A_CLOCKSOURCE_DIVIDER_3;
    param.timerInterruptEnable_TAIE = TIMER_A_TAIE_INTERRUPT_ENABLE;
    param.timerClear = TIMER_A_DO_CLEAR;

    Timer_A_initContinuousMode(TIMER_A0_BASE, &param);

    // Enable NVIC interrupts.
    Interrupt_enableInterrupt(INT_TA0_0);
    Interrupt_enableInterrupt(INT_TA0_N);

    return;
}


static void ccr_interrupt_handler(int ccr_num)
{
    struct tusb422_timer_t *timer = timer_list[ccr_num];

    Timer_A_disableCaptureCompareInterrupt(TIMER_A0_BASE,
                                           TIMER_A_CAPTURECOMPARE_REGISTER_0 + (ccr_num << 1));

    if (timer != NULL)
    {
        // Clear timer pointer.
        timer_list[ccr_num] = NULL;

        if (get_time() >= timer->expires)  // Extra check (might be superfluous).
        {
            if (timer->function)
            {
                g_timer_expired |= (1 << timer->data);
                g_timer_callback[timer->data] = timer->function;
            }
        }
        else
        {
            CRIT("Error: t:%lx < expiry:%lx\n", get_time(), timer->expires);
            CRIT("CCR%u\n", ccr_num);
        }
    }

    return;
}

/* Do not use msleep() within interrupt context */
void msleep(uint16_t msecs)
{
    int idx;
    struct tusb422_timer_t t;

    idx = timer_start(&t, msecs, NULL);

    if (idx >= 0)
    {
        __enable_irq();
        __sleep();  
        // In case we are woken up by some other IRQ.
        while (TIMER_A_CMSIS(TIMER_A0_BASE)->CCTL[idx] & TIMER_A_CCTLN_CCIE);
    }

    return;
}

void timer0_A0_isr(void)
{
    // Clear CCR0 interrupt flag.
    Timer_A_clearCaptureCompareInterrupt(TIMER_A0_BASE, TIMER_A_CAPTURECOMPARE_REGISTER_0);

    // CCR0
    ccr_interrupt_handler(0);
    return;
}


void timer0_A1_isr(void)
{
    unsigned int i;
    bool active_timers = false;
    uint16_t vect = TIMER_A_CMSIS(TIMER_A0_BASE)->IV;

    // Any access, read or write, of the TAIV register automatically resets the
    // highest "pending" interrupt flag.
    switch (vect)
    {
        case  0: // No interrupt
            break;                              

        case  2: // CCR1
        case  4: // CCR2
        case  6: // CCR3
        case  8: // CCR4
        case 10: // CCR5
        case 12: // CCR6
            ccr_interrupt_handler(vect >> 1);
            break;                              

        case 14: // Overflow
            timer_cnt_hi++;

            for (i = 0; i < MAX_TIMERS; i++)
            {
                if (timer_list[i])
                {
                    active_timers = true;

                    if (timer_list[i]->queued)
                    {
                        // Start compare for any applicable queued timers.
                        _start_timer(i);
                    }
                }
            }

            if (!active_timers)
            {
                // Stop timer.
                Timer_A_stop(TIMER_A0_BASE);

                // Zero out time.
                Timer_A_clear(TIMER_A0_BASE);
                timer_cnt_hi = 0;
            }
            break;

        default: 
            break;
    }

    return;
}


