/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//
// Include files
//
#include <stdint.h>
#include <string.h>
#include "msp430.h"
#include "i2c_master.h"
#include "tusb422_common.h"
//#include "inc/hw_memmap.h"
#include "usci_b_i2c.h"

#define HWREG16(x)   (*((volatile uint16_t *)((uint16_t)x)))
#define HWREG8(x)    (*((volatile uint8_t *)((uint16_t)x)))


uint8_t tx_buf[33];

#define I2C_MASTER_XFER_TIMEOUT_MS  100

static i2c_status_t i2c_master_xfer_start(i2c_master_t *i2c, uint8_t slave_addr, uint8_t *data, uint8_t len)
{
    i2c_status_t status;

    // Wait for stop to be sent for any previous transfers.
    while (HWREG8(i2c->base_addr + OFS_UCBxCTL1) & UCTXSTP);

    i2c->xfer_timeout = false;
    i2c->xfer_cmplt = false;
    i2c->buf = data;
    i2c->xfer_len = len;
    i2c->byte_cnt = 0;

    // Set slave address.
    HWREG8(i2c->base_addr + OFS_UCBxI2CSA) = slave_addr;

    // Send start.
    HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTXSTT;

    if (!(HWREG8(i2c->base_addr + OFS_UCBxCTL1) & UCTR))
    {
        // If reading one byte, STOP must be set while receiving the byte.
        if (len == 1)
        {
            // Wait for Start to be sent.
            while (HWREG8(i2c->base_addr + OFS_UCBxCTL1) & UCTXSTT);

            // Set stop.
            HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTXSTP;
        }
    }

    // Start timeout timer.
//    timer_start(I2C_MASTER_XFER_TIMEOUT_MS);

    while (!i2c->xfer_cmplt && !i2c->xfer_timeout)
    {
        _BIS_SR(LPM1_bits + GIE);    // Enter LPM with interrupt.
        //DEBUG_NOP();
    }

    if (i2c->xfer_timeout)
    {
        status = STATUS_TIMEOUT;
    }
    else if (i2c->nack)
    {
        status = STATUS_NACK;
    }
    else
    {
        status = STATUS_OK;
    }

    return status;
}


i2c_status_t i2c_master_read(i2c_master_t *i2c, uint8_t slave_addr, uint8_t *data, uint8_t len)
{
    INFO("%s - len = %u\n", __func__, len);

    // Clear transmitter.
    HWREG8(i2c->base_addr + OFS_UCBxCTL1) &= ~UCTR;

    return i2c_master_xfer_start(i2c, slave_addr, data, len);

}

i2c_status_t i2c_master_write(i2c_master_t *i2c, uint8_t slave_addr, uint8_t *data, uint8_t len, bool send_stop)
{
    INFO("%s - len = %u, stop = %u\n", __func__, len, send_stop);
    i2c->send_stop = send_stop;

    // Set transmitter.
    HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTR;

    return i2c_master_xfer_start(i2c, slave_addr, data, len);
}


i2c_status_t i2c_master_read_register(i2c_master_t *i2c, uint8_t slave_addr, uint8_t reg, void *data, uint8_t len)
{
    i2c_status_t status;

    // Write the register address with no Stop.
    status = i2c_master_write(i2c, slave_addr, &reg, 1, false);

    if (status == STATUS_OK)
    {
        // Read the data.
        status = i2c_master_read(i2c, slave_addr, data, len);
    }

    return status;
}


i2c_status_t i2c_master_write_register(i2c_master_t *i2c, uint8_t slave_addr, uint8_t reg, const void *data, uint8_t len)
{
    tx_buf[0] = reg;
    memcpy(&tx_buf[1], data, len);

    // Write the reg and data followed by Stop.
    return i2c_master_write(i2c, slave_addr, tx_buf, len + 1, true);
}


#define EDDC_SEGMENT_POINTER_ADDR (0x60 >> 1)

i2c_status_t i2c_master_hdmi_read(i2c_master_t *i2c, uint8_t slave_addr, uint8_t segment, uint8_t reg, void *data, uint8_t len)
{
    i2c_status_t status = STATUS_OK;

    INFO("hdmi_read - len = %u, seg = %u, reg = %u\n", len, segment, reg);

    i2c->xfer_len = len;
    i2c->buf = data;

    if (segment)
    {
        // Write the segment pointer with no stop.
        status = i2c_master_write(i2c, EDDC_SEGMENT_POINTER_ADDR, &segment, 1, false);

        if (status == STATUS_OK)
        {
            // Write the word offset with no stop.

            i2c->xfer_timeout = false;
            i2c->xfer_cmplt = false;
            i2c->xfer_len = 1;
            // Set byte_cnt to 1 (instead of 0) because there is an errata where
            // UCTXIFG is not set after sending the repeated start as a transmitter.
            i2c->byte_cnt = 1; 

            // Set slave address.
            HWREG8(i2c->base_addr + OFS_UCBxI2CSA) = slave_addr;

            // Send repeated start.
            HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTXSTT;

            // Write the TxBuf immediately because there is an errata where 
            // UCTXIFG is not set after sending a repeated start as a transmitter.
            //
            // Send the register offset.
            HWREG8(i2c->base_addr + OFS_UCBxTXBUF) = reg;

            while (!i2c->xfer_cmplt && !i2c->xfer_timeout)
            {
                _BIS_SR(LPM1_bits + GIE);    // Enter LPM with interrupt.
            }
        }
    }
    else
    {
        // Write the word offset with no stop.
        status = i2c_master_write(i2c, slave_addr, &reg, 1, false);
    }


    if (status == STATUS_OK)
    {
        status = i2c_master_read(i2c, slave_addr, data, len);
    }

    return status;
}


void i2c_master_init(i2c_master_t *i2c, uint32_t base_addr, uint32_t bus_clk, i2c_freq_t freq)
{
    uint16_t Prescaler;

    // Save the I2C Base Address.
    i2c->base_addr = base_addr;

    // Software reset of I2C.
    HWREG8(i2c->base_addr + OFS_UCBxCTL1) = UCSWRST;

    // Use SMCLK, I2C mode, sync mode.
    HWREG8(i2c->base_addr + OFS_UCBxCTL0) = UCMST + UCMODE_3 + UCSYNC;
    HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCSSEL__SMCLK;

    // Compute the prescaler. 
    CRIT("I2C: %s kbps\n", (freq == I2C_FREQ_400KHZ) ? "400" : "100"); 
    if (freq == I2C_FREQ_400KHZ)
    {
        Prescaler = (uint16_t )(bus_clk / 400000);
    }
    else /* 100KHz */
    {
        Prescaler = (uint16_t )(bus_clk / 100000);
    }

    // Set prescaler.
    HWREG16(i2c->base_addr + OFS_UCBxBR0) = Prescaler;

    // Clear interrupt flags.
    HWREG8(i2c->base_addr + OFS_UCBxIFG) = 0x00;

    // Disable I2C SW reset.
    HWREG8(i2c->base_addr + OFS_UCBxCTL1) &= ~UCSWRST;

    // Enable USCI Interrupts (TX, RX, NACK)
    HWREG8(i2c->base_addr + OFS_UCBxIE) = UCTXIE | UCRXIE | UCNACKIE;

    return;
}



bool i2c_master_isr(i2c_master_t *i2c)
{
    uint8_t flags;

    flags = HWREG8(i2c->base_addr + OFS_UCBxIFG);

    switch (flags)
    {
        case UCNACKIFG: /* NACK */
            i2c->nack = true;
            i2c->xfer_cmplt = true;
            // Send stop.
            HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTXSTP;
            HWREG8(i2c->base_addr + OFS_UCBxIFG) &= ~UCNACKIFG;
            break;

        case UCALIFG: /* Arbitration lost - multi-master mode only */
            break;

        case UCSTPIFG: /* STOP - slave mode only */
            break;

        case UCSTTIFG: /* START - slave mode only */
            break;

        case UCTXIFG: /* Tx */
            i2c->byte_cnt++;

            if (i2c->byte_cnt > i2c->xfer_len)
            {
                if (i2c->send_stop)
                {
                    // Last byte of the transfer, set STOP.
                    HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTXSTP;               
                }

                i2c->xfer_cmplt = true;
                HWREG8(i2c->base_addr + OFS_UCBxIFG) &= ~UCTXIFG;
            }
            else
            {
                HWREG8(i2c->base_addr + OFS_UCBxTXBUF) = i2c->buf[i2c->byte_cnt-1];
            }
            break;

        case UCRXIFG: /* Rx */
            i2c->buf[i2c->byte_cnt++] = HWREG16(i2c->base_addr + OFS_UCBxRXBUF);

            if (i2c->byte_cnt == i2c->xfer_len)
            {
                i2c->xfer_cmplt = true;
            }
            else if (i2c->byte_cnt == (i2c->xfer_len - 1))
            {
                // Last byte of the transfer.
                HWREG8(i2c->base_addr + OFS_UCBxCTL1) |= UCTXSTP;   
            }

            break;
    }

//	__bic_SR_register_on_exit(LPM1_bits); // Exit LPM

    // Return true to Wake from LPM.
    return true;
}
