/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include "smbus_phy.h"
#include "driverlib.h"
#include "smbus.h"
#include "smbus_nwk.h"
#include "tusb422_common.h"
#include <stdint.h>

#ifdef __MSP432P401R__
#include "i2c.h"
#else
#include "eusci_b_i2c.h"
#endif
#if (SMB_MANUAL_ACK_ENABLE == 1)
#include "dma.h"
#endif

#if (SMB_MANUAL_ACK_ENABLE == 1)
/* eUSCI doesn't support Manual ACK, a workaround needs to be implemented
 * The workaround uses DMA to stretch SCL low immediately after a byte is
 * received. The DMA ISR is attended and the application then decides if it
 * needs to ACK or NACK the byte. The following Hardware definitions are used
 * in order to use GPIOs and DMA:
 */
#define SMB_SWACK_I2CPORT_BASE  (__MSP430_BASEADDRESS_PORT1_R__)
#define SMB_SWACK_SDA_PIN       (BIT6)
#define SMB_SWACK_SCL_PIN       (BIT7)
#define SMB_SWACK_PxSEL_OFSET   (OFS_P1SEL1)    // P1SEL1
#define SMB_SWACK_DEFAULT_STATE (BIT6 | BIT4)
#define SWB_SWACK_DMA_SOURCE    (DMA_TRIGGERSOURCE_18)
const uint8_t u8DefaultDMA_State = SMB_SWACK_DEFAULT_STATE;
#endif

void SMBus_PHY_disable(SMBus *smbus)
{
    // Reset the eUSCI module
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCSWRST;
    smbus->ctrl.bits.phyEn = 0;      // Update control flag
}

void SMBus_PHY_enable(SMBus *smbus)
{
    if(smbus->ctrl.bits.master)
    {
        SMBus_PHY_masterEnable(smbus);
    }
    else
    {
        CRIT("Error: SMBus slave not supported!\n");
    }
}


void SMBus_PHY_masterEnable(SMBus *smbus)
{
    // Make sure the PHY is disabled
    SMBus_PHY_disable(smbus);

    // Write the Slave Address
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->I2COA0 = smbus->ownSlaveAddr + UCOAEN;

    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->IFG = 0x00;              // Clear all flags
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 &= ~UCSWRST;      // Start USCI I2C

    // Re-enable Interrupts since the Reset will clear them
    if(smbus->ctrl.bits.intEn == 1)
    {
        SMBus_masterEnableInt(smbus);
    }

    smbus->ctrl.bits.phyEn = 1;      // Set global flag
}

void SMBus_PHY_masterEnableInt(SMBus *smbus)
{
    // Clear all flags
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->IFG = 0x00;

    // Enable eUSCI Interrupts (Start, Stop, TX, RX, Timeout, NACK, Arbitration Lost)
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->IE = UCSTTIE | UCSTPIE | UCTXIE0 |
                        UCRXIE0 | UCCLTOIE |
                        UCNACKIE |
                        UCALIE;

    // Set global flag
    smbus->ctrl.bits.intEn = 1;
}

void SMBus_PHY_masterInit(SMBus *smbus,
                          uint32_t i2cAddr,
                          uint32_t busClk,
                          SMBus_Freq_t freq)
{
    SMBus_Phy *SMBusPHY = &smbus->phy;
    uint16_t Prescaler;

    // Set the I2C Base Address
    SMBusPHY->SMBus_Phy_i2cBase = i2cAddr;

    //Software reset of I2C
    EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->CTLW0 = UCSWRST;
    // Use SMCLK, I2C mode, sync mode, Multi-Master mode (BQ - disabling MM mode since it reduces I2C speed)
    EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->CTLW0 |= UCMST | /*UCMM |*/
                                                          UCSSEL__SMCLK |
                                                          UCMODE_3 + UCSYNC;
    // Select deglitch = 50ns, SW Acknownedge
    EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->CTLW1 = UCGLIT_0 | UCSWACK;
    // Enable Clock low timeout (150,000 MODCLK cycles = ~31ms)
    EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->CTLW1 |= UCCLTO_2;

    // Slave Address not initialized by default
    smbus->ownSlaveAddr = 0;

    // PreStop is cleared
    SMBusPHY->SMBus_Phy_stop = SMBus_Stop_No;

    CRIT("I2C: %s kbps\n", (freq == SMBus_Freq_400KHz) ? "400" : (freq == SMBus_Freq_1MHz) ? "1000" : "100"); 
    //Compute the prescaler. 
    if (freq == SMBus_Freq_400KHz)
    {
        Prescaler = (uint16_t )(busClk / 400000);
    }
    else if (freq == SMBus_Freq_1MHz)
    {
        Prescaler = (uint16_t )(busClk / 1000000);
    }
    else /* 100kHz */
    {
        Prescaler = (uint16_t )(busClk / 100000);
    }

    EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->BRW = Prescaler;

    // Enable the I2C module
    SMBus_PHY_masterEnable(smbus);

    // Enable NVIC interrupt.
    Interrupt_enableInterrupt(INT_EUSCIB0);
}

void SMBus_PHY_masterSendStop(SMBus *smbus)
{
    // Generate Stop condition if it hasn't been sent
    if(smbus->phy.SMBus_Phy_stop != SMBus_Stop_Sent)
    {
        EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCTXSTP;
        smbus->phy.SMBus_Phy_stop = SMBus_Stop_Sent;
    }
}

void SMBus_PHY_masterSendPreStop(SMBus *smbus)
{
    smbus->phy.SMBus_Phy_stop = SMBus_Stop_PreRead;
}

void SMBus_PHY_masterStartTx(SMBus *smbus,
                             uint8_t targetaddr,
                             SMBus_Stop stopFlag)
{
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->I2CSA = targetaddr;

    smbus->phy.SMBus_Phy_stop = SMBus_Stop_No;
    if(stopFlag == SMBus_Stop_Immediate)
    {
        EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCMST + UCTR + UCTXSTT + UCTXSTP;
        smbus->phy.SMBus_Phy_stop = SMBus_Stop_Sent;
    }
    else
    {
        EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCMST + UCTR + UCTXSTT;
    }
}

void SMBus_PHY_masterStartRx(SMBus *smbus,
                             uint8_t targetaddr,
                             SMBus_Stop stopFlag)
{
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->I2CSA = targetaddr;
    EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 &= ~UCTR;

    smbus->phy.SMBus_Phy_stop = SMBus_Stop_No;
    if(stopFlag == SMBus_Stop_Immediate)
    {
        EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCMST + UCTXSTT + UCTXSTP;
        smbus->phy.SMBus_Phy_stop = SMBus_Stop_Sent;
    }
    else
    {
        EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCMST + UCTXSTT;
    }

    if(stopFlag == SMBus_Stop_Addr)
    {
        // Wait for the Start to be sent
        while(EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 & UCTXSTT)
        {
            ;
        }
        EUSCI_B_CMSIS(smbus->phy.SMBus_Phy_i2cBase)->CTLW0 |= UCTXSTP;
        smbus->phy.SMBus_Phy_stop = SMBus_Stop_Sent;
    }

    if(stopFlag == SMBus_Stop_PreRead)
    {
        smbus->phy.SMBus_Phy_stop = SMBus_Stop_PreRead;
    }
}

SMBus_State SMBus_PHY_masterProcessInt(SMBus *smbus)
{
    uint8_t data;
    SMBus_State ret_state;
    SMBus_Phy *SMBusPHY = &smbus->phy;

    // Check for NACK
    
    if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCNACKIFG)
    {
        // Pass data to NWK layer
        ret_state = SMBus_NWK_masterProcessNACK(smbus);
        // Clear flag
        EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG &= ~UCNACKIFG;
    }
    else if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCALIFG)
    {
        // Arbitration lost
        // Clear start flag
        EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG &= ~UCALIFG;
    }     // STT condition
    else if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCSTTIFG)
    {
        // Start is detected when we get a packet as Slave
        // Clear start flag
        EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG &= ~UCSTTIFG;
    }     // STT condition
    // Check if there is a RX pending
    else if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCRXIFG0)
    {
        // Send stop before receiving
        if(SMBusPHY->SMBus_Phy_stop == SMBus_Stop_PreRead)
        {
            EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->CTLW0 |= UCTXSTP;
            SMBusPHY->SMBus_Phy_stop = SMBus_Stop_Sent;
        }
        // RXIFG0 is set when a byte is received from slave
        data = EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->RXBUF;
        // Pass data to NWK layer
        ret_state = SMBus_NWK_masterProcessRx(smbus, data);
    }
    else if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCTXIFG0)
    {
        // TXIFG0 is set when the TX buffer is empty and we need to send data to
        // slave
        ret_state = SMBus_NWK_masterProcessTx(smbus, &data);
        // Send the data
        EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->TXBUF = data;
    }     // TXIFG
    else if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCSTPIFG)
    {
        // Inform the NWK later
        ret_state = SMBus_NWK_masterProcessStop(smbus);
        // Clear flag
        EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG &= ~UCSTPIFG;
    }     // STP
    else if(EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG & UCCLTOIFG)
    {
        // This flag is set when a clock timeout is detected
        // Inform the NWK of the error
        ret_state = SMBus_NWK_masterProcessTimeout(smbus);
        // Clear flag
        EUSCI_B_CMSIS(SMBusPHY->SMBus_Phy_i2cBase)->IFG &= ~UCCLTOIFG;
    }

    return(ret_state);
}
