/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*! @file
 * 
 * This file contains the implementation of the UART.
 *
 * A circular print buffer is used to minimize execution delays caused by debug prints.  If the 
 * interface cannot keep up with the debug, the output will be corrupted.  To avoid this,
 * the print buffer may be disabled but at the cost of execution speed.
 *
 * The interface is configured for the following: 
 * - BAUD rate: 115,200
 * - Data bits: 8
 * - Stop bits: 1
 * - Parity: None.
 * - Flow control: None.
 *  
 */

#include "uart.h"
#include "tcpm_hal.h"
#include "uart_hal.h"
#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>

#define ENABLE_PRINT_BUFFER   1    // Set to 1 to enable circular UART debug print buffer.

//#define PRINT_BUFFER_SIZE        1024   // Size of circular print buffer in bytes.
//#define PRINT_BUFFER_SIZE_MASK   0x3FF  // Must correspond to the buffer size.

//#define PRINT_BUFFER_SIZE        2048   // Size of circular print buffer in bytes.
//#define PRINT_BUFFER_SIZE_MASK   0x7FF  // Must correspond to the buffer size.

#if defined __MSP432P401R__ || defined __MSP430F5528__

#define PRINT_BUFFER_SIZE        4096   // Size of circular print buffer in bytes.
#define PRINT_BUFFER_SIZE_MASK   0xFFF  // Must correspond to the buffer size.

#else

#define PRINT_BUFFER_SIZE        128    // Size of circular print buffer in bytes.
#define PRINT_BUFFER_SIZE_MASK   0x1FF  // Must correspond to the buffer size.

#endif

#if ENABLE_PRINT_BUFFER & (DEBUG_LEVEL >= 1)

static char print_buffer[PRINT_BUFFER_SIZE];

static volatile uint_fast16_t write_offset;
static volatile uint_fast16_t read_offset;

static volatile bool data_ready;
static volatile bool tx_now;

static void uart_tx(void);

#endif

/*****************************************************************************
 * Function: uart_init
 *************************************************************************//**
 * This function initializes the UART module.
 *
 * @param None.
 *                    
 * @retval None.
 *
 ****************************************************************************** 
 */

void uart_init(void)
{
#if ENABLE_PRINT_BUFFER & (DEBUG_LEVEL >= 1)
    write_offset = 0;
    read_offset = 0;

    data_ready = false;
    tx_now = false;    

    HAL_UART_RegisterTxBufEmptyCallback(uart_tx);
#endif

    HAL_UART_Init();

    return;
}


#if DEBUG_LEVEL >= 1

    #if ENABLE_PRINT_BUFFER

/*****************************************************************************
 * Function: uart_tx
 *************************************************************************//**
 * This function takes data from the circular print buffer and writes it to the
 * UART Tx data buffer.
 *
 * @param None.
 *                    
 * @retval None.
 *
 ****************************************************************************** 
 */

static void uart_tx(void)
{
    uint_fast16_t ro = read_offset;

    if (ro == write_offset)
    {
        // Clear data ready flag if we have output all buffered data.
        data_ready = false;
    }

    if (data_ready)
    {
        // Write character to Tx Data buffer.
        HAL_UART_TxByte(print_buffer[read_offset++]);

        // Handle wrap around.
        read_offset &= PRINT_BUFFER_SIZE_MASK;
    }
    else
    {
        tx_now = true;
    }

    return;
}


/*****************************************************************************
 * Function: putchar
 *************************************************************************//**
 * This function writes data to the circular print buffer and starts UART Tx
 * if necessary.
 *
 * @param[in] bData byte data to be output.
 *                    
 * @retval None.
 *
 ****************************************************************************** 
 */

static void putchar(char bData)
{
    uint_fast16_t ro;
//    HAL_UART_EnableTxInt(false);

    // Store character in print buffer.
    print_buffer[write_offset++] = bData;

    // Handle wrap around.
    write_offset &= PRINT_BUFFER_SIZE_MASK;

    ro = read_offset;
    
    // Check for overwrite of buffered data.
    if (write_offset == ro)
    {
        read_offset++;

        // Handle wrap around.
        read_offset &= PRINT_BUFFER_SIZE_MASK;
    }
    
    // Check for line feed escape character.
    if (bData == '\n')
    {
        // Add carriage return.
        print_buffer[write_offset++] = '\r';

        // Handle wrap around.
        write_offset &= PRINT_BUFFER_SIZE_MASK;

        ro = read_offset;
        
        // Check for overwrite of buffered data.
        if (write_offset == ro)
        {
            read_offset++;

            // Handle wrap around.
            read_offset &= PRINT_BUFFER_SIZE_MASK;
        }
    }

    // Set data ready flag.
    data_ready = true;

    if (tx_now)
    {
        HAL_UART_SetTxIFG();
        tx_now = false;
    }

//    HAL_UART_EnableTxInt(true);
    
    return;
}


    #else


static void putchar(char bData)
{
    // Write character to Tx Data buffer.
    HAL_UART_TxByte(bData);

    // Write carriage return if new line was sent.
    if (bData == '\n')
    {
        HAL_UART_TxByte('\r');
    }

    return;
}


    #endif


    #define DECIMAL_BASE    10
    #define HEX_BASE        16

/*****************************************************************************
 * Function: print_strings
 *************************************************************************//**
 * This function sends the characters of a string to the putchar function for 
 * printing them. 
 *
 * @param[in] print_string pointer to character array.
 * @param[in] str_width minimum number of characters to be printed.
 * @param[in] zero_align flag to indicate padding the string with zeros.
 * @param[in] string_flag flag for printing a string (for the 's' format)
 *
 ****************************************************************************** 
 */

static void print_strings(char *print_string, uint_fast8_t str_width, bool zero_align, bool string_flag)
{
    uint_fast16_t string_length = 0;
    int diff;

    while (*print_string != '\0')
    {
        if (string_flag == true)
        {
            putchar(*print_string);
        }
        else
        {
            string_length++;
        }

        print_string++;
    }

    if (!string_flag)
    {
        diff = str_width - string_length;

        while (diff > 0)
        {
            putchar((zero_align) ? '0' : ' ');
            diff--;
        }

        while (string_length > 0)
        {
            print_string--;
            putchar(*print_string);
            string_length--;
        }
    }

    return;
}

/*****************************************************************************
 * Function: number_to_string
 *************************************************************************//**
 * This function converts a number to a string
 *
 * @param[in] arg number to be converted into a string.
 * @param[in] base base of the number (Dec or Hex).
 * @param[in] sign flag to indicate converting a signed number. 
 * @param[in] print_string pointer to the converted character array.
 *                    
 * @retval None.
 *
 ****************************************************************************** 
 */
void number_to_string(int32_t arg, uint32_t base, bool sign, char *print_string)
{
    uint32_t quotient = 0;
    uint32_t reminder = 0;
    uint32_t dividend = (uint32_t)arg;  

    if (arg == 0)
    {
        *print_string = '0';
        print_string++;
        *print_string = '\0';
    }
    else
    {
        if ((arg < 0) && (sign == true))
        {
            dividend = -arg;
        }

        while (dividend)
        {
            if (base == HEX_BASE)
            {
                quotient = dividend >> 4;
                reminder = dividend - (quotient << 4);
            }
            else
            {
                quotient = dividend / base;
                reminder = dividend - (quotient * base);
            }

            if (reminder > 9)
                reminder += 39; /*Adding offset for getting the lower cases code*/

            reminder += '0';

            *(print_string++) = reminder;

            if (base == HEX_BASE)
            {
                dividend = dividend >> 4;
            }
            else
            {
                dividend /= base;
            }
        }

        if ((arg < 0) && (sign == true))
        {
            *(print_string++) = '-';
        }

        *(print_string++) = '\0';
    }
}



/*****************************************************************************
 * Function: kprintf
 *************************************************************************//**
 * This function outputs to the UART port a sequence of data formatted as the 
 * format_str argument specifies.
 *
 * Supported format specifiers: 'd', 'ld', 'u', 'ul', 's', 'x', 'lx'.
 * Zero padding and field width up to 9 places are also supported.
 *
 * @param[in] format_str string that contains the text to sent to the UART port.
 * @param[in] ... subsequent arguments to be formatted as specified by the format parameters.
 *  
 ****************************************************************************** 
 */

void kprintf(const char *format_str, va_list va)
{
    bool zero_align;
    bool lng;
    char print_string[12];
    uint_fast8_t str_width;

    while (*format_str != '\0')
    {
        if (*format_str != '%')
        {
            putchar(*format_str);
            format_str++;
            continue;
        }

        str_width = 0;
        lng = false;
        zero_align = false;
        format_str++;

        if (*format_str == '0')
        {
            zero_align = true;
            format_str++;

            str_width = *format_str - '0';
            format_str++;
        }

        if (*format_str == 'l')
        {
            lng = true;
            format_str++;
        }

        switch (*format_str)
        {
            case '%':
                putchar('%');
                break;

            case 'd':
                if (lng)
                {
                    number_to_string(va_arg(va, int32_t), DECIMAL_BASE, true, print_string);
                }
                else
                {
                    number_to_string((int16_t)va_arg(va, int), DECIMAL_BASE, true, print_string);
                }

                print_strings(print_string, str_width, zero_align, false);
                break;

            case 'u':
                if (lng)
                {
                    number_to_string(va_arg(va, uint32_t), DECIMAL_BASE, false, print_string);
                }
                else
                {
                    number_to_string((uint16_t)va_arg(va, int), DECIMAL_BASE, false, print_string);
                }

                print_strings(print_string, str_width, zero_align, false);
                break;

            case 'x':
                if (lng)
                {
                    number_to_string(va_arg(va, uint32_t), HEX_BASE, false, print_string);
                }
                else
                {
                    number_to_string((uint16_t)va_arg(va, int), HEX_BASE, false, print_string);
                }

                print_strings(print_string, str_width, zero_align, false);
                break;

            case 's':
                print_strings(va_arg(va, char*), str_width, zero_align, true);
                break;

            default:
                // Do nothing.
                break;
        }

        format_str++;
    }

    return;
}

void printk(const char *fmt, ...)
{
    va_list va;
    va_start(va,fmt);
    kprintf(fmt,va);
    va_end(va);
}

#endif /* DEBUG_LEVEL >= 1 */

