/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016-2017 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "usb_pd_policy_engine.h"
#include "tcpm.h"
#include "tusb422.h"
#include "tusb422_common.h"
#include "usb_pd.h"
#ifdef CONFIG_TUSB422_PAL
	#include "usb_pd_pal.h"
#endif
#include "usb_pd_protocol.h"
#ifdef CONFIG_TUSB422
	#include <linux/string.h>
#else
	#include <string.h>
#endif
#ifdef CONFIG_DUAL_ROLE_USB_INTF
	#include "tusb422_linux_dual_role.h"
	#include <linux/device.h>
	#include <linux/usb/class-dual-role.h>
#endif
#if defined HDMI_DONGLE || defined DP_DONGLE
#include "usb.h"
#endif
#ifdef HDMI_DONGLE
#include "tcpci.h"
#endif
#ifdef DP_DONGLE
#include "tusb544.h"
#endif

#include "pb.h"

/* PD Counters Max */
#define N_CAPS_COUNT        50
#define N_HARD_RESET_COUNT  2
#define N_DISCOVER_IDENTITY_COUNT 20 /* Due to send source cap timeout, the counter will only reach 16 if no GoodCRC received or 5 if VDM sender response times out repeatly */ 
#define N_BUSY_COUNT 1  /* 5 per PD3.0 spec but use 1 to meet USB-C v1.2 requirement for mode entry */

/* PD Timeout Values */
#define T_NO_RESPONSE_MS            5000   /* 4.5 - 5.5 s */
#define T_SENDER_RESPONSE_MS          25   /*  24 - 30 ms */
#define T_VCONN_STABLE_MS             50   /* 50ms max from Type-C spec */
#define T_SWAP_SOURCE_START_MS        40   /*  20 - ? ms*/
#define T_TYPEC_SEND_SOURCE_CAP_MS   150   /* 100 - 200 ms */
#if defined HDMI_DONGLE || defined DP_DONGLE
#define T_TYPEC_SINK_WAIT_CAP_MS     320   /* 310 - 620 ms */
#else
#define T_TYPEC_SINK_WAIT_CAP_MS     500   /* 310 - 620 ms */
#endif
#define T_PS_HARD_RESET_MS            25   /* 25 - 35 ms */
#define T_PS_TRANSITION_MS           500   /* 450 - 550 ms */
#define T_SRC_RECOVER_MS             700   /* 660 - 1000 ms */
#define T_SRC_TRANSITION_MS           25   /* 25 - 35 ms */
#define T_SINK_REQUEST_MS            200   /* 100 - ? ms */
#define T_BIST_CONT_MODE_MS           50   /* 30 - 60 ms */
//#define T_SRC_TURN_ON_MS             275   /* tSrcTurnOn: 0 - 275ms */
//#define T_SRC_SETTLE_MS              275   /* tSrcSettle: 0 - 275ms */
#define T_PS_SOURCE_OFF_MS           880   /* 750 - 920 ms */
#define T_PS_SOURCE_ON_MS            440   /* 390 - 480 ms */
#define T_VCONN_SOURCE_ON_MS         100   /* ? - 100 ms */
#define T_SINK_TX_MS                  18   /* 16 - 20 ms */
#define T_SINK_VBUS_OFF_MS           685   /* (tPSHardReset + tSafe0V) = 35 + 650ms */
#define T_SINK_VBUS_RECOVERY_MS     1275   /* (tSrcRecover + tSrcTurnOn) = 1000 + 275ms */

#define T_CHUNKING_NOT_SUPPORTED_MS   40   /* 40 - 50 ms */
#define T_PPS_REQUEST_MS           10000   /*  ? - 10 sec */
#define T_PPS_TIMEOUT_MS           15000   /* 12 - 15 sec */
#define T_VBUS_5V_STABLIZE_MS         20   /* delay for VBUS to stablize after vSafe5V-min is reached */

#ifdef ENABLE_VDM_SUPPORT
#define T_DISCOVER_IDENTITY_MS        45   /* 40 - 50 ms */
#define T_VDM_BUSY_MS                100   /* 50 - ? ms */
//#define T_VDM_ENTER_MODE_MS           25   /* ?  - 25ms */
//#define T_VDM_EXIT_MODE_MS            25   /* ?  - 25ms */
#define T_VDM_SENDER_RESPONSE_MS      25   /* 24 - 30ms */
#define T_VDM_WAIT_MODE_ENTRY_MS      45   /* 40 - 50 ms */
#define T_VDM_WAIT_MODE_EXIT_MS       45   /* 40 - 50 ms */
#define T_AME_TIMEOUT_MS             900   /*  ? - 1 sec */
#endif

#define CABLE_DEFAULT_MAX_VOLTAGE_MV  20000
#define CABLE_DEFAULT_MAX_CURRENT_MA  3000

#ifdef CONFIG_DUAL_ROLE_USB_INTF
	#define DUAL_ROLE_CHANGED()  dual_role_instance_changed(tusb422_dual_role_phy)
#else
	#define DUAL_ROLE_CHANGED()  {}
#endif

extern power_bank_t pb;

usb_pd_port_t pd[NUM_TCPC_DEVICES];

extern void usb_pd_pm_evaluate_src_caps(unsigned int port);
extern usb_pd_port_config_t* usb_pd_pm_get_config(unsigned int port);
extern void build_rdo(unsigned int port);
extern uint32_t get_data_object(uint8_t *obj_data);
extern void build_src_caps(unsigned int port);
extern void build_snk_caps(unsigned int port);

extern void pb_pd_cc_cb(bool,bool);

#ifdef ENABLE_VDM_SUPPORT
static void usb_pd_pe_vdm_handler(usb_pd_port_t *dev);
#endif 

#ifdef ENABLE_USB_PD_REV3_SUPPORT
static void timeout_pps_timeout(unsigned int port);
#endif

const char * const pdstate2string[PE_NUM_STATES] =
{
	"UNATTACHED",

	/* Source */
	"SRC_STARTUP",
	"SRC_DISCOVERY",
	"SRC_SEND_CAPS",
	"SRC_NEGOTIATE_CAPABILITY",
	"SRC_TRANSITION_SUPPLY",
	"SRC_TRANSITION_SUPPLY_EXIT",
	"SRC_READY",
	"SRC_DISABLED",
	"SRC_CAPABILITY_RESPONSE",
	"SRC_HARD_RESET",
	"SRC_HARD_RESET_RECEIVED",
	"SRC_TRANSITION_TO_DEFAULT",
	"SRC_GET_SINK_CAP",
	"SRC_WAIT_NEW_CAPS",
	"SRC_SEND_SOFT_RESET",
	"SRC_SOFT_RESET",
	"SRC_PING",
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	"SRC_SEND_SOURCE_ALERT",
	"SRC_SINK_ALERT_RECEIVED",
	"SRC_GIVE_SOURCE_CAP_EXT",
	"SRC_GIVE_SOURCE_STATUS",
	"SRC_GET_SINK_STATUS",
#endif

	/* Sink */
	"SNK_STARTUP",
	"SNK_DISCOVERY",
	"SNK_WAIT_FOR_CAPS",
	"SNK_EVALUATE_CAPABILITY",
	"SNK_SELECT_CAPABILITY",
	"SNK_TRANSITION_SINK",
	"SNK_READY",
	"SNK_HARD_RESET",
	"SNK_TRANSITION_TO_DEFAULT",
	"SNK_GIVE_SINK_CAP",
	"SNK_GET_SOURCE_CAP",
	"SNK_SEND_SOFT_RESET",
	"SNK_SOFT_RESET",
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	"SNK_SOURCE_ALERT_RECEIVED",
	"SNK_SEND_SINK_ALERT",
	"SNK_GET_SOURCE_CAP_EXT",
	"SNK_GET_SOURCE_STATUS",
	"SNK_GIVE_SINK_STATUS",
#endif

	/* Sink or Source */
	"SEND_REJECT",
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	"SEND_NOT_SUPPORTED",
#endif

	/* Dual-role */
	"DR_SRC_GIVE_SINK_CAP",
	"DR_SNK_GIVE_SOURCE_CAP",
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	"DR_SRC_GET_SOURCE_CAP_EXT",
	"DR_SNK_GIVE_SOURCE_CAP_EXT",
#endif

	/* BIST */
	"BIST_CARRIER_MODE",
	"BIST_TEST_MODE",

	/* Error Recovery */
	"ERROR_RECOVERY",

	/* Data Role Swap */
	"DRS_SEND_SWAP",
	"DRS_EVALUATE_SWAP",
	"DRS_ACCEPT_SWAP",
	"DRS_CHANGE_ROLE",

	/* Power Role Swap */
	"PRS_SEND_SWAP",
	"PRS_EVALUATE_SWAP",
	"PRS_WAIT_SWAP",
	"PRS_ACCEPT_SWAP",
	"PRS_TRANSITION_TO_OFF",
	"PRS_ASSERT_RD",
	"PRS_WAIT_SOURCE_ON",
	"PRS_ASSERT_RP",
	"PRS_SOURCE_ON",
	"PRS_SOURCE_ON_EXIT",

	/* VCONN Swap */
	"VCS_SEND_SWAP",
	"VCS_EVALUATE_SWAP",
	"VCS_ACCEPT_SWAP",
	"VCS_WAIT_FOR_VCONN",
	"VCS_TURN_OFF_VCONN",
	"VCS_TURN_ON_VCONN",
	"VCS_SEND_PS_RDY",

#ifdef ENABLE_VDM_SUPPORT
	/* VDM */
	"SRC_VDM_IDENTITY_REQUEST",
	"INIT_PORT_VDM_IDENTITY_REQUEST",
	"INIT_VDM_SVIDS_REQUEST",
	"INIT_VDM_MODES_REQUEST",
	"INIT_VDM_ATTENTION_REQUEST",
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	"INIT_VDM_DP_STATUS_UPDATE",
	"INIT_VDM_DP_CONFIG",
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	"INIT_VDM_HDMI_STATUS_UPDATE",
//	"INIT_VDM_HDMI_CEC_WRITE",
//	"INIT_VDM_HDMI_CEC_READ",
	"INIT_VDM_HDMI_DDC_WRITE",
	"INIT_VDM_HDMI_DDC_READ",
#endif

	"RESP_VDM_NAK",
	"RESP_VDM_GET_IDENTITY",
	"RESP_VDM_GET_SVIDS",
	"RESP_VDM_GET_MODES",
	"RCV_VDM_ATTENTION_REQUEST",
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	"RESP_VDM_DP_STATUS_UPDATE",
	"RESP_VDM_DP_CONFIG",
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	"RESP_VDM_HDMI_STATUS_UPDATE",
//	"RESP_VDM_HDMI_CEC_WRITE",
//	"RESP_VDM_HDMI_CEC_READ",
	"RESP_VDM_HDMI_DDC_WRITE",
	"RESP_VDM_HDMI_DDC_READ",
#endif

	"DFP_VDM_MODE_ENTRY_REQUEST",
	"DFP_VDM_MODE_EXIT_REQUEST",

	"UFP_VDM_EVAL_MODE_ENTRY",
	"UFP_VDM_MODE_EXIT",
#endif

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	"GET_BATTERY_CAP",
	"GIVE_BATTERY_CAP",
	"GET_BATTERY_STATUS",
	"GIVE_BATTERY_STATUS",
	"GET_MANUFACTURER_INFO",
	"GIVE_MANUFACTURER_INFO",
	"SEND_SECURITY_REQUEST",
	"SEND_SECURITY_RESPONSE",
	"SECURITY_RESP_RECEIVED",
	"SEND_FW_UPDATE_REQUEST",
	"SEND_FW_UPDATE_RESPONSE",
	"FW_UPDATE_RESP_RECEIVED",
#endif

};


#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	#define DP_PORT_CAPABILITY(vdo)  ((vdo) & 0x3)
	#define UFP_D_CAPABLE_BIT (1 << 0)
	#define DFP_D_CAPABLE_BIT (1 << 1)
	#define RECEPTACLE_INDICATION_BIT (1 << 6)
	#define DFP_D_PIN_ASSIGNMENTS(vdo)  (((vdo) >> 8) & 0xFF)
	#define UFP_D_PIN_ASSIGNMENTS(vdo)  (((vdo) >> 16) & 0xFF)
#endif

#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	#define HDMI_CABLE_ADAPTER_BIT 0x1
#endif

#ifndef CONFIG_TUSB422
#if DEBUG_LEVEL >= 1
void pe_debug_state_history(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];
	uint8_t i;

	PRINT("\nPolicy Engine State History:\n");
	for (i = 0; i < PD_STATE_HISTORY_LEN; i++)
	{
		if (dev->state[i] < PE_NUM_STATES)
		{
			PRINT("%s[%u] %s\n", (&dev->state[i] == dev->current_state) ? "->" : "  ", i, pdstate2string[dev->state[i]]);
		}
	}

	return;
}
#endif
#endif

static void pe_set_state(usb_pd_port_t *dev, usb_pd_pe_state_t new_state)
{
	CRIT("PE_%s\n", pdstate2string[new_state]);

	dev->state[dev->state_idx] = new_state;
	dev->current_state = &dev->state[dev->state_idx];
	dev->state_idx++;
	dev->state_idx &= PD_STATE_INDEX_MASK;
	dev->state_change = true;

	return;
}

static void timeout_no_response(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	CRIT("%s\n", __func__);

	if (dev->hard_reset_cnt > N_HARD_RESET_COUNT)
	{
		if (!dev->pd_connected_since_attach)
		{
			if ((*dev->current_state == PE_SRC_SEND_CAPS) ||
				((*dev->current_state == PE_SRC_DISCOVERY) && (dev->caps_cnt > N_CAPS_COUNT)))
			{
				pe_set_state(dev, PE_SRC_DISABLED);
			}
		}
		else /* previously PD connected */
		{
			pe_set_state(dev, PE_ERROR_RECOVERY);
		}
	}

	return;
}

/* No Response timer is for source only */
static void timer_start_no_response(usb_pd_port_t *dev)
{
	INFO("%s\n", __func__);

	tusb422_lfo_timer_start(&dev->timer2, T_NO_RESPONSE_MS, timeout_no_response);

	return;
}

static void timer_cancel_no_response(usb_pd_port_t *dev)
{
	INFO("%s\n", __func__);

	if (dev->timer2.function == timeout_no_response)
	{
		tusb422_lfo_timer_cancel(&dev->timer2);
	}

	return;
}


usb_pd_port_t * usb_pd_pe_get_device(unsigned int port)
{
	return &pd[port];
}

#ifdef ENABLE_VDM_SUPPORT
#if defined HDMI_DONGLE || defined DP_DONGLE
static void timeout_alt_mode_entry(unsigned int port)
{
	CRIT("Alt-Mode entry timeout!\n");

	// Enable billboard.
	USB_setup(TRUE, TRUE);    // Init USB & events; if a host is present, connect.

	return;
}
#endif
#endif

extern void pb_pd_explicit_contract_cb(bool,bool);

static void usb_pd_pe_handle_unattach(usb_pd_port_t *dev)
{
	dev->swap_source_start = false;
	dev->start_src_no_response_timer = false;
	dev->pd_connected = false;
	dev->pd_connected_since_attach = false;
	dev->explicit_contract = false;
	dev->vbus_present = false;
	dev->power_role_swap_in_progress = false;
	dev->active_alt_modes = 0;
	dev->active_cable = false;
	dev->request_goto_min = false;
	dev->hard_reset_cnt = 0;
	dev->vconn_source = false;
	dev->hard_reset_complete = false;

	pb_pd_explicit_contract_cb(true,false);
//	pb.otg_active = false;

	// Set defaults for cable voltage and current limits.
	dev->cable_max_voltage_mv = CABLE_DEFAULT_MAX_VOLTAGE_MV;
	dev->cable_max_current_ma = CABLE_DEFAULT_MAX_CURRENT_MA;

	dev->cable_plug_comm_required = false;
	dev->get_sink_caps_complete = false;
	dev->error_recovery_pending = false;
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	dev->p2p_spec_rev = PD_REV30;
	dev->p2c_spec_rev = PD_REV30;
#else
	dev->p2p_spec_rev = PD_REV20;
	dev->p2c_spec_rev = PD_REV20;
#endif

#ifdef ENABLE_VDM_SUPPORT
	#ifdef ENABLE_USB_PD_REV3_SUPPORT
		dev->p2p_vdm_version = STRUCT_VDM_VER_2P0;
		dev->p2c_vdm_version = STRUCT_VDM_VER_2P0;
	#else
		dev->p2p_vdm_version = STRUCT_VDM_VER_1P0;
		dev->p2c_vdm_version = STRUCT_VDM_VER_1P0;
	#endif

	dev->port_partner_id_req_complete = false;
	dev->discover_identity_cnt = 0;
	dev->send_cable_soft_reset = false;
	dev->cable_id_req_complete = false;
	dev->send_src_cap_timeout = false;
	dev->vdm_in_progress = false;
	dev->vdm_initiator = false;
	dev->matched_alt_modes = 0;
	dev->vdm_pending = false;
	dev->mode_exit_pending = false;
	dev->attention_pending = false;

	#ifdef ENABLE_DP_ALT_MODE_SUPPORT
		dev->displayport_alt_mode_configured = false;
		dev->displayport_config = CONFIG_FOR_USB;
		dev->displayport_status = 0;
		dev->displayport_remote_status = 0;
		dev->displayport_matched_pin_assignments = 0;
		dev->displayport_lane_cnt = 0;
		dev->hpd_in_changed = false;
#ifdef DP_DONGLE
		dev->redriver_configured = false;

		// Check HPD in case UFP_D was attached before Type-C.
		dev->hpd_in_previous_value = tcpm_get_hpd_in(0);
#else
		// Assume this a DFP_D port and deassert HPD.
		tcpm_hpd_out_control(0, HPD_LOW);
#endif
	#elif defined HDMI_DONGLE
		dev->hpd_in_previous_value = 0;
		dev->hpd_in_changed = false;
		dev->hdmi_initial_status_update_complete = false;
	#endif
#endif

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	dev->rch_state = RCH_WAIT_FOR_MSG;
	dev->tch_state = TCH_WAIT_FOR_MSG_REQ;
#endif

	return;
}

void usb_pd_pe_connection_state_change_handler(unsigned int port, tcpc_state_t state)
{
	usb_pd_port_t *dev = &pd[port];
	usb_pd_port_config_t *config = usb_pd_pm_get_config(port);

	switch (state)
	{
		case TCPC_STATE_UNATTACHED_SRC:
		case TCPC_STATE_UNATTACHED_SNK:
			timer_cancel(&dev->timer);
			tusb422_lfo_timer_cancel(&dev->timer2);
			usb_pd_pe_handle_unattach(dev);
			pe_set_state(dev, PE_UNATTACHED);
			pb_pd_cc_cb(false,false);
			if (!(dev->tcpc_dev->flags & TC_FLAGS_TEMP_ROLE))
			{
				DUAL_ROLE_CHANGED();
			}
			break;

		case TCPC_STATE_ATTACHED_SRC:
			dev->data_role = PD_DATA_ROLE_DFP;
			dev->power_role = PD_PWR_ROLE_SRC;
			dev->vconn_source = true;
			pe_set_state(dev, PE_SRC_STARTUP);
			break;

		case TCPC_STATE_ATTACHED_SNK:
			dev->data_role = PD_DATA_ROLE_UFP;
			dev->power_role = PD_PWR_ROLE_SNK;
			dev->vbus_present = true;
			if (config->ufp_alt_mode_entry_timeout_enable)
			{
#if defined HDMI_DONGLE || defined DP_DONGLE
				tusb422_lfo_timer_start(&dev->timer2, T_AME_TIMEOUT_MS, timeout_alt_mode_entry);
#endif
			}
			pe_set_state(dev, PE_SNK_STARTUP);
			break;

		case TCPC_STATE_ORIENTED_DEBUG_ACC_SRC:
		case TCPC_STATE_DEBUG_ACC_SNK:
			break;

		case TCPC_STATE_UNORIENTED_DEBUG_ACC_SRC:
		case TCPC_STATE_AUDIO_ACC:
			// Do nothing.
			break;

		default:
			break;
	}

	return;
}

static void usb_pd_pe_tx_data_msg(unsigned int port, msg_hdr_data_msg_type_t msg_type, tcpc_transmit_t sop_type)
{
	usb_pd_port_t *dev = &pd[port];
	usb_pd_port_config_t *config = usb_pd_pm_get_config(port);
	uint8_t *payload_ptr = &dev->msg_buf[3];
	uint8_t ndo = 0;
	uint32_t *pdo;
	uint8_t pdo_idx;

	if ((msg_type == DATA_MSG_TYPE_SRC_CAPS) ||
		(msg_type == DATA_MSG_TYPE_SNK_CAPS))
	{
		if (msg_type == DATA_MSG_TYPE_SRC_CAPS)
		{
			build_src_caps(port);
			pdo = dev->src_pdo;
			ndo = config->num_src_pdos;
		}
		else /* SNK CAPS */
		{
			build_snk_caps(port);
			pdo = dev->snk_pdo;
			ndo = config->num_snk_pdos;
		}

		for (pdo_idx = 0; pdo_idx < ndo; pdo_idx++)
		{
			*payload_ptr++ = (uint8_t)(pdo[pdo_idx] & 0xFF);
			*payload_ptr++ = (uint8_t)((pdo[pdo_idx] & 0xFF00) >> 8);
			*payload_ptr++ = (uint8_t)((pdo[pdo_idx] & 0xFF0000) >> 16);
			*payload_ptr++ = (uint8_t)((pdo[pdo_idx] & 0xFF000000) >> 24);
		}
	}
	else if (msg_type == DATA_MSG_TYPE_REQUEST)
	{
		ndo = 1;

		payload_ptr[0] = (uint8_t)(dev->rdo & 0xFF);
		payload_ptr[1] = (uint8_t)((dev->rdo & 0xFF00) >> 8);
		payload_ptr[2] = (uint8_t)((dev->rdo & 0xFF0000) >> 16);
		payload_ptr[3] = (uint8_t)((dev->rdo & 0xFF000000) >> 24);
	}
	else
	{
		CRIT("%s: msg_type %u not supported.\n", __func__, msg_type);
	}

	if (ndo > 0)
	{
		usb_pd_prl_tx_data_msg(port, msg_type, sop_type, ndo);
	}

	return;
}

static void pe_send_accept_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_ACCEPT, TCPC_TX_SOP);
	return;
}

static void pe_send_reject_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_REJECT, TCPC_TX_SOP);
	return;
}

static void pe_send_wait_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_WAIT, TCPC_TX_SOP);
	return;
}

static void pe_send_soft_reset_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_SOFT_RESET, TCPC_TX_SOP);
	return;
}

#ifdef ENABLE_USB_PD_REV3_SUPPORT

static void pe_send_not_supported_entry(usb_pd_port_t *dev)
{
	// BQ - add option to send SOP'/SOP".
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_NOT_SUPPORTED, TCPC_TX_SOP);
	return;
}

static void timeout_sink_tx(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	dev->state_change = true;
	return;
}

static void usb_pd_pe_check_sink_tx_ok(usb_pd_port_t *dev)
{
	if ((dev->p2p_spec_rev > PD_REV20) &&
		(dev->power_role == PD_PWR_ROLE_SRC) &&
		dev->state_change && dev->sink_tx_ok)
	{
		// Set Rp value to 1.5A (SinkTxNG) for collision avoidance.
		tcpm_set_rp_value(dev->port, RP_MEDIUM_CURRENT);
		dev->sink_tx_ok = false;

		// Temporarily suspend the state change until tSinkTx times out.
		dev->state_change = false;
		timer_start(&dev->timer, T_SINK_TX_MS, timeout_sink_tx);
	}

	return;
}

#endif

typedef struct
{
	usb_pd_pe_state_t ready;
	usb_pd_pe_state_t hard_reset;
	usb_pd_pe_state_t send_soft_reset;
} unhandled_rx_states_t;

static void usb_pd_pe_unhandled_rx_msg(usb_pd_port_t *dev, bool unrecognized_or_unsupported)
{
	const unhandled_rx_states_t rx_handler_state[2] = 
	{
		/* Index 0 = SNK, Index 1 = SRC */
		{PE_SNK_READY, PE_SNK_HARD_RESET, PE_SNK_SEND_SOFT_RESET},
		{PE_SRC_READY, PE_SRC_HARD_RESET, PE_SRC_SEND_SOFT_RESET}
	};

	const unhandled_rx_states_t * pe_state = &rx_handler_state[dev->power_role];

	if ((*dev->current_state == PE_SNK_TRANSITION_SINK) ||
		(*dev->current_state == PE_SRC_NEGOTIATE_CAPABILITY) ||
		(*dev->current_state == PE_SRC_TRANSITION_SUPPLY))
	{
		// Issue hard reset per USB-PD spec.
		pe_set_state(dev, pe_state->hard_reset);
	}
	else if (*dev->current_state == pe_state->ready)
	{
		if (unrecognized_or_unsupported)
		{
			if (dev->p2p_spec_rev < PD_REV30)
			{
				pe_set_state(dev, PE_SEND_REJECT);
			}
#ifdef ENABLE_USB_PD_REV3_SUPPORT
			else
			{
				pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			}
#endif
		}
		else /* Unexpected */
		{
			pe_set_state(dev, pe_state->send_soft_reset);
		}
	}
	else if (dev->non_interruptable_ams)
	{
		// USB Type-C Error Recovery will be triggered externally if a protocol 
		// error occurs during a power role swap.
		if (!dev->power_role_swap_in_progress)
		{
			if (dev->explicit_contract)
			{
				pe_set_state(dev, pe_state->hard_reset);
			}
			else /* not power transitioned */
			{
				pe_set_state(dev, pe_state->send_soft_reset);
			}
		}
	}
	else /* Interruptible AMS */
	{
		// Return to PE_SNK_READY or PE_SRC_READY and process msg.
		pe_set_state(dev, pe_state->ready);
		dev->state_change = false;
		usb_pd_pe_notify(dev->port, PRL_ALERT_MSG_RECEIVED);
	}

	return;
}

#ifdef ENABLE_VDM_SUPPORT
static void usb_pd_pe_handle_vdm_interruption(usb_pd_port_t *dev)
{
	// Stop any VDM timer running. (e.g. VDMResponseTimer, VDMModeEntryTimer, VDMModeExitTimer)
	timer_cancel(&dev->timer);

	// If interrupted while initiating a VDM request, store it as pending to
	// retry next time PE_SRC_READY or PE_SNK_READY is reached.
	if (dev->vdm_initiator)
	{
		dev->vdm_initiator = false;
		dev->vdm_pending_state = *dev->current_state;
		dev->vdm_pending = true;
	}

	// VDM is being interrupted.  Return to Ready state.
	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_READY);
	}
	else /* SRC */
	{
		pe_set_state(dev, PE_SRC_READY);
	}

	dev->state_change = false;
	dev->vdm_in_progress = false;
}
#endif

static void timeout_sender_response(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	CRIT("%s\n", __func__);

	if ((*dev->current_state == PE_SRC_SEND_CAPS) ||
		(*dev->current_state == PE_SRC_SEND_SOFT_RESET))
	{
		pe_set_state(dev, PE_SRC_HARD_RESET);
	}
	else if ((*dev->current_state == PE_SNK_SELECT_CAPABILITY) ||
			 (*dev->current_state == PE_SNK_SEND_SOFT_RESET))
	{
		pe_set_state(dev, PE_SNK_HARD_RESET);
	}
	else 
	{
		if (dev->power_role == PD_PWR_ROLE_SNK)
		{
			pe_set_state(dev, PE_SNK_READY);
		}
		else
		{
			pe_set_state(dev, PE_SRC_READY);
		}
	}

	return;
}

static void ctrl_msg_handle_accept(usb_pd_port_t *dev)
{
	if (*dev->current_state == PE_SRC_SEND_SOFT_RESET)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);
		dev->non_interruptable_ams = false;
		pe_set_state(dev, PE_SRC_SEND_CAPS);
	}
	else if (*dev->current_state == PE_SNK_SEND_SOFT_RESET)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);
		dev->non_interruptable_ams = false;
		pe_set_state(dev, PE_SNK_WAIT_FOR_CAPS);
	}
	else if (*dev->current_state == PE_SNK_SELECT_CAPABILITY)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);
		pe_set_state(dev, PE_SNK_TRANSITION_SINK);
	}
	else if (*dev->current_state == PE_DRS_SEND_SWAP)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);
		pe_set_state(dev, PE_DRS_CHANGE_ROLE);
	}
	else if (*dev->current_state == PE_PRS_SEND_SWAP)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);
		dev->power_role_swap_in_progress = true;
		pe_set_state(dev, PE_PRS_TRANSITION_TO_OFF);
	}
	else if (*dev->current_state == PE_VCS_SEND_SWAP)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);

		if (dev->vconn_source)
		{
			pe_set_state(dev, PE_VCS_WAIT_FOR_VCONN);
		}
		else
		{
			pe_set_state(dev, PE_VCS_TURN_ON_VCONN);
		}
	}

	return;
}

static void ctrl_msg_handle_wait_or_reject(usb_pd_port_t *dev)
{
	if (*dev->current_state == PE_SNK_SELECT_CAPABILITY)
	{
		if (dev->explicit_contract)
		{
			if (dev->rx_msg_type == CTRL_MSG_TYPE_WAIT)
			{
				dev->snk_wait = true;
			}

			// Restore previously selected PDO.
			dev->selected_pdo = dev->prev_selected_pdo;

			pe_set_state(dev, PE_SNK_READY);

		}
		else
		{
			pe_set_state(dev, PE_SNK_WAIT_FOR_CAPS);
		}
	}
	else if ((*dev->current_state == PE_DRS_SEND_SWAP) ||
			 (*dev->current_state == PE_PRS_SEND_SWAP) ||
			 (*dev->current_state == PE_VCS_SEND_SWAP))
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);

		if (dev->power_role == PD_PWR_ROLE_SNK)
		{
			pe_set_state(dev, PE_SNK_READY);
		}
		else
		{
			pe_set_state(dev, PE_SRC_READY);
		}
	}

	return;
}

static void ctrl_msg_handle_ps_ready(usb_pd_port_t *dev)
{
	if (*dev->current_state == PE_SNK_TRANSITION_SINK)
	{
		// Cancel PSTransition timer.
		timer_cancel(&dev->timer);
		pe_set_state(dev, PE_SNK_READY);
	}
	else if (*dev->current_state == PE_PRS_WAIT_SOURCE_ON)
	{
		// Cancel PSSourceOn timer.
		timer_cancel(&dev->timer);

		// VBUS is now present. 
		dev->vbus_present = true;

		// Enable VBUS present detection for disconnect.
		tcpm_enable_vbus_detect(dev->port);

		// Enable bleed discharge to ensure timely discharge of 
		// VBUS bulk cap upon disconnect.
		tcpm_set_bleed_discharge(dev->port, true);

		tcpm_sink_vbus(dev->port, false, 5000, GET_SRC_CURRENT_MA(dev->tcpc_dev->src_current_adv));

		pe_set_state(dev, PE_SNK_STARTUP);
	}
	else if (*dev->current_state == PE_PRS_TRANSITION_TO_OFF)
	{
		if (dev->power_role == PD_PWR_ROLE_SNK)
		{
			// Cancel PSSourceOff timer.
			timer_cancel(&dev->timer);
			pe_set_state(dev, PE_PRS_ASSERT_RP);
		}
	}
	else if (*dev->current_state == PE_VCS_WAIT_FOR_VCONN)
	{
		// Cancel VCONNOn timer.
		timer_cancel(&dev->timer);
		pe_set_state(dev, PE_VCS_TURN_OFF_VCONN);
	}

	return;
}

static void ctrl_msg_handle_dr_swap(usb_pd_port_t *dev)
{
	if ((*dev->current_state == PE_SNK_READY) ||
		(*dev->current_state == PE_SRC_READY))
	{
		if (!dev->active_alt_modes)
		{
			pe_set_state(dev, PE_DRS_EVALUATE_SWAP);
		}
		else
		{
			// Send hard reset.
			if (*dev->current_state == PE_SNK_READY)
			{
				pe_set_state(dev, PE_SNK_HARD_RESET);
			}
			else
			{
				pe_set_state(dev, PE_SRC_HARD_RESET);
			}
		}
	}

	return;
}

#ifdef ENABLE_USB_PD_REV3_SUPPORT

static void ctrl_msg_handle_not_supported(usb_pd_port_t *dev)
{
	if (dev->timer.function == timeout_sender_response)
	{
		// Stop sender response timer.
		timer_cancel(&dev->timer);
	}

	if ((*dev->current_state == PE_VCS_SEND_SWAP) && 
		!dev->vconn_source)
	{
		// Enable VCONN. (PE_VCS_FORCE_VCONN)
		tcpm_set_vconn_enable(dev->port, true);
		dev->vconn_source = true;
	}

	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_READY);
	}
	else
	{
		pe_set_state(dev, PE_SRC_READY);
	}

	return;
}
#endif

static void usb_pd_pe_ctrl_msg_rx_handler(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	bool unrecognized_or_unsupported = false;

#ifdef ENABLE_VDM_SUPPORT
	if (dev->vdm_in_progress)
	{
		usb_pd_pe_handle_vdm_interruption(dev);
	}
#endif

	switch (dev->rx_msg_type)
	{
		case CTRL_MSG_TYPE_GOTO_MIN:
			if (*dev->current_state == PE_SNK_READY)
			{
				if (config->giveback_flag)
				{
					dev->snk_goto_min = true;
					pe_set_state(dev, PE_SNK_TRANSITION_SINK);
				}
				else /* Giveback is not supported */
				{
					unrecognized_or_unsupported = true;
				}
			}
			else if (dev->tcpc_dev->role == ROLE_SRC)
			{
				unrecognized_or_unsupported = true;
			}
			break;

		case CTRL_MSG_TYPE_ACCEPT:
			ctrl_msg_handle_accept(dev);
			break;

		case CTRL_MSG_TYPE_WAIT: // Fall through...
		case CTRL_MSG_TYPE_REJECT:
			CRIT("%s Rx'd\n", (dev->rx_msg_type == CTRL_MSG_TYPE_REJECT) ? "Reject" : "Wait");
			ctrl_msg_handle_wait_or_reject(dev);
			break;

		case CTRL_MSG_TYPE_PING:
			// Sink may receive ping msgs but should ignore them.
			if (dev->tcpc_dev->role == ROLE_SRC)
			{
				unrecognized_or_unsupported = true;
			}
			break;

		case CTRL_MSG_TYPE_PS_RDY:
			ctrl_msg_handle_ps_ready(dev);
			break;

		case CTRL_MSG_TYPE_GET_SRC_CAP:
			if (*dev->current_state == PE_SRC_READY)
			{
				pe_set_state(dev, PE_SRC_SEND_CAPS);
			}
			else if (*dev->current_state == PE_SNK_READY)
			{
				if (dev->tcpc_dev->role == ROLE_DRP)
				{
					pe_set_state(dev, PE_DR_SNK_GIVE_SOURCE_CAP);
				}
				else
				{
					// Sinks do not support this msg.
					unrecognized_or_unsupported = true;
				}
			}
			break;

		case CTRL_MSG_TYPE_GET_SNK_CAP:
			if (*dev->current_state == PE_SNK_READY)
			{
				pe_set_state(dev, PE_SNK_GIVE_SINK_CAP);
			}
			else if (*dev->current_state == PE_SRC_READY)
			{
				if (dev->tcpc_dev->role == ROLE_DRP)
				{
					pe_set_state(dev, PE_DR_SRC_GIVE_SINK_CAP);
				}
				else
				{
					// Sources do not support this msg.
					unrecognized_or_unsupported = true;
				}
			}
			break;

		case CTRL_MSG_TYPE_DR_SWAP:
			ctrl_msg_handle_dr_swap(dev);
			break;

		case CTRL_MSG_TYPE_PR_SWAP:
			if ((*dev->current_state == PE_SNK_READY) ||
				(*dev->current_state == PE_SRC_READY))
			{
				pe_set_state(dev, PE_PRS_EVALUATE_SWAP);
			}
			break;

		case CTRL_MSG_TYPE_VCONN_SWAP:
			if ((*dev->current_state == PE_SNK_READY) ||
				(*dev->current_state == PE_SRC_READY))
			{
				pe_set_state(dev, PE_VCS_EVALUATE_SWAP);
			}
			break;

		case CTRL_MSG_TYPE_SOFT_RESET:
			if (dev->power_role == PD_PWR_ROLE_SNK)
			{
				pe_set_state(dev, PE_SNK_SOFT_RESET);
			}
			else
			{
				pe_set_state(dev, PE_SRC_SOFT_RESET);
			}
			break;

#ifdef ENABLE_USB_PD_REV3_SUPPORT
		case CTRL_MSG_TYPE_NOT_SUPPORTED:
			CRIT("Not Supported Rx'd\n");
			ctrl_msg_handle_not_supported(dev);
			break;

#ifdef ENABLE_EXTENDED_MSG_SUPPORT

		case CTRL_MSG_TYPE_GET_SRC_CAP_EXT:
			if (*dev->current_state == PE_SRC_READY)
			{
				dev->non_interruptable_ams = true;
				pe_set_state(dev, PE_SRC_GIVE_SOURCE_CAP_EXT);
			}
			else if (*dev->current_state == PE_SNK_READY)
			{
				if (dev->tcpc_dev->role == ROLE_DRP)
				{
					dev->non_interruptable_ams = true;
					pe_set_state(dev, PE_SNK_GET_SOURCE_CAP_EXT);
				}
				else
				{
					// Sinks do not support this msg.
					unrecognized_or_unsupported = true;
				}
			}
			break;

		case CTRL_MSG_TYPE_GET_STATUS:
			if (*dev->current_state == PE_SRC_READY)
			{
				pe_set_state(dev, PE_SRC_GIVE_SOURCE_STATUS);
			}
			else if (*dev->current_state == PE_SNK_READY)
			{
				pe_set_state(dev, PE_SNK_GIVE_SINK_STATUS);
			}
			break;

		case CTRL_MSG_TYPE_FR_SWAP:
			// TUSB422 WCSP cannot control power directly so fast role swap is not supported.
			pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			break;

		case CTRL_MSG_TYPE_GET_PPS_STATUS:
		case CTRL_MSG_TYPE_GET_COUNTRY_CODES:
			// These msgs are currently not supported. Todo: Add support.
			pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			break;

#else /* Extended msgs not supported */

		case CTRL_MSG_TYPE_GET_SRC_CAP_EXT:
		case CTRL_MSG_TYPE_GET_STATUS:
		case CTRL_MSG_TYPE_FR_SWAP:
		case CTRL_MSG_TYPE_GET_PPS_STATUS:
		case CTRL_MSG_TYPE_GET_COUNTRY_CODES:
			pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			break;
#endif
#endif /* ENABLE_USB_PD_REV3_SUPPORT */
		default:
			CRIT("Invalid ctrl rx_msg_type 0x%x!\n", dev->rx_msg_type);
			unrecognized_or_unsupported = true;
			break;
	}

	if (!dev->state_change && (dev->rx_msg_type != CTRL_MSG_TYPE_PING))
	{
		usb_pd_pe_unhandled_rx_msg(dev, unrecognized_or_unsupported);
	}

	return;
}

#define BIST_CARRIER_MODE_REQUEST  5
#define BIST_TEST_DATA             8

static void usb_pd_pe_data_msg_rx_handler(usb_pd_port_t *dev)
{
	bool unrecognized_or_unsupported = false;

#ifdef ENABLE_VDM_SUPPORT
	if ((dev->vdm_in_progress) && (dev->rx_msg_type != DATA_MSG_TYPE_VENDOR))
	{
		usb_pd_pe_handle_vdm_interruption(dev);
	}
#endif

	switch (dev->rx_msg_type)
	{
		case DATA_MSG_TYPE_SRC_CAPS:
			if (*dev->current_state == PE_SNK_WAIT_FOR_CAPS)
			{
				// Cancel SinkWaitCap timer.
				timer_cancel(&dev->timer);
				dev->pd_connected = true;
				dev->pd_connected_since_attach = true;
				pe_set_state(dev, PE_SNK_EVALUATE_CAPABILITY);
			}
			else if (*dev->current_state == PE_SNK_READY)
			{
				pe_set_state(dev, PE_SNK_EVALUATE_CAPABILITY);
			}
			else if (dev->tcpc_dev->role == ROLE_SRC)
			{
				unrecognized_or_unsupported = true;
			}
			else
			{
				// Hard reset for this particular protocol error.
				pe_set_state(dev, PE_SNK_HARD_RESET);
			}
			break;

		case DATA_MSG_TYPE_REQUEST:
			if ((*dev->current_state == PE_SRC_SEND_CAPS) ||
				(*dev->current_state == PE_SRC_READY))
			{
				// Cancel sender response timer.
				timer_cancel(&dev->timer);
				pe_set_state(dev, PE_SRC_NEGOTIATE_CAPABILITY);

#ifdef ENABLE_USB_PD_REV3_SUPPORT
				if (dev->timer2.function == timeout_pps_timeout)
				{
					// Stop the SourcePPSCommTimer if there is a Request msg, PR swap, or Hard Reset.
					tusb422_lfo_timer_cancel(&dev->timer2);
				}
#endif
			}
			else if (dev->tcpc_dev->role == ROLE_SNK)
			{
				unrecognized_or_unsupported = true;
			}
			break;

		case DATA_MSG_TYPE_BIST:
			if ((*dev->current_state == PE_SNK_READY) ||
				(*dev->current_state == PE_SRC_READY))
			{
				// Check we are operating at vSafe5V and power role is not swapped.
				if ((dev->object_position == 1) &&
					(dev->power_role == dev->data_role))
				{
					// Check BIST param, data_obj[31:28].
					if ((dev->msg_buf[3] >> 4) == BIST_CARRIER_MODE_REQUEST)
					{
						pe_set_state(dev, PE_BIST_CARRIER_MODE);
					}
					else if ((dev->msg_buf[3] >> 4) == BIST_TEST_DATA)
					{
						// Set BIST mode.
						tcpm_set_bist_test_mode(dev->port);

						pe_set_state(dev, PE_BIST_TEST_MODE);

						// This test mode shall be ended by a Hard Reset.
					}
				}
			}
			break;

		case DATA_MSG_TYPE_SNK_CAPS:
			if (*dev->current_state == PE_SRC_GET_SINK_CAP)
			{
				// Cancel sender response timer.
				timer_cancel(&dev->timer);

				dev->get_sink_caps_complete = true;
				// Pass sink caps to policy manager. - BQ

				pe_set_state(dev, PE_SRC_READY);
			}
			else if (dev->tcpc_dev->role == ROLE_SNK)
			{
				unrecognized_or_unsupported = true;
			}
			break;

#ifdef ENABLE_USB_PD_REV3_SUPPORT 
#ifdef ENABLE_EXTENDED_MSG_SUPPORT

		case DATA_MSG_TYPE_BATT_STATUS:
			// This is the response to a Get_Battery_Status Message.
			// Currently, we do not request battery status so this is not supported.
			pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			break;

		case DATA_MSG_TYPE_ALERT:
			if (*dev->current_state == PE_SRC_READY)
			{
				pe_set_state(dev, PE_SRC_SINK_ALERT_RECEIVED);
			}
			else if (*dev->current_state == PE_SNK_READY)
			{
				pe_set_state(dev, PE_SNK_SOURCE_ALERT_RECEIVED);
			}
			break;

		case DATA_MSG_TYPE_GET_COUNTRY_INFO:
			pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			break;

#else /* Extended msgs not supported */

		case DATA_MSG_TYPE_BATT_STATUS:
		case DATA_MSG_TYPE_ALERT:
		case DATA_MSG_TYPE_GET_COUNTRY_INFO:
			pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			break;
#endif
#endif /* ENABLE_USB_PD_REV3_SUPPORT */

		case DATA_MSG_TYPE_VENDOR:
#ifdef ENABLE_VDM_SUPPORT
			usb_pd_pe_vdm_handler(dev);

#elif defined ENABLE_USB_PD_REV3_SUPPORT && !defined CABLE_PLUG 
			if (dev->p2p_spec_rev > PD_REV20)
			{
				// For PD r3.0, DFP or UFP should return Not_Supported msg if not supported.
				pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
			}
#endif
			break;

		default:
			CRIT("Invalid data rx_msg_type 0x%x!\n", dev->rx_msg_type);
			unrecognized_or_unsupported = true;
			break;
	}

	// Certain Vendor and BIST msgs may be ignored if not supported so don't check here.
	// For PD r2.0, DFP or UFP should ignore the Vendor msg if not supported.
	// Unsupported BIST msgs are always ignored.
	if (!dev->state_change && 
		(dev->rx_msg_type != DATA_MSG_TYPE_BIST) && 
		(dev->rx_msg_type != DATA_MSG_TYPE_VENDOR))
	{
		usb_pd_pe_unhandled_rx_msg(dev, unrecognized_or_unsupported);
	}

	return;
}

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT

static void usb_pd_pe_ext_msg_rx_handler(usb_pd_port_t *dev)
{
	bool unrecognized = false;
	bool goto_ready_state = false;

	INFO("%s: %u\n", __func__, dev->rx_msg_type);
	switch (dev->rx_msg_type)
	{
		case EXT_MSG_TYPE_SRC_CAPS_EXT:
			if (*dev->current_state == PE_SNK_GET_SOURCE_CAP_EXT)
			{
				// Report capabilities to policy manager.
				goto_ready_state = true;
			}
			break;

		case EXT_MSG_TYPE_STATUS:
			if ((*dev->current_state == PE_SRC_GET_SINK_STATUS) ||
				(*dev->current_state == PE_SNK_GET_SOURCE_STATUS))
			{
				// Report status to policy manager.
				goto_ready_state = true;
			}
			break;

		case EXT_MSG_TYPE_GET_BATT_CAP:
			dev->non_interruptable_ams = true;
			pe_set_state(dev, PE_GIVE_BATTERY_CAP);
			break;

		case EXT_MSG_TYPE_GET_BATT_STATUS:
			dev->non_interruptable_ams = true;
			pe_set_state(dev, PE_GIVE_BATTERY_STATUS);
			break;

		case EXT_MSG_TYPE_BATT_CAPABILITIES:
			if (*dev->current_state == PE_GET_BATTERY_CAP)
			{
				// Report capabilities to policy manager.
				goto_ready_state = true;
			}
			break;

		case EXT_MSG_TYPE_GET_MANUF_INFO:
			dev->non_interruptable_ams = true;
			pe_set_state(dev, PE_GIVE_MANUFACTURER_INFO);
			break;

		case EXT_MSG_TYPE_MANUF_INFO:
			if (*dev->current_state == PE_GET_MANUFACTURER_INFO)
			{
				// Report info to policy manager.
				goto_ready_state = true;
			}
			break;

		/* Interruptible extended msgs */
		case EXT_MSG_TYPE_SECURITY_REQUEST:
		case EXT_MSG_TYPE_SECURITY_RESPONSE:
		case EXT_MSG_TYPE_FW_UPDATE_REQUEST:
		case EXT_MSG_TYPE_FW_UPDATE_RESPONSE:
			break;

		default:
			unrecognized = true;
			break;
	}

	if (goto_ready_state)
	{
		if (dev->power_role == PD_PWR_ROLE_SNK)
		{
			pe_set_state(dev, PE_SNK_READY);
		}
		else
		{
			pe_set_state(dev, PE_SRC_READY);
		}
	}
	else if ((*dev->current_state == PE_SRC_READY) || 
			 (*dev->current_state == PE_SNK_READY))
	{
		// Send Not Supported response to all Extended msgs not handled while in Ready state.
		pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
	}

	if (!dev->state_change)
	{
		usb_pd_pe_unhandled_rx_msg(dev, unrecognized);
	}

	return;
}

#endif /* ENABLE_USB_PD_REV3_SUPPORT */

#ifdef ENABLE_VDM_SUPPORT

static void usb_pd_pe_tx_vendor_msg(unsigned int port, uint8_t ndo, tcpc_transmit_t sop_type)
{
	usb_pd_port_t *dev = &pd[port];
	uint8_t *payload_ptr = &dev->msg_buf[3];
	uint8_t vdo_idx;

	for (vdo_idx = 0; vdo_idx < ndo; vdo_idx++)
	{
		*payload_ptr++ = (uint8_t)(dev->vdm_buf[vdo_idx] & 0xFF);
		*payload_ptr++ = (uint8_t)((dev->vdm_buf[vdo_idx] & 0xFF00) >> 8);
		*payload_ptr++ = (uint8_t)((dev->vdm_buf[vdo_idx] & 0xFF0000) >> 16);
		*payload_ptr++ = (uint8_t)((dev->vdm_buf[vdo_idx] & 0xFF000000) >> 24);
	}

	dev->vdm_in_progress = true;
	dev->vdm_initiator = (USB_PD_VDM_HDR_GET_CMD_TYPE(dev->vdm_buf[0]) == VDM_INITIATOR) ? true : false;

	usb_pd_prl_tx_data_msg(port, DATA_MSG_TYPE_VENDOR, sop_type, ndo);

	return;
}

/* Send structured VDM with no VDOs */
static void vdm_send_structured_cmd(unsigned int port, uint16_t svid, vdm_command_t cmd, uint8_t obj_pos, tcpc_transmit_t sop_type)
{
	usb_pd_port_t *dev = &pd[port];

	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(svid, 
										 VDM_TYPE_STRUCTURED, 
										 (sop_type == TCPC_TX_SOP) ? dev->p2p_vdm_version : dev->p2c_vdm_version, 
										 obj_pos, VDM_INITIATOR, cmd);

	usb_pd_pe_tx_vendor_msg(port, 1, sop_type);

	return;
}

#define INVALID_SVID_INDEX  0xFF

static uint8_t find_svid_index(usb_pd_port_t *dev, uint16_t svid)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t svid_idx;

	for (svid_idx = 0; svid_idx < config->num_svids; svid_idx++)
	{
		if (svid == config->svids[svid_idx])
		{
			return svid_idx;
		}
	}

	return INVALID_SVID_INDEX;
}

static void timeout_vdm_busy(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if ((*dev->current_state == PE_SRC_READY) ||
		(*dev->current_state == PE_SNK_READY))
	{
		pe_set_state(dev, dev->vdm_pending_state);
#ifdef ENABLE_USB_PD_REV3_SUPPORT
		usb_pd_pe_check_sink_tx_ok(dev);
#endif
	}
	else
	{
		dev->vdm_pending = true;
	}

	return;
}

#ifdef ENABLE_DP_ALT_MODE_SUPPORT

/* Use by UPF_D only */
void usb_pd_pe_update_dp_status(unsigned int port, uint32_t clr_bits, uint32_t set_bits) 
{
	usb_pd_port_t *dev = &pd[port];
	uint32_t dp_status = dev->displayport_status;

	dp_status &= ~clr_bits;
	dp_status |= set_bits;

	if (dev->displayport_status != dp_status)
	{
		dev->displayport_status = dp_status;

		CRIT("%s: DP status = 0x%x\n", __func__, dev->displayport_status);

		if ((dev->data_role == PD_DATA_ROLE_UFP) &&
			dev->displayport_alt_mode_configured)
		{
			if (*dev->current_state == PE_SNK_READY)
			{
				pe_set_state(dev, PE_INIT_VDM_ATTENTION_REQUEST);
			}
			else
			{
				dev->attention_pending = true;
			}
		}
	}

	return;
}

static void vdm_send_dp_status(usb_pd_port_t *dev, vdm_command_type_t cmd_type, uint8_t cmd)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t svid_index = find_svid_index(dev, VDM_DISPLAYPORT_VID);

	// Build header.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_DISPLAYPORT_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, cmd_type, cmd); 

	if (dev->displayport_status & ADAPTER_POWER_LOW)
	{
		dev->displayport_status &= ~(ADAPTER_ENABLED | HPD_STATE_HIGH | IRQ_HPD);
	}

	// Build status.
	dev->vdm_buf[1] = dev->displayport_status;

	// Populate DFP_D/UFP_D connected status based on port capability.
	// Assumption is that this is not a DP cable adapter and only supports UFP_D or DFP_D and not both.
	// Otherwise, HPD and AUX should be used per DP Alt-Mode Spec section 4.2.4.2 for detection.
//	if (DP_PORT_CAPABILITY(config->modes[svid_index]))
//	{
//		dev->vdm_buf[1] |= DP_PORT_CAPABILITY(~config->modes[svid_index]);
//	}

	// If UFP_U, populate bits [6:2].
	if (dev->data_role == PD_DATA_ROLE_UFP)
	{
		if ((dev->vdm_buf[1] & (UFP_D_CONNECTED | DFP_D_CONNECTED)) &&
			!(dev->displayport_status & ADAPTER_POWER_LOW))
		{
			// Set DP functionality to enabled if anything is connected and power is not low.
			dev->vdm_buf[1] |= ADAPTER_ENABLED;
		}

		if (config->multi_function_preferred)
		{
			dev->vdm_buf[1] |= MULTIFUNC_PREFERRED;
		}
	}
	else /* DFP */
	{
		// Clear UFP_U specific bits [6:2].
		dev->vdm_buf[1] &= 0x183;
	}

	// Tx length is 1 for the VDM header + 1 for the status.
	usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);

	return;
}


static void pe_init_vdm_dp_status_update_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	// Only a receptacle-based DFP_U can initiate messages other than Attention command.
	// BQ - violating this to allow plug-based DP dongle to negotiate alt mode.
	if ((dev->active_alt_modes & DP_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP)/* && 
		(config->modes[find_svid_index(dev, VDM_DISPLAYPORT_VID)] & RECEPTACLE_INDICATION_BIT)*/)
	{
		// Send DP Status Update VDM.
		vdm_send_dp_status(dev, VDM_INITIATOR, VDM_CMD_DP_STATUS_UPDATE);
	}

	return;
}

typedef enum
{
	PIN_ASSIGNMENT_A = 1,
	PIN_ASSIGNMENT_B = (1 << 1),
	PIN_ASSIGNMENT_C = (1 << 2),
	PIN_ASSIGNMENT_D = (1 << 3),
	PIN_ASSIGNMENT_E = (1 << 4),
	PIN_ASSIGNMENT_F = (1 << 5)	  /* for DockPort 1.0a */
} pin_assignment_supported_t;

typedef enum
{
	DP_V1P3_RATE = 1,
	DP_GEN2_RATE = 2
} dp_signaling_rate_t;

static void pe_init_vdm_dp_config_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint16_t config_pin_assignment = 0;
	uint8_t lane_cnt;

	// Only a receptacle-based DFP_U can initiate messages other than Attention command.
	// BQ - violating this to allow plug-based DP dongle to negotiate alt mode.
	if ((dev->active_alt_modes & DP_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP)/* && 
		(config->modes[find_svid_index(dev, VDM_DISPLAYPORT_VID)] & RECEPTACLE_INDICATION_BIT)*/)
	{
		// Build header.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_DISPLAYPORT_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_INITIATOR, VDM_CMD_DP_CONFIG); 

		// Build DP config.
		dev->vdm_buf[1] = 0;

		dev->vdm_buf[1] |= dev->displayport_config;

		if (dev->displayport_config != CONFIG_FOR_USB)
		{
			// Select DP pin assignment. Give priority to higher DP data rate and then multifuction.
			// Pin assignments A and B use DP Gen2 signaling.  
			// If multifunction is preferred, use Pin Assignment B, D, or F if possible.
			if ((dev->displayport_matched_pin_assignments & PIN_ASSIGNMENT_A))
			{
				config_pin_assignment = PIN_ASSIGNMENT_A;

				if (dev->active_cable)
				{
					lane_cnt = 2;
				}
				else
				{
					lane_cnt = 4;
				}
			}

			if ((dev->displayport_matched_pin_assignments & PIN_ASSIGNMENT_B) && 
				(dev->displayport_remote_status & MULTIFUNC_PREFERRED))
			{
				config_pin_assignment = PIN_ASSIGNMENT_B;

				if (dev->active_cable)
				{
					lane_cnt = 1;
				}
				else
				{
					lane_cnt = 2;
				}
			}

			// If assignment A or B was not selected.
			if (!config_pin_assignment)
			{
				if ((dev->displayport_matched_pin_assignments & PIN_ASSIGNMENT_C))
				{
					config_pin_assignment = PIN_ASSIGNMENT_C;
					lane_cnt = 4;
				}

				if ((dev->displayport_matched_pin_assignments & PIN_ASSIGNMENT_D) && 
					(dev->displayport_remote_status & MULTIFUNC_PREFERRED))
				{
					config_pin_assignment = PIN_ASSIGNMENT_D;
					lane_cnt = 2;
				}
			}

			// If assignment A, B, C, or D was not selected.
			if (!config_pin_assignment)
			{
				if ((dev->displayport_matched_pin_assignments & PIN_ASSIGNMENT_E))
				{
					config_pin_assignment = PIN_ASSIGNMENT_E;
					lane_cnt = 4;
				}

				if ((dev->displayport_matched_pin_assignments & PIN_ASSIGNMENT_F) && 
					(dev->displayport_remote_status & MULTIFUNC_PREFERRED))
				{
					config_pin_assignment = PIN_ASSIGNMENT_F;
					lane_cnt = 2;
				}
			}

			// Set pin assignment configuration field.
			dev->vdm_buf[1] |= (config_pin_assignment << 8);

			// Set DP data rate field based on pin assignment.
			if (config_pin_assignment & (PIN_ASSIGNMENT_A | PIN_ASSIGNMENT_B))
			{
				// DP Gen2 rate.
				dev->vdm_buf[1] |= DP_GEN2_RATE << 2;
			}
			else
			{
				// DP v1.3 rate.
				dev->vdm_buf[1] |= DP_V1P3_RATE << 2;
			}

			dev->displayport_selected_pin_assignment = config_pin_assignment;

			if (lane_cnt != dev->displayport_lane_cnt)
			{
				// Place any Type-C pins to be reconfigured to in Safe state.
				if (lane_cnt == 4)
				{
					tcpm_mux_control(dev->port, dev->data_role, MUX_DISABLE, dev->tcpc_dev->plug_polarity);
				}
				else /* 2-lane */
				{
					tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);
				}

				if (dev->displayport_lane_cnt)
				{
					// If changing the number of DP lanes, force HPD low and isolate SBU pins.
					// Then ensure HPD is low for at least 3ms.
					tcpm_hpd_out_control(dev->port, HPD_LOW);
#ifndef CONFIG_TUSB422
					tcpm_msleep(3);
#endif
				}

				dev->displayport_lane_cnt = lane_cnt;
			}
		}
		else /* Config for USB */
		{
			// Deassert HPD.
			tcpm_hpd_out_control(dev->port, HPD_LOW);

			// Isolate any pins (including SBUs) previously configured for DP.
			if (dev->displayport_lane_cnt == 4)
			{
				tcpm_mux_control(dev->port, dev->data_role, MUX_DISABLE, dev->tcpc_dev->plug_polarity);
			}
			else if (dev->displayport_lane_cnt == 2)
			{
				tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);
			}

			dev->displayport_lane_cnt = 0;
		}

		// Tx length is 1 for the VDM header + 1 for the configuration.
		usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);
	}

	return;
}


static void pe_resp_vdm_dp_status_update_entry(usb_pd_port_t *dev)
{
	vdm_send_dp_status(dev, VDM_RESP_ACK, VDM_CMD_DP_STATUS_UPDATE);
	return;
}

static void pe_resp_vdm_dp_config_entry(usb_pd_port_t *dev)
{
	// Send ACK.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_DISPLAYPORT_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_RESP_ACK, VDM_CMD_DP_CONFIG); 

	// Tx length is 1 for the VDM header.
	usb_pd_pe_tx_vendor_msg(dev->port, 1, TCPC_TX_SOP);

	return;
}

static void vdm_handle_dp_status_update(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP responds to DP Status Update.  
		// UFP doesn't need to use any of the data sent by the DFP.
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			pe_set_state(dev, PE_RESP_VDM_DP_STATUS_UPDATE);
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_DP_STATUS_UPDATE)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			// Store the remote DP status.
			dev->displayport_remote_status = get_data_object(dev->msg_buf + 4);

			if (dev->active_alt_modes & DP_ALT_MODE)
			{
				// Enter Mode completed.

				if (!dev->displayport_alt_mode_configured)
				{
					// Set state to send DP Config if connected but not configured.
					if (dev->displayport_remote_status & UFP_D_CONNECTED)
					{
						dev->displayport_config = CONFIG_UFP_U_AS_UFP_D;
						pe_set_state(dev, PE_INIT_VDM_DP_CONFIG);
					}
					else if (dev->displayport_remote_status & DFP_D_CONNECTED)
					{
						dev->displayport_config = CONFIG_UFP_U_AS_DFP_D;
						pe_set_state(dev, PE_INIT_VDM_DP_CONFIG);
					}
				}

				/* 
				 * HPD status will not be reported to the DisplayPort source until
				 * after the high-speed signal muxes have been configured
				 * which occurs when handling the ACK for the DP Config VDM.
				 */
			}
		}
		else if (cmd_type == VDM_RESP_BUSY)
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
	}

	return;
}

#ifdef DP_DONGLE
static void dp_dongle_config_active(usb_pd_port_t *dev)
{
	// Check if redriver is powered.
	if (!(dev->displayport_status & ADAPTER_POWER_LOW))
	{
		if (dev->displayport_config == CONFIG_UFP_U_AS_UFP_D)
		{
			// UFP_U: Configure redriver for USB to DP direction.
			// DFP_U: Configure redriver for DP to USB direction.
			tusb544_config(dev->port, (dev->data_role == PD_DATA_ROLE_UFP) ? false : true);
			dev->redriver_configured = true;
		}
		else if (dev->displayport_config == CONFIG_UFP_U_AS_DFP_D)
		{
			// UFP_U: Configure redriver for DP to USB direction.
			// DFP_U: Configure redriver for USB to DP direction.
			tusb544_config(dev->port, (dev->data_role == PD_DATA_ROLE_UFP) ? true : false);
			dev->redriver_configured = true;
		}
	}

	return;
}
#endif

static void vdm_handle_dp_config(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	uint32_t dp_config;

	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP responds to DP Config.  
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			dp_config = get_data_object(dev->msg_buf + 4);

			dev->displayport_config	= (dp_config_sel_t)(dp_config & CONFIG_MASK);
			dev->displayport_alt_mode_configured = true;

			PRINT("DP Config = %u\n", dev->displayport_config);

			// Configure pins.
			if (dev->displayport_config == CONFIG_FOR_USB)
			{
#ifdef DP_DONGLE
				// Check if redriver is powered.
				if (!(dev->displayport_status & ADAPTER_POWER_LOW))
				{
					// Disable the redriver to isolate SBU pins.
					tusb544_disable(dev->port);
				}

				dev->redriver_configured = false;

				// Ensure HPD is set to an input.
				tcpm_get_hpd_in(dev->port);
#endif
				tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);
			}
			else /* CONFIG_UFP_U_AS_UFP_D or CONFIG_UFP_U_AS_DFP_D */
			{
#ifdef DP_DONGLE
				dp_dongle_config_active(dev);
#endif
				switch ((dp_config >> 8) & 0xFF)
				{
					case PIN_ASSIGNMENT_A:
					case PIN_ASSIGNMENT_C:
					case PIN_ASSIGNMENT_E:
						// 4-lane.
						tcpm_mux_control(dev->port, dev->data_role, MUX_DP_4LANE, dev->tcpc_dev->plug_polarity);
						break;

					case PIN_ASSIGNMENT_B:
					case PIN_ASSIGNMENT_D:
					case PIN_ASSIGNMENT_F: /* DFP_D only */
						// 2-lane.
						tcpm_mux_control(dev->port, dev->data_role, MUX_DP_2LANE, dev->tcpc_dev->plug_polarity);
						break;

					default:
						PRINT("Invalid UFP_U pin assignment = 0x%x\n", (dp_config >> 8));
						break;
				}

				// Check for HPD high.
				if (tcpm_get_hpd_in(0))
				{
					dev->hpd_in_queue[dev->hpd_in_queue_idx++] = NOTIFY_HPD_HIGH;
				}
			}

			pe_set_state(dev, PE_RESP_VDM_DP_CONFIG);
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_DP_CONFIG)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			dev->displayport_alt_mode_configured = true;

			PRINT("DP Config = %u\n", dev->displayport_config);

#ifdef DP_DONGLE
			dp_dongle_config_active(dev);
#endif

			// Configure pins.
			if (dev->displayport_config == CONFIG_FOR_USB)
			{
				tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);
			}
			else /* CONFIG_UFP_U_AS_UFP_D or CONFIG_UFP_U_AS_DFP_D */
			{
				switch (dev->displayport_selected_pin_assignment)
				{
					case PIN_ASSIGNMENT_A:
					case PIN_ASSIGNMENT_C:
					case PIN_ASSIGNMENT_E:
						// 4-lane.
						tcpm_mux_control(dev->port, dev->data_role, MUX_DP_4LANE, dev->tcpc_dev->plug_polarity);
						break;

					case PIN_ASSIGNMENT_B:
					case PIN_ASSIGNMENT_D:
					case PIN_ASSIGNMENT_F: /* DFP_D only */
						// 2-lane.
						tcpm_mux_control(dev->port, dev->data_role, MUX_DP_2LANE, dev->tcpc_dev->plug_polarity);
						break;

					default:
						break;
				}

				if (dev->displayport_remote_status & HPD_STATE_HIGH)
				{
					// Assert HPD to DFP_D.
					tcpm_hpd_out_control(dev->port, HPD_HIGH);

#ifndef CONFIG_TUSB422
					// For MCU platforms only. Assume Linux-based systems handle this externally.
					if (dev->displayport_remote_status & IRQ_HPD)
					{
						// If HPD high and IRQ are reported together, make sure HPD is high for at least 2ms.
						tcpm_msleep(4);
					}
#endif
				}
				else
				{
					// Deassert HPD to DFP_D.
					tcpm_hpd_out_control(dev->port, HPD_LOW);
				}

				if (dev->displayport_remote_status & IRQ_HPD)
				{
					// Pulse HPD pin low for 1ms.
					tcpm_hpd_out_control(dev->port, HPD_IRQ);
				}
			}
		}
		else if (cmd_type == VDM_RESP_NACK)
		{
			// UFP was unable to set configuration.
			// Try another config or give up?? - BQ
		}
		else
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
	}

	return;
}

static void usb_pd_pe_update_displayport_status(usb_pd_port_t *dev)
{
	if (dev->hpd_in_queue[dev->hpd_in_send_idx] != NOTIFY_HPD_NONE)
	{
		// Update the DisplayPort status.
		if (dev->hpd_in_queue[dev->hpd_in_send_idx] == NOTIFY_HPD_LOW)
		{
			// If HPD is low, IRQ_HPD must also be low.
			dev->displayport_status &= ~(HPD_STATE_HIGH | IRQ_HPD);
		}
		else if (dev->hpd_in_queue[dev->hpd_in_send_idx] == NOTIFY_HPD_HIGH)
		{
			dev->displayport_status |= HPD_STATE_HIGH;
		}
		else if (dev->hpd_in_queue[dev->hpd_in_send_idx] == NOTIFY_HPD_IRQ)
		{
			dev->displayport_status |= (HPD_STATE_HIGH | IRQ_HPD);
		}

		// Clear the item in the queue.
		dev->hpd_in_queue[dev->hpd_in_send_idx] = NOTIFY_HPD_NONE;

		dev->hpd_in_send_idx++;
		if (dev->hpd_in_send_idx >= HPD_IN_QUEUE_SIZE)
		{
			dev->hpd_in_send_idx = 0;
		}
	}

	return;
}

static void timeout_irq_hpd(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	// Clear the timer callback function.
	dev->timer2.function = NULL;

	if (tcpm_get_hpd_in(port) == 0)
	{
		// HPD is still low, no IRQ_HPD.
		if (dev->active_alt_modes & DP_ALT_MODE)
		{
			// HPD is low. We can discard any queued status updates.
			dev->hpd_in_queue_idx = 0;
			dev->hpd_in_send_idx = 0;
			dev->hpd_in_queue[dev->hpd_in_queue_idx++] = NOTIFY_HPD_LOW; 

			// If HPD is low, UFP_D is disconnected.
			usb_pd_pe_update_dp_status(port, UFP_D_CONNECTED | HPD_STATE_HIGH | IRQ_HPD, 0);

			// Drive HPD to redriver low.
			tcpm_redriver_hpd_ctrl(port, 0);
		}
	}

	return;
}

#define T_IRQ_HPD_MAX_MS  2

#ifdef DP_DONGLE

#define HPD_DEBOUNCE_MS  100

void timeout_hpd_debounce(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];
	uint8_t val = tcpm_get_hpd_in(port);

	CRIT("timeout_hpd_debounce\n");

	if (dev->hpd_in_value == val)
	{
		if (dev->hpd_in_value != dev->hpd_in_previous_value)
		{
			dev->hpd_in_changed = true;
			dev->hpd_in_previous_value = dev->hpd_in_value;
		}
	}
	else
	{
		CRIT("*HPD: %u\n", val);
		dev->hpd_in_value = val;

		// Re-Start debounce.
		timer_start(&dev->dp_timer, HPD_DEBOUNCE_MS, timeout_hpd_debounce);
	}

	return;
}


void usb_pd_pe_hpd_in_event(unsigned int port, uint8_t val)
{
	usb_pd_port_t *dev = &pd[port];

	CRIT("HPD-IN: %u\n", val);
	dev->hpd_in_value = val;

	// Ignore HPD changes if DP alt-mode isn't active.
	if (dev->active_alt_modes & DP_ALT_MODE)
	{
		if (dev->displayport_status & UFP_D_CONNECTED)
		{
			if (dev->hpd_in_value != dev->hpd_in_previous_value)
			{
				dev->hpd_in_changed = true;
				dev->hpd_in_previous_value = dev->hpd_in_value;
			}
		}
		else if (dev->hpd_in_value)
		{
			// Debounce HPD high if not currently connected.
			timer_start(&dev->dp_timer, HPD_DEBOUNCE_MS, timeout_hpd_debounce);
		}
	}

	return;
}

void usb_pd_pe_aux_event(unsigned int port, uint8_t aux_p, uint8_t aux_n)
{
	usb_pd_port_t *dev = &pd[port];

	CRIT("-> AUX_P = %u, AUX_N = %u\n", aux_p, aux_n);

	// Do we need to check HPD-IN?  - BQ
	if (dev->data_role == PD_DATA_ROLE_DFP)
	{
		if (!aux_p && !aux_n)
		{
			// Nothing is connected to DP connector.
			usb_pd_pe_update_dp_status(port, (DFP_D_CONNECTED | UFP_D_CONNECTED), 0);
	
			// Ensure HPD is set to an input.
	//		tcpm_get_hpd_in(port);
	
			// Drive HPD to the redriver low to save power.
//			tcpm_redriver_hpd_ctrl(port, 0);
		}
		else if (aux_p && !aux_n)
		{
			// If AUX_P is high, UFP_D is connected.
			usb_pd_pe_update_dp_status(port, DFP_D_CONNECTED, UFP_D_CONNECTED);
	
//			if (dev->redriver_configured)
//			{
//				tcpm_redriver_hpd_ctrl(port, tcpm_get_hpd_in(port));
//			}
		}
		else if (!aux_p && aux_n)
		{
			// If AUX_N is high, DFP_D is connected.
			usb_pd_pe_update_dp_status(port, UFP_D_CONNECTED, DFP_D_CONNECTED);
	
//			if (dev->redriver_configured && (dev->displayport_remote_status & HPD_STATE_HIGH))
//			{
//				tcpm_redriver_hpd_ctrl(port, 1);
//				// Assert HPD to DFP_D.
//	//			tcpm_hpd_out_control(dev->port, HPD_HIGH);
//			}
		}	
	}

//	else if (dev->redriver_configured)
//	{
//		// SBU switch is closed when redriver is configured.
//
//		if (aux_p && !aux_n)
//		{
//			// If AUX_N is low and AUX_P is high, the DFP_D is disconnected.
//			usb_pd_pe_update_dp_status(port, DFP_D_CONNECTED, 0);
//		}
//
//		// HPD is monitored to check for UFP_D disconnect.
//	}
//	else /* SBU switch is open */
//	{
//		if (aux_p && !aux_n)
//		{
//			// If AUX_P is high, UFP_D is connected.
//			usb_pd_pe_update_dp_status(port, DFP_D_CONNECTED, UFP_D_CONNECTED);
//		}
//		else if (!aux_p && aux_n)
//		{
//			// If AUX_N is high, DFP_D is connected.
//			usb_pd_pe_update_dp_status(port, UFP_D_CONNECTED, DFP_D_CONNECTED);
//		}
//	}

	return;
}

void usb_pd_pe_redriver_pg_event(unsigned int port, uint8_t pwr_good)
{
	usb_pd_port_t *dev = &pd[port];

	PRINT("544-PWR_GOOD = %u\n", pwr_good);

	if (pwr_good)
	{
		usb_pd_pe_update_dp_status(0, ADAPTER_POWER_LOW, 0);

		if (dev->displayport_alt_mode_configured && !dev->redriver_configured)
		{
			dp_dongle_config_active(dev);
		}
	}
	else
	{
		usb_pd_pe_update_dp_status(0, 0, ADAPTER_POWER_LOW);
		dev->redriver_configured = false;
	}

	return;
}

#endif

#endif /* ENABLE_DP_ALT_MODE_SUPPORT */

#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT

#define DDC_HEADER_REG_OFFSET(hdr) ((hdr) & 0xFF)
#define DDC_HEADER_REG_SIZE(hdr)   (((uint32_t)(hdr) >> 8) & 0x3FF)
#define DDC_HEADER_SEQ_NUM(hdr)    (((uint32_t)(hdr) >> 18) & 0x3F)
#define DDC_HEADER_SLAVE_ADDR(hdr) (((uint32_t)(hdr) >> 24) & 0x7)

static const uint8_t ddc_slave_addr_lookup[4] = {
	HDCP_PRIMARY_SLAVE_ADDR,
	HDCP_SECONDARY_SLAVE_ADDR,
	EDID_SLAVE_ADDR,
	0xA8
};

static uint32_t slave_addr_to_ddc_header(uint8_t slave_addr)
{
	uint32_t ddc_hdr_slv_addr = 0xFF;

	if (slave_addr == HDCP_PRIMARY_SLAVE_ADDR)
		ddc_hdr_slv_addr = 0;
	else if (slave_addr == HDCP_SECONDARY_SLAVE_ADDR)
		ddc_hdr_slv_addr = 1;
	else if (slave_addr == EDID_SLAVE_ADDR)
		ddc_hdr_slv_addr = 2;
	else if (slave_addr == 0xA8)
		ddc_hdr_slv_addr = 3;

	return ddc_hdr_slv_addr;
}

#define DDC_HDR_GEN(reg_off, reg_sz, seq_num, slv_addr)  ( (slave_addr_to_ddc_header(slv_addr) << 24) | ((uint32_t)(seq_num) << 18) | ((uint32_t)(reg_sz) << 8) | (uint32_t)(reg_off) )


static void pe_resp_vdm_hdmi_status_update_entry(usb_pd_port_t *dev)
{
	// Build header.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_RESP_ACK, VDM_CMD_HDMI_STATUS_UPDATE); 

	// Build HDMI cable adapter status.
	dev->vdm_buf[1] = dev->hdmi_status;

	DEBUG("hdmi_status = 0x%x\n", dev->hdmi_status);

	// Tx length is 1 for the VDM header + 1 for the status.
	usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);

	return;
}

static void pe_resp_vdm_hdmi_ddc_read_entry(usb_pd_port_t *dev)
{
	uint8_t len = 0;
	bool ack = true;

//	dev->ddc_reg_offset = DDC_HEADER_REG_OFFSET(ddc_hdr);
//	dev->ddc_reg_size = DDC_HEADER_REG_SIZE(ddc_hdr);
//	dev->ddc_sequence_num = DDC_HEADER_SEQ_NUM(ddc_hdr);
//	dev->ddc_slave_addr = ddc_slave_addr_lookup[DDC_HEADER_SLAVE_ADDR(ddc_hdr)];

	// Clear VDM buffer.
	memset(dev->vdm_buf, 0, sizeof(dev->vdm_buf));

	if (dev->ddc_slave_addr == EDID_SLAVE_ADDR)
	{
		len = (dev->ddc_sequence_num < 5) ? 24 : 8;

		memcpy(&dev->vdm_buf[1],
			   &dev->hdmi_edid[(dev->ddc_reg_offset * 128) + (dev->ddc_sequence_num * 24)],
			   len);
	}
#if defined ENABLE_HDMI_HDCP_SUPPORT && defined HDMI_DONGLE
	else if ((dev->ddc_slave_addr == HDCP_PRIMARY_SLAVE_ADDR) || 
			 (dev->ddc_slave_addr == HDCP_SECONDARY_SLAVE_ADDR))
	{
		if (dev->ddc_sequence_num == 0)
		{
			dev->hdcp_byte_cnt = 0;
			// Perform the I2C read to HDMI slave.
			hdmi_read(dev->port, dev->ddc_slave_addr, 0, dev->ddc_reg_offset, dev->hdcp_buf, dev->ddc_reg_size);
		}

		// Return a max of 24-bytes at a time.
		if (dev->ddc_reg_size >= dev->hdcp_byte_cnt)
		{
			len = MIN((dev->ddc_reg_size - dev->hdcp_byte_cnt), 24);
			memcpy(&dev->vdm_buf[1], &dev->hdcp_buf[dev->hdcp_byte_cnt], len);
			dev->hdcp_byte_cnt += len;
		}
	}
#endif
	else /* Unknown address */
	{
		ack = false;
	}

	CRIT("slv_addr = 0x%x, reg_off = %u, reg_size = %u, seq_num = %u\n", dev->ddc_slave_addr, dev->ddc_reg_offset, dev->ddc_reg_size, dev->ddc_sequence_num);

	// Build header.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, (ack) ? VDM_RESP_ACK : VDM_RESP_NACK, VDM_CMD_HDMI_DDC_READ); 

	// Tx length is 1 for the VDM header + number of VDOs.
	usb_pd_pe_tx_vendor_msg(dev->port, (len >> 2) + 1, TCPC_TX_SOP);

	return;
}

static void pe_resp_vdm_hdmi_ddc_write_entry(usb_pd_port_t *dev)
{
	bool ack = false;
#if defined ENABLE_HDMI_HDCP_SUPPORT && defined HDMI_DONGLE
	uint8_t len = (dev->rx_msg_data_len - 8); /* minus 8-bytes for VDM header and DDC write header */

	if ((dev->ddc_slave_addr == HDCP_PRIMARY_SLAVE_ADDR) || 
		(dev->ddc_slave_addr == HDCP_SECONDARY_SLAVE_ADDR))
	{
		if (dev->ddc_sequence_num == 0)
		{
			dev->hdcp_byte_cnt = 0;
		}

		if ((dev->hdcp_byte_cnt + len) < HDCP_BUF_SIZE)
		{
			memcpy(&dev->hdcp_buf[dev->hdcp_byte_cnt], &dev->msg_buf[8], len);

			dev->hdcp_byte_cnt += len;

			if (dev->hdcp_byte_cnt >= dev->ddc_reg_size)
			{
				// Perform the I2C write to HDMI slave.
				hdmi_write(dev->port, dev->ddc_slave_addr, dev->ddc_reg_offset, dev->hdcp_buf, dev->ddc_reg_size);
			}

			ack = true;
		}
	}
#endif

	// Build header.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, (ack) ? VDM_RESP_ACK : VDM_RESP_NACK, VDM_CMD_HDMI_DDC_WRITE); 

	// Tx length is 1 for the VDM header.
	usb_pd_pe_tx_vendor_msg(dev->port, 1, TCPC_TX_SOP);

	return;
}

static void vdm_handle_hdmi_status_update(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	uint32_t vdo;

	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP responds to DP Status Update.  
		// UFP doesn't need to use any of the data sent by the DFP.
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			pe_set_state(dev, PE_RESP_VDM_HDMI_STATUS_UPDATE);
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_HDMI_STATUS_UPDATE)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			vdo = get_data_object(dev->msg_buf + 4);

			// If HPD was previously low or EDID was not available.
			if (!(dev->hdmi_status & HPD_STATUS) || !(dev->hdmi_status & EDID_AVAILABLE))
			{
				if (vdo & (HPD_STATUS | EDID_AVAILABLE))
				{
					// Request to read the EDID.
					dev->ddc_slave_addr = EDID_SLAVE_ADDR;
					dev->ddc_reg_offset = 0;  /* EDID block zero */
					dev->ddc_reg_size = 128;
					dev->ddc_sequence_num = 0;
					pe_set_state(dev, PE_INIT_VDM_HDMI_DDC_READ);
				}
			}
			else if (!(dev->hdmi_status & CEC_AVAILABLE))
			{
				if (vdo & CEC_AVAILABLE)
				{
					// BQ - set flag to read CEC later.
				}
			}

			// Save the new status.
			dev->hdmi_status = vdo;
		}
		else /* NAK or BUSY */
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
	}

	return;
}

static void pe_init_vdm_hdmi_status_update_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	// Only a DFP_U can initiate messages other than Attention command.
	if ((dev->active_alt_modes & HDMI_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP))
	{
		// Build header.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_INITIATOR, VDM_CMD_HDMI_STATUS_UPDATE); 

		// Build source status. (Cable adapter status request = 00b, CEC logical address not allocated = 0b)
		dev->vdm_buf[1] = 0;

		// Tx length is 1 for the VDM header + 1 for the status.
		usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);
	}

	return;
}

static void pe_init_vdm_hdmi_ddc_read_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	// Only a DFP_U can initiate messages other than Attention command.
	if ((dev->active_alt_modes & HDMI_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP))
	{
		// Build header.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_INITIATOR, VDM_CMD_HDMI_DDC_READ); 

		// Populate DDC read header.
		dev->vdm_buf[1] = DDC_HDR_GEN(dev->ddc_reg_offset, dev->ddc_reg_size, dev->ddc_sequence_num, dev->ddc_slave_addr);

		// Tx length is 1 for the VDM header + 1 for the DDC read header.
		usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);
	}

	return;
}

static void pe_init_vdm_hdmi_ddc_write_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	// Only a DFP_U can initiate messages other than Attention command.
	if ((dev->active_alt_modes & HDMI_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP))
	{
		// Build header.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_INITIATOR, VDM_CMD_HDMI_DDC_WRITE); 

		// Populate DDC write header.
		dev->vdm_buf[1] = DDC_HDR_GEN(dev->ddc_reg_offset, dev->ddc_reg_size, dev->ddc_sequence_num, dev->ddc_slave_addr);

		// Tx length is 1 for the VDM header + 1 for the DDC read header.
		usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);
	}

	return;
}


static void vdm_handle_hdmi_ddc_read(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	uint8_t len = MIN((dev->rx_msg_data_len - 4), 24); /* Each VDM is limited to 6 VDOs or 24-bytes of data */
	uint32_t ddc_hdr;
	uint32_t i, j;

	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP responds to DDC read.  
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			ddc_hdr = get_data_object(dev->msg_buf + 4);
			dev->ddc_reg_offset = DDC_HEADER_REG_OFFSET(ddc_hdr);
			dev->ddc_reg_size = DDC_HEADER_REG_SIZE(ddc_hdr);
			dev->ddc_sequence_num = DDC_HEADER_SEQ_NUM(ddc_hdr);
			dev->ddc_slave_addr = ddc_slave_addr_lookup[DDC_HEADER_SLAVE_ADDR(ddc_hdr)];

			pe_set_state(dev, PE_RESP_VDM_HDMI_DDC_READ);
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_HDMI_DDC_READ)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			if (dev->ddc_slave_addr == EDID_SLAVE_ADDR)
			{
				// Each VDM is limited to 6 VDOs or 24-bytes of data.
				// Each EDID block is 128-bytes so requires 6 DDC reads.
				memcpy(&dev->hdmi_edid[(dev->ddc_reg_offset * 128) + (dev->ddc_sequence_num * 24)],
					   &dev->msg_buf[4], len);

				if (++dev->ddc_sequence_num <= 5)
				{
					// Read next sequence.
					pe_set_state(dev, PE_INIT_VDM_HDMI_DDC_READ);
				}
				else
				{
					if (dev->ddc_reg_offset == 0)
					{
						dev->edid_ext_blocks = MIN(dev->hdmi_edid[0x7E], NUM_EDID_BLOCKS);
					}
					else
					{
						dev->edid_ext_blocks--;
					}

					if (dev->edid_ext_blocks)
					{
						// Go to next EDID block.
						dev->ddc_reg_offset++;
						dev->ddc_sequence_num = 0;
						pe_set_state(dev, PE_INIT_VDM_HDMI_DDC_READ);
					}
				}

//				PRINT("ddc_reg_offset = %u, seq_num = %u\n", dev->ddc_reg_offset, dev->ddc_sequence_num);

				if (!dev->state_change)
				{
					for (j = 0; j < (dev->hdmi_edid[0x7E] + 1); j++)
					{
						PRINT("EDID[%u] = 0x", j);
						for (i = 0; i < 128; i++)
						{
							PRINT("%02x ", dev->hdmi_edid[i+(j*128)]);
						}
						PRINT("\n");
					}

				}
			}
#ifdef ENABLE_HDMI_HDCP_SUPPORT
			else if ((dev->ddc_slave_addr == HDCP_PRIMARY_SLAVE_ADDR) || 
					 (dev->ddc_slave_addr == HDCP_SECONDARY_SLAVE_ADDR))
			{
				if (dev->ddc_sequence_num == 0)
				{
					dev->hdcp_byte_cnt = 0;
				}

				memcpy(&dev->hdcp_buf[dev->hdcp_byte_cnt], &dev->vdm_buf[1], len);

				dev->hdcp_byte_cnt += len;

				if (dev->hdcp_byte_cnt == dev->ddc_reg_size)
				{
					// Read is complete.
					// Set RxStatus (0x70) to READY (bit 10) with the number of 
					// bytes available in bits 9-0. - BQ
				}
				else
				{
					// Read next chuck.
					dev->ddc_sequence_num++;
					pe_set_state(dev, PE_INIT_VDM_HDMI_DDC_READ);
				}
			}
#endif
		}
		else /* NAK or BUSY */
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
	}

	return;
}

static void vdm_handle_hdmi_ddc_write(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
#ifdef ENABLE_HDMI_HDCP_SUPPORT
	uint8_t len;
#endif
	uint32_t ddc_hdr;

	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP responds to DDC write.  
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			ddc_hdr = get_data_object(dev->msg_buf + 4);
			dev->ddc_reg_offset = DDC_HEADER_REG_OFFSET(ddc_hdr);
			dev->ddc_reg_size = DDC_HEADER_REG_SIZE(ddc_hdr);
			dev->ddc_sequence_num = DDC_HEADER_SEQ_NUM(ddc_hdr);
			dev->ddc_slave_addr = ddc_slave_addr_lookup[DDC_HEADER_SLAVE_ADDR(ddc_hdr)];

			pe_set_state(dev, PE_RESP_VDM_HDMI_DDC_WRITE);
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_HDMI_DDC_WRITE)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
#ifdef ENABLE_HDMI_HDCP_SUPPORT
			if ((dev->ddc_slave_addr == HDCP_PRIMARY_SLAVE_ADDR) || 
				(dev->ddc_slave_addr == HDCP_SECONDARY_SLAVE_ADDR))
			{
				if (dev->ddc_sequence_num == 0)
				{
					dev->hdcp_byte_cnt = 0;
				}

				if (dev->hdcp_byte_cnt < dev->ddc_reg_size)
				{
					/* Each VDM is limited to 5 VDOs or 20-bytes of data */
					len = MIN(20, (dev->ddc_reg_size - dev->hdcp_byte_cnt));
					memcpy(&dev->vdm_buf[2], &dev->hdcp_buf[dev->hdcp_byte_cnt], len);
					dev->hdcp_byte_cnt += len;
				}

				if (dev->hdcp_byte_cnt == dev->ddc_reg_size)
				{
					// Write is complete.
					// Notify HDMI source. - BQ
				}
				else
				{
					// Read next chuck.
					dev->ddc_sequence_num++;
					pe_set_state(dev, PE_INIT_VDM_HDMI_DDC_WRITE);
				}
			}
#endif
		}
		else /* NAK or BUSY */
		{
			// Do nothing.
		}
	}

	return;
}

#ifdef HDMI_DONGLE

#define HPD_DEBOUNCE_MS  250

void timeout_hpd_debounce(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];
	uint8_t val = tcpm_get_hpd_in(port);

	if (dev->hpd_in_value == val)
	{
		if (dev->hpd_in_value != dev->hpd_in_previous_value)
		{
			dev->hpd_in_changed = true;
			dev->hpd_in_previous_value = dev->hpd_in_value;
		}
	}
	else
	{
		CRIT("*HPD: %u\n", val);
		dev->hpd_in_value = val;

		// Re-Start debounce.
		timer_start(&dev->hdmi_timer, HPD_DEBOUNCE_MS, timeout_hpd_debounce);
	}

	return;
}

void usb_pd_pe_hpd_in_event(unsigned int port, uint8_t val)
{
	usb_pd_port_t *dev = &pd[port];

	CRIT("HPD: %u\n", val);
	dev->hpd_in_value = val;

	// Start debounce.
	timer_start(&dev->hdmi_timer, HPD_DEBOUNCE_MS, timeout_hpd_debounce);

	return;
}
#endif

#endif /* ENABLE_HDMI_ALT_MODE_SUPPORT */


static void pe_init_vdm_svids_request_entry(usb_pd_port_t *dev)
{
	// Send Discover SVIDs VDM to port partner.
	vdm_send_structured_cmd(dev->port, VDM_PD_SID, VDM_CMD_DISCOVER_SVIDS, 0, TCPC_TX_SOP);
	return;
}

static void pe_init_vdm_modes_request_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	// Implement Alt-Modes here in order of priority.

#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	// Only a receptacle-based DFP_U can initiate messages other than Attention command.
	// BQ - violating this to allow plug-based DP dongle to negotiate alt mode.
	if ((dev->matched_alt_modes & DP_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP)/* && 
		(config->modes[find_svid_index(dev, VDM_DISPLAYPORT_VID)] & RECEPTACLE_INDICATION_BIT)*/)
	{
		// Send Discover Modes VDM to port partner.
		vdm_send_structured_cmd(dev->port, VDM_DISPLAYPORT_VID, VDM_CMD_DISCOVER_MODES, 0, TCPC_TX_SOP);
		return;
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	if ((dev->matched_alt_modes & HDMI_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP))
	{
		// Send Discover Modes VDM to port partner.
		vdm_send_structured_cmd(dev->port, VDM_HDMI_VID, VDM_CMD_DISCOVER_MODES, 0, TCPC_TX_SOP);
		return;
	}
#endif
	// [VDM] Add other supported modes here...

	return;
}


/* used by UFP_U to signal change in status */
static void pe_init_vdm_attention_request_entry(usb_pd_port_t *dev)
{
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	// Only a UFP_U (plug or receptacle) can issue an Attention command in DP mode.  
	if ((dev->active_alt_modes & DP_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_UFP))
	{
		usb_pd_pe_update_displayport_status(dev);

		// Send Attention VDM to port partner.
		vdm_send_dp_status(dev, VDM_INITIATOR, VDM_CMD_ATTENTION);
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	// Only a UFP_U (plug or receptacle) can issue an Attention command in HDMI mode.
	if ((dev->active_alt_modes & HDMI_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_UFP))
	{
		// Send Attention VDM to port partner.
		// Build header.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_INITIATOR, VDM_CMD_ATTENTION);

		dev->vdm_buf[1] = dev->hdmi_status;

		// Tx length is 1 for the VDM header plus 1 for status.
		usb_pd_pe_tx_vendor_msg(dev->port, 2, TCPC_TX_SOP);
	}
#endif
	// [VDM] Add other supported alt modes here...

	return;
}


static void pe_resp_vdm_nak_entry(usb_pd_port_t *dev)
{
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);

	// NACK if not supported.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(USB_PD_VDM_HDR_GET_SVID(vdm_hdr), 
										 VDM_TYPE_STRUCTURED,
										 (dev->rx_sop == TCPC_TX_SOP) ? dev->p2p_vdm_version : dev->p2c_vdm_version, 
										 USB_PD_VDM_HDR_GET_OBJ_POS(vdm_hdr), 
										 VDM_RESP_NACK, 
										 USB_PD_VDM_HDR_GET_CMD(vdm_hdr)); 

	usb_pd_pe_tx_vendor_msg(dev->port, 1, (tcpc_transmit_t)dev->rx_sop);
	return;
}


static void pe_resp_vdm_get_svids_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t i;
	uint8_t num_vdos;
	uint16_t *svid = (uint16_t*)&dev->vdm_buf[1];

	// Clear VDM buffer.
	memset(dev->vdm_buf, 0, sizeof(dev->vdm_buf));

	if (config->num_svids)
	{
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_PD_SID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 0, VDM_RESP_ACK, VDM_CMD_DISCOVER_SVIDS); 

		for (i = 0; i < config->num_svids; i++)
		{
			// Put odd number SVIDs in the lower 16-bits of the VDO
			// and even number SVIDs i the upper 16-bits of the VDO.
			if (i & 0x1)
			{
				svid[i-1] = config->svids[i];
			}
			else
			{
				svid[i+1] = config->svids[i];
			}
		}
	}
	else
	{
		// Return NAK if no SVIDs.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_PD_SID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 0, VDM_RESP_NACK, VDM_CMD_DISCOVER_SVIDS); 
	}

	// Determine the number of data objects for the SVIDs.
	num_vdos = (config->num_svids + 1) >> 1;

	if (config->num_svids && !(config->num_svids & 0x01))
	{
		// If even number of SVIDs, we need to send an extra VDO with all zeros.
		num_vdos++;
	}

	// Tx length is 1 for the VDM header + number of VDOs.
	usb_pd_pe_tx_vendor_msg(dev->port, (1 + num_vdos), (tcpc_transmit_t)dev->rx_sop);

	return;
}

static void pe_resp_vdm_get_modes_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t svid_idx;
	uint8_t num_modes = 0;
	uint16_t svid;
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);

	svid = USB_PD_VDM_HDR_GET_SVID(vdm_hdr);

	// Search for matching SVID.
	for (svid_idx = 0; svid_idx < config->num_svids; svid_idx++)
	{
		if (svid == config->svids[svid_idx])
		{
			// Currently, only one mode per SVID is supported.
			dev->vdm_buf[1] = config->modes[svid_idx];
			num_modes = 1;
			break;
		}
	}

	// Build header.  Return NAK if no modes supported.
	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(svid, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 0, (num_modes) ? VDM_RESP_ACK : VDM_RESP_NACK, VDM_CMD_DISCOVER_MODES); 

	// Tx length is 1 for the VDM header + number of modes.
	usb_pd_pe_tx_vendor_msg(dev->port, (1 + num_modes), (tcpc_transmit_t)dev->rx_sop);

	return;
}

static void pe_resp_vdm_get_identity_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t i;
	uint32_t *prod_type_vdos = &dev->vdm_buf[4];

	dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_PD_SID, 
										 VDM_TYPE_STRUCTURED, 
										 (dev->rx_sop == TCPC_TX_SOP) ? dev->p2p_vdm_version : dev->p2c_vdm_version,
										 0, VDM_RESP_ACK, VDM_CMD_DISCOVER_IDENTITY); 
	dev->vdm_buf[1] = config->id_header_vdo;
	dev->vdm_buf[2] = config->cert_stat_vdo;
	dev->vdm_buf[3] = config->product_vdo;

	if (((dev->rx_sop == TCPC_TX_SOP) && (dev->p2p_vdm_version < STRUCT_VDM_VER_2P0)) ||
		((dev->rx_sop == TCPC_TX_SOP_P) && (dev->p2c_vdm_version < STRUCT_VDM_VER_2P0)))
	{
		// Ensure reserved bits of ID header are zero.
		dev->vdm_buf[1] &= 0xFC00FFFF; 
	}

	for (i = 0; i < config->num_product_type_vdos; i++)
	{
		prod_type_vdos[i] = config->product_type_vdos[i];
	}

	usb_pd_pe_tx_vendor_msg(dev->port, (4 + config->num_product_type_vdos), (tcpc_transmit_t)dev->rx_sop);

	return;
}

static void timeout_send_src_cap(unsigned int port)
{
	pd[port].send_src_cap_timeout = true;
	return;
}

#define ID_HDR_GET_CABLE_PLUG_PROD_TYPE(id_hdr)  (((id_hdr) >> 27) & 0x7)
#define ID_HDR_GET_UFP_PROD_TYPE(id_hdr)  (((id_hdr) >> 27) & 0x7)
#define ID_HDR_GET_DFP_PROD_TYPE(id_hdr)  (((id_hdr) >> 23) & 0x7)

typedef enum 
{
	CABLE_PROD_TYPE_PASSIVE_CABLE = 3,
	CABLE_PROD_TYPE_ACTIVE_CABLE = 4,
} cable_product_type_t;

#define CURRENT_CAP_5A_BIT      (1 << 6)
#define VBUS_THROUGH_CABLE_BIT  (1 << 4)
#define CABLE_VDO_GET_MAX_VBUS_VOLTAGE_MV(vdo)  (((((vdo) >> 9) & 0x3) * 10000) + 20000);

typedef enum 
{
	UFP_PROD_TYPE_PD_USB_HUB = 1,
	UFP_PROD_TYPE_PD_USB_PERIPHERAL = 2,
	UFP_PROD_TYPE_ALT_MODE_ADAPTER = 5,
} ufp_prod_type_t;

typedef enum 
{
	DFP_PROD_TYPE_PD_USB_HUB = 1,
	DFP_PROD_TYPE_PD_USB_HOST = 2,
	DFP_PROD_TYPE_POWER_BRICK = 3,
	DFP_PROD_TYPE_ALT_MODE_CTRL = 4,
} dfp_prod_type_t;

#define MODAL_OPERATION_SUPPORTED ((uint32_t)1 << 26)

static void vdm_handle_discover_identity(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	uint32_t id_hdr;
	uint32_t prod_type_vdo;
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);
	uint8_t vdm_ver = USB_PD_VDM_HDR_GET_VER(vdm_hdr);

	if (cmd_type == VDM_INITIATOR)
	{
		if (dev->rx_sop == TCPC_TX_SOP)
		{
			// For PD2.0, DFP must respond with NAK.
			if ((dev->data_role == PD_DATA_ROLE_DFP) && (dev->p2p_spec_rev == PD_REV20))
			{
				pe_set_state(dev, PE_RESP_VDM_NAK);
			}
			else
			{
				pe_set_state(dev, PE_RESP_VDM_GET_IDENTITY);
			}
		}
#ifdef CABLE_PLUG
		else if (dev->rx_sop == TCPC_TX_SOP_P)
		{
			pe_set_state(dev, PE_RESP_VDM_GET_IDENTITY);
		}
#endif
	}
	else if ((*dev->current_state == PE_SRC_VDM_IDENTITY_REQUEST) && 
			 (dev->rx_sop == TCPC_TX_SOP_P))
	{
		dev->cable_id_req_complete = true;

		// Cancel the send source cap timer.
		if (dev->timer2.function == timeout_send_src_cap)
		{
			tusb422_lfo_timer_cancel(&dev->timer2);
		}

		// Handle Discovery Identity response from cable.
		if (cmd_type == VDM_RESP_ACK)
		{
			// Save port to cable operating spec revision.
			dev->p2c_spec_rev = dev->rx_msg_spec_rev;

			// Interpret Cable VDO.
			id_hdr = get_data_object(dev->msg_buf + 4);
			prod_type_vdo = get_data_object(dev->msg_buf + 16);

			if (ID_HDR_GET_CABLE_PLUG_PROD_TYPE(id_hdr) == CABLE_PROD_TYPE_PASSIVE_CABLE)
			{
				if (prod_type_vdo & CURRENT_CAP_5A_BIT)
				{
					dev->cable_max_current_ma = 5000;
				}
			}
			else if (ID_HDR_GET_CABLE_PLUG_PROD_TYPE(id_hdr) == CABLE_PROD_TYPE_ACTIVE_CABLE)
			{
				dev->active_cable = true;

				if (prod_type_vdo & VBUS_THROUGH_CABLE_BIT)
				{
					if (prod_type_vdo & CURRENT_CAP_5A_BIT)
					{
						dev->cable_max_current_ma = 5000;
					}
				}
				else
				{
					dev->cable_max_current_ma = 0;
				}
			}

			// Check cable SuperSpeed signaling support. - BQ

			if (vdm_ver > STRUCT_VDM_VER_1P0)
			{
				dev->cable_max_voltage_mv = CABLE_VDO_GET_MAX_VBUS_VOLTAGE_MV(prod_type_vdo);
			}

			CRIT("Cable supports %uV %uA\n", dev->cable_max_voltage_mv / 1000,
				 dev->cable_max_current_ma / 1000);

			pe_set_state(dev, PE_SRC_SEND_CAPS);
		}
		else if (cmd_type == VDM_RESP_BUSY)
		{
			if (dev->discover_identity_cnt < (N_DISCOVER_IDENTITY_COUNT - 1))
			{
				// Retry after tVDMBusy.
				dev->vdm_pending_state = PE_SRC_DISCOVERY;
				timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
			}
			else
			{
				// Go directly to PE_SRC_DISCOVERY which will start PE_SRC_SEND_CAPS.
				pe_set_state(dev, PE_SRC_DISCOVERY);
			}
		}
		else /* NACK */
		{
			pe_set_state(dev, PE_SRC_DISCOVERY);
		}
	}
	else if ((*dev->current_state == PE_INIT_PORT_VDM_IDENTITY_REQUEST) && 
			 (dev->rx_sop == TCPC_TX_SOP))
	{
		// Mark request as complete.
		dev->port_partner_id_req_complete = true;

		// Save supported port to port structured VDM version.
		dev->p2p_vdm_version = vdm_ver;

		// Handle Discovery Identity response from cable.
		if (cmd_type == VDM_RESP_ACK)
		{
			id_hdr = get_data_object(dev->msg_buf + 4);

			// If product type is alt mode adapter.
			if (ID_HDR_GET_UFP_PROD_TYPE(id_hdr) & UFP_PROD_TYPE_ALT_MODE_ADAPTER)
			{
				// Get AMA VDO.
				prod_type_vdo = get_data_object(dev->msg_buf + 16);

				// Check AMA SuperSpeed signaling support. - BQ
			}

			if (id_hdr & MODAL_OPERATION_SUPPORTED)
			{
				pe_set_state(dev, PE_INIT_VDM_SVIDS_REQUEST);
			}
		}
		else if (cmd_type == VDM_RESP_BUSY)
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
		else /* NACK */
		{
			// Do not retry.
		}
	}

	return;
}

static uint16_t get_svid(uint32_t vdo, uint8_t svid_num)
{
	return(svid_num == 0) ? (vdo >> 16) : (vdo & 0xFFFF);
}

static void vdm_handle_discover_svids(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	bool last_svid = false;
	bool svid_matched = false;
	uint16_t svid;
	uint32_t vdo;
	uint8_t *vdo_ptr = dev->msg_buf + 4;
	uint8_t num_vdos = (dev->rx_msg_data_len >> 2) - 1;
	uint8_t svid_idx, vdo_idx, svid_num;
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	if (cmd_type == VDM_INITIATOR)
	{
		// Only valid for SOP/SOP'.
		if (dev->rx_sop <= TCPC_TX_SOP_P)
		{
			// For PD2.0, DFP must respond with NAK.
			if ((dev->data_role == PD_DATA_ROLE_DFP) && (dev->p2p_spec_rev == PD_REV20))
			{
				pe_set_state(dev, PE_RESP_VDM_NAK);
			}
			else
			{
				pe_set_state(dev, PE_RESP_VDM_GET_SVIDS);
			}
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_SVIDS_REQUEST)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			// Check SVIDs for any we support.  
			for (svid_idx = 0; (svid_idx < config->num_svids) && !last_svid; svid_idx++)
			{
				for (vdo_idx = 0; (vdo_idx < num_vdos) && !last_svid; vdo_idx++)
				{
					vdo = get_data_object(vdo_ptr + (vdo_idx << 2));

					// Two SVIDs per VDO.
					for (svid_num = 0; svid_num < 2; svid_num++)
					{
						svid = get_svid(vdo, svid_num);

						INFO("SVID = 0x%04x\n", svid);

						if (svid == 0)
						{
							last_svid = true;
							break;
						}
						else if (svid == config->svids[svid_idx])
						{
							// Matched SVID to one we support.
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
							if (svid == VDM_DISPLAYPORT_VID)
							{
								dev->matched_alt_modes |= DP_ALT_MODE;
								svid_matched = true;
							}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
							if (svid == VDM_HDMI_VID)
							{
								dev->matched_alt_modes |= HDMI_ALT_MODE;
								svid_matched = true;
							}
#endif
							// [VDM] Add other supported Alt modes here...
						}
					}
				}
			}

			if (!last_svid)
			{
				pe_set_state(dev, PE_INIT_VDM_SVIDS_REQUEST);
			}
		}
		else if (cmd_type == VDM_RESP_BUSY)
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
	}

	if (svid_matched)
	{
		pe_set_state(dev, PE_INIT_VDM_MODES_REQUEST);
	}

	return;
}

static void vdm_handle_discover_modes(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	uint32_t mode;
	uint8_t *mode_ptr = dev->msg_buf + 4; /* add 4 to skip VDM header */
	uint8_t mode_idx;
	uint8_t num_modes = (dev->rx_msg_data_len >> 2) - 1; /* subtract 1 for the VDM header */
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t remote_pin_assignments;

	INFO("num_modes = %u\n", num_modes);

	if (cmd_type == VDM_INITIATOR)
	{
		// Only valid for SOP/SOP'.
		if (dev->rx_sop <= TCPC_TX_SOP_P)
		{
			// For PD2.0, DFP must respond with NAK.
			if ((dev->data_role == PD_DATA_ROLE_DFP) && (dev->p2p_spec_rev == PD_REV20))
			{
				pe_set_state(dev, PE_RESP_VDM_NAK);
			}
			else
			{
				pe_set_state(dev, PE_RESP_VDM_GET_MODES);
			}
		}
	}
	else if (*dev->current_state == PE_INIT_VDM_MODES_REQUEST)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			for (mode_idx = 0; mode_idx < num_modes; mode_idx++)
			{
				mode = get_data_object(mode_ptr + (mode_idx << 2));

#ifdef ENABLE_DP_ALT_MODE_SUPPORT
				if (config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID)
				{
					dev->displayport_remote_caps = mode;

					if (mode & RECEPTACLE_INDICATION_BIT)
					{
						remote_pin_assignments = UFP_D_PIN_ASSIGNMENTS(mode);
					}
					else
					{
						remote_pin_assignments = DFP_D_PIN_ASSIGNMENTS(mode);
					}

					// Check if UFP_U capabilities and at least one pin assignment matches ours.
					// Assume we are DFP_D or UFP_D capable but not both.
					if ((config->modes[dev->svid_index] & DFP_D_CAPABLE_BIT) && 
						(mode & UFP_D_CAPABLE_BIT))
					{
						dev->displayport_matched_pin_assignments = DFP_D_PIN_ASSIGNMENTS(config->modes[dev->svid_index]) & remote_pin_assignments;
					}
					else if ((config->modes[dev->svid_index] & UFP_D_CAPABLE_BIT) && 
							 (mode & DFP_D_CAPABLE_BIT))
					{
						dev->displayport_matched_pin_assignments = UFP_D_PIN_ASSIGNMENTS(config->modes[dev->svid_index]) & remote_pin_assignments;
					}

					if (dev->displayport_matched_pin_assignments)
					{
						pe_set_state(dev, PE_DFP_VDM_MODE_ENTRY_REQUEST);
					}
				}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
				if (config->svids[dev->svid_index] == VDM_HDMI_VID)
				{
					if (mode & HDMI_CABLE_ADAPTER_BIT)
					{
						pe_set_state(dev, PE_DFP_VDM_MODE_ENTRY_REQUEST);
					}
				}
#endif
				// [VDM] Add other supported modes here...
			}
		}
		else if (cmd_type == VDM_RESP_BUSY)
		{
			// Retry after tVDMBusy.
			dev->vdm_pending_state = *dev->current_state;
			timer_start(&dev->timer, T_VDM_BUSY_MS, timeout_vdm_busy);
		}
	}

	return;
}

static void pe_dfp_vdm_mode_entry_request(usb_pd_port_t *dev)
{
	// If multiple modes are matched, we must implement logic on which one(s)
	// should be entered. For now assume only one match. - BQ
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	if (dev->matched_alt_modes & DP_ALT_MODE)
	{
		// Object position is 1.  No VDO for DP Alt Mode.
		vdm_send_structured_cmd(dev->port, VDM_DISPLAYPORT_VID, VDM_CMD_ENTER_MODE, 1, TCPC_TX_SOP);
		return;
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	if (dev->matched_alt_modes & HDMI_ALT_MODE)
	{
		// Object position is 1.  No VDO for HDMI Alt Mode.
		vdm_send_structured_cmd(dev->port, VDM_HDMI_VID, VDM_CMD_ENTER_MODE, 1, TCPC_TX_SOP);
		return;
	}
#endif
	// [VDM] Add other supported modes here...

	return;
}

static void pe_dfp_vdm_mode_exit_request(usb_pd_port_t *dev)
{
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	if (dev->active_alt_modes & DP_ALT_MODE)
	{
		// DFP_U shall only issue Exit Mode when the port is configured to be in USB Config.
		if (dev->displayport_config != CONFIG_FOR_USB)
		{
			dev->mode_exit_pending = true;

			// Set config to USB mode.
			dev->displayport_config = CONFIG_FOR_USB;
			pe_set_state(dev, PE_INIT_VDM_DP_CONFIG);
		}
		else
		{
			// Object position is 1.
			vdm_send_structured_cmd(dev->port, VDM_DISPLAYPORT_VID, VDM_CMD_EXIT_MODE, 1, TCPC_TX_SOP);
		}
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	if (dev->active_alt_modes & HDMI_ALT_MODE)
	{
		// Object position is 1.
		vdm_send_structured_cmd(dev->port, VDM_HDMI_VID, VDM_CMD_EXIT_MODE, 1, TCPC_TX_SOP);
	}
#endif
	// [VDM] Add other supported modes here...

	return;
}


static void pe_ufp_vdm_eval_mode_entry(usb_pd_port_t *dev)
{
	bool mode_entry_ack = false;
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
#ifdef DP_DONGLE
	uint8_t aux_p;
	uint8_t aux_n;
	uint8_t hpd;
#endif

#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	// Object position must be 1 for DP Alt Mode.
	if ((config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID) &&
		(USB_PD_VDM_HDR_GET_OBJ_POS(vdm_hdr) == 1))
	{
		// Reset HPD_IN queue.
		dev->hpd_in_queue_idx = 0;
		dev->hpd_in_send_idx = 0;
		dev->hpd_in_queue[0] = NOTIFY_HPD_NONE;

		dev->active_alt_modes |= DP_ALT_MODE;

		// Populate the VDM header Object position is always 1.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_DISPLAYPORT_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_RESP_ACK, VDM_CMD_ENTER_MODE); 

		// Set flag so ACK will be sent.
		mode_entry_ack = true;
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	// Object position must be 1 for HDMI Alt Mode.
	if ((config->svids[dev->svid_index] == VDM_HDMI_VID) &&
		(USB_PD_VDM_HDR_GET_OBJ_POS(vdm_hdr) == 1))
	{
		dev->active_alt_modes |= HDMI_ALT_MODE;

		// Populate the VDM header Object position is always 1.
		dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, VDM_TYPE_STRUCTURED, dev->p2p_vdm_version, 1, VDM_RESP_ACK, VDM_CMD_ENTER_MODE); 

		// Configure pins for HDMI.
		tcpm_mux_control(dev->port, dev->data_role, MUX_HDMI, dev->tcpc_dev->plug_polarity);

		// Set flag so ACK will be sent.
		mode_entry_ack = true;
	}
#endif
	// [VDM] Add other supported modes here...

	if (mode_entry_ack)
	{
#if defined HDMI_DONGLE || defined DP_DONGLE
		if (dev->timer2.function == timeout_alt_mode_entry)
		{
			// Cancel tAMETimeout.
			tusb422_lfo_timer_cancel(&dev->timer2);
		}
#endif
#ifdef DP_DONGLE
		// This handles the case where DP is connected before Type-C.

		// Check DP connection status.
		tcpm_get_aux_state(dev->port, &aux_p, &aux_n);
		hpd = tcpm_get_hpd_in(dev->port);

		PRINT("AUX_P = %u, AUX_N = %u, HPD: %u\n", aux_p, aux_n, hpd);

		if ((aux_p && !aux_n) || hpd)
		{
			// If AUX_P is high or HPD is high, UFP_D is connected.
			if (!(dev->displayport_status & UFP_D_CONNECTED))
			{
				usb_pd_pe_update_dp_status(dev->port, DFP_D_CONNECTED, UFP_D_CONNECTED);
			}

			if (hpd)
			{
				usb_pd_pe_update_dp_status(dev->port, 0, HPD_STATE_HIGH);	

				// Drive HPD to redriver high.
				tcpm_redriver_hpd_ctrl(dev->port, 1);
			}
		}
		else if (aux_n && !aux_p)
		{
			// If AUX_N is high, DFP_D is connected.
			if (!(dev->displayport_status & DFP_D_CONNECTED))
			{
				usb_pd_pe_update_dp_status(dev->port, UFP_D_CONNECTED, DFP_D_CONNECTED);
			}

			// Drive HPD to DFP_D low until we receive DP status.
			// This also drives HPD to the redriver accordingly.
			tcpm_hpd_out_control(dev->port, HPD_LOW);
		}

#endif
		// Send the ACK. Tx length is 1 for the VDM header.
		usb_pd_pe_tx_vendor_msg(dev->port, 1, (tcpc_transmit_t)dev->rx_sop);
	}
	else /* NACK */
	{
		// Respond with NACK if not supported.
		pe_set_state(dev, PE_RESP_VDM_NAK);
	}

	return;
}

static void pe_ufp_vdm_mode_exit_entry(usb_pd_port_t *dev)
{
	bool mode_exit_ack = false;
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t obj_pos;

	obj_pos = USB_PD_VDM_HDR_GET_OBJ_POS(vdm_hdr);

#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	if ((config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID) && 
		(dev->active_alt_modes & DP_ALT_MODE))
	{
		// DisplayPort has 1 mode so object position must be 1 or 7 (all modes).
		if ((obj_pos == 1) || (obj_pos == 7))
		{
#ifdef DP_DONGLE
			// Check if redriver is powered.
			if (!(dev->displayport_status & ADAPTER_POWER_LOW))
			{
				// Disable the redriver to isolate SBU pins.
				tusb544_disable(dev->port);
			}

			dev->redriver_configured = false;
#endif
			dev->displayport_alt_mode_configured = false;
			dev->displayport_config = CONFIG_FOR_USB;

			// Configure pins for USB.
			tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);

			// Generate ACK.
			dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_DISPLAYPORT_VID, 
												 VDM_TYPE_STRUCTURED, 
												 dev->p2p_vdm_version, 
												 obj_pos, 
												 VDM_RESP_ACK, 
												 VDM_CMD_EXIT_MODE); 

			dev->active_alt_modes &= ~DP_ALT_MODE;

			mode_exit_ack = true;
		}
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	if ((config->svids[dev->svid_index] == VDM_HDMI_VID) && 
		(dev->active_alt_modes & HDMI_ALT_MODE))
	{
		// HDMI has 1 mode so object position must be 1 or 7 (all modes).
		if ((obj_pos == 1) || (obj_pos == 7))
		{
			// Configure pins for USB.
			tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);

			// Generate ACK.
			dev->vdm_buf[0] = USB_PD_VDM_HDR_GEN(VDM_HDMI_VID, 
												 VDM_TYPE_STRUCTURED, 
												 dev->p2p_vdm_version, 
												 obj_pos, 
												 VDM_RESP_ACK, 
												 VDM_CMD_EXIT_MODE); 

			dev->active_alt_modes &= ~HDMI_ALT_MODE;

			mode_exit_ack = true;
		}
	}
#endif
	// [VDM] Add other supported modes here...

	if (mode_exit_ack)
	{
		// Tx length is 1 for the VDM header.
		usb_pd_pe_tx_vendor_msg(dev->port, 1, (tcpc_transmit_t)dev->rx_sop);
	}
	else
	{
		// Respond with NACK.
		pe_set_state(dev, PE_RESP_VDM_NAK);
	}

	return;
}

static void vdm_handle_enter_mode(usb_pd_port_t *dev, vdm_command_type_t cmd_type, uint8_t obj_pos)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP and Cable Plug respond to Enter mode.
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			pe_set_state(dev, PE_UFP_VDM_EVAL_MODE_ENTRY);
		}
	}
	else if (*dev->current_state == PE_DFP_VDM_MODE_ENTRY_REQUEST)
	{
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
		if (config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID)
		{
			if (cmd_type == VDM_RESP_ACK)
			{
				// Configure pins for new mode.
				// For DP Alt-Mode, pins are configured when sending DP Config command so don't configure here.

				dev->active_alt_modes |= DP_ALT_MODE;

				pe_set_state(dev, PE_INIT_VDM_DP_STATUS_UPDATE);
			}
		}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
		if (config->svids[dev->svid_index] == VDM_HDMI_VID)
		{
			if (cmd_type == VDM_RESP_ACK)
			{
				dev->active_alt_modes |= HDMI_ALT_MODE;

				pe_set_state(dev, PE_INIT_VDM_HDMI_STATUS_UPDATE);
			}
		}
#endif
		// [VDM] Add other supported SVIDs here...
	}

	return;
}

static void timeout_vdm_mode_exit(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_HARD_RESET);
	}
	else
	{
		pe_set_state(dev, PE_SRC_HARD_RESET);
	}

	return;
}


static void vdm_handle_exit_mode(usb_pd_port_t *dev, vdm_command_type_t cmd_type, uint8_t obj_pos)
{
	if (cmd_type == VDM_INITIATOR)
	{
		// Only UFP and Cable Plug respond to Exit mode.
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			pe_set_state(dev, PE_UFP_VDM_MODE_EXIT);
		}
	}
	else if (*dev->current_state == PE_DFP_VDM_MODE_EXIT_REQUEST)
	{
		if (cmd_type == VDM_RESP_ACK)
		{
			// Configure pins for USB mode.
			tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);
		}
		else if (cmd_type == VDM_RESP_BUSY)
		{
			// Call timeout function to issue Hard Reset.
			timeout_vdm_mode_exit(dev->port);
		}
	}

	return;
}

static void vdm_handle_attention(usb_pd_port_t *dev, vdm_command_type_t cmd_type)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t num_vdos = (dev->rx_msg_data_len >> 2) - 1;	/* subtract 1 for the VDM header */
	uint8_t *vdo_ptr = dev->msg_buf + 4;	/* add 4 to skip VDM header */
	uint32_t vdo;
	uint32_t connected_status;

	// Log dummy state change because there is no response for Attention command.
	pe_set_state(dev, PE_RCV_VDM_ATTENTION_REQUEST);
	dev->state_change = false;

	// Attention command must be initiator type and SOP.
	if ((cmd_type != VDM_INITIATOR) || (dev->rx_sop != TCPC_TX_SOP))
	{
		return;
	}

	// Attention can have zero or one VDOs.
	if (num_vdos)
	{
		vdo = get_data_object(vdo_ptr);
	}

#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	// Only DFP_U accept attention from UFP_U and there should be 1 VDO.
	if ((config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID) &&
		(dev->active_alt_modes &= DP_ALT_MODE) &&
		(dev->data_role == PD_DATA_ROLE_DFP) && 
		num_vdos)
	{
		connected_status = vdo & (UFP_D_CONNECTED | DFP_D_CONNECTED);

		// Check for connection status change to connected.
		if ((connected_status != (dev->displayport_remote_status & (UFP_D_CONNECTED | DFP_D_CONNECTED))) &&
			connected_status)
		{
			dev->displayport_config = (connected_status & UFP_D_CONNECTED) ? CONFIG_UFP_U_AS_UFP_D : CONFIG_UFP_U_AS_DFP_D;

			pe_set_state(dev, PE_INIT_VDM_DP_CONFIG);
		}

		// Exit request trumps a connected status change.
		if (vdo & REQ_DP_MODE_EXIT)
		{
			pe_set_state(dev, PE_DFP_VDM_MODE_EXIT_REQUEST);
		}
		else if (vdo & REQ_USB_CONFIG)
		{
			// USB config request trumps a DP config due to connected status change.
			dev->displayport_config = CONFIG_FOR_USB;
			pe_set_state(dev, PE_INIT_VDM_DP_CONFIG);
		}

		// DP alt mode is configured, report HPD status.
		if (dev->displayport_alt_mode_configured)
		{
			if (vdo & HPD_STATE_HIGH)
			{
				// Assert HPD to DFP_D.
				tcpm_hpd_out_control(dev->port, HPD_HIGH);

#ifndef CONFIG_TUSB422
				if ((vdo & IRQ_HPD) && !(dev->displayport_remote_status & HPD_STATE_HIGH))
				{
					// If HPD high and IRQ are reported together, make sure HPD is high for at least 2ms.
					tcpm_msleep(4);
				}
#endif
			}
			else
			{
				// Deassert HPD to DFP_D.
				tcpm_hpd_out_control(dev->port, HPD_LOW);

#ifndef CONFIG_TUSB422
				// Ensure HPD is low for at least 3ms in case a HPD high event follows immediately.
				tcpm_msleep(4);
#endif
			}

			if (vdo & IRQ_HPD)
			{
				// Pulse HPD pin low for 1ms.
				tcpm_hpd_out_control(dev->port, HPD_IRQ);
			}

			/*** TUSB422 PG1.x workaround for VCONN disabled during PR swap (CDDS #77) ***/
			// Attempt error recovery if adapter is reporting low power status (i.e. DisplayPort disabled)
			// when it was previously powered and VCONN is enabled and PG1.x silicon. This may occur
			// after plugging external power into an alt-mode dock/dongle resulting in a power role swap.
			if ((vdo & ADAPTER_POWER_LOW) &&
				!(dev->displayport_remote_status & ADAPTER_POWER_LOW) &&
				tcpm_is_vconn_enabled(dev->port) &&
				(dev->tcpc_dev->silicon_revision <= 2))
			{
				CRIT("DP adapter power is low! Attempting recovery.\n");
				// Temporarily force the power role during the error recovery to ensure it is maintained.
				tcpm_change_role(dev->port, (dev->power_role == PD_PWR_ROLE_SNK) ? ROLE_SNK : ROLE_SRC, true);
			}
		}

		// Save the new status.
		dev->displayport_remote_status = vdo;
	}
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	if ((config->svids[dev->svid_index] == VDM_HDMI_VID) &&
		(dev->active_alt_modes &= HDMI_ALT_MODE))
	{
		pe_set_state(dev, PE_INIT_VDM_HDMI_STATUS_UPDATE);
	}
#endif
	// [VDM] Add other supported SVIDs here...

	return;
}


static void usb_pd_pe_structured_vdm_handler(usb_pd_port_t *dev)
{
	bool valid_svid = false;
	bool valid_cmd = true;
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);
	uint8_t svid_idx;
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);
	vdm_command_type_t cmd_type;
	uint8_t cmd;
	uint8_t svid_specific_cmd_limit;
	uint8_t vdm_ver;

	cmd = USB_PD_VDM_HDR_GET_CMD(vdm_hdr);

	// Stop any VDM timer running. (e.g. VDMResponseTimer, VDMModeEntryTimer, VDMModeExitTimer)
	timer_cancel(&dev->timer);

	if (!dev->explicit_contract && 
		!((cmd == VDM_CMD_DISCOVER_IDENTITY) &&
		  (dev->rx_sop == TCPC_TX_SOP_P)))
	{
		// Ignore if no explicit contract in place with the exception of 
		// Discovery Identity command sent to cable plug. 
		return;
	}

	// Validate SVID.
	if ((cmd == VDM_CMD_DISCOVER_IDENTITY) || (cmd == VDM_CMD_DISCOVER_SVIDS))
	{
		if (USB_PD_VDM_HDR_GET_SVID(vdm_hdr) == VDM_PD_SID)
		{
			valid_svid = true;
		}
	}
	else
	{
		svid_idx = find_svid_index(dev, USB_PD_VDM_HDR_GET_SVID(vdm_hdr));

		if (svid_idx < config->num_svids)
		{
			dev->svid_index = svid_idx;
			valid_svid = true;

			if (config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID)
			{
				svid_specific_cmd_limit = VDM_CMD_DP_LIMIT;
			}
			else if (config->svids[dev->svid_index] == VDM_HDMI_VID)
			{
				svid_specific_cmd_limit = VDM_CMD_HDMI_LIMIT;
			}
			else
			{
				svid_specific_cmd_limit = VDM_CMD_SVID_SPECIFIC_MIN;
			}

			if (!((cmd >= VDM_CMD_DISCOVER_MODES) && (cmd <= VDM_CMD_ATTENTION)) &&
				((cmd < VDM_CMD_SVID_SPECIFIC_MIN) || (cmd >= svid_specific_cmd_limit)))
			{
				valid_cmd = false;
			}
		}
		else
		{
			valid_cmd = false;
		}
	}

	if (valid_svid && valid_cmd)
	{
		vdm_ver = USB_PD_VDM_HDR_GET_VER(vdm_hdr);

		if (dev->rx_sop == TCPC_TX_SOP)
		{
			// Use highest commonly supportedly port to port structured VDM version.
			if (dev->p2p_vdm_version > vdm_ver)
			{
				dev->p2p_vdm_version = vdm_ver;
			}
		}
		else if (dev->rx_sop == TCPC_TX_SOP_P)
		{
			// Use highest commonly supportedly port to cable structured VDM version.
			if (dev->p2c_vdm_version > vdm_ver)
			{
				dev->p2c_vdm_version = vdm_ver;
			}
		}

		cmd_type = (vdm_command_type_t)USB_PD_VDM_HDR_GET_CMD_TYPE(vdm_hdr);

		switch (cmd)
		{
			case VDM_CMD_DISCOVER_IDENTITY:
				vdm_handle_discover_identity(dev, cmd_type);
				break;

			case VDM_CMD_DISCOVER_SVIDS:
				vdm_handle_discover_svids(dev, cmd_type);
				break;

			case VDM_CMD_DISCOVER_MODES:
				vdm_handle_discover_modes(dev, cmd_type);
				break;

			case VDM_CMD_ENTER_MODE:
				vdm_handle_enter_mode(dev, cmd_type, USB_PD_VDM_HDR_GET_OBJ_POS(vdm_hdr));
				break;

			case VDM_CMD_EXIT_MODE:
				vdm_handle_exit_mode(dev, cmd_type, USB_PD_VDM_HDR_GET_OBJ_POS(vdm_hdr));
				break;

			case VDM_CMD_ATTENTION:
				vdm_handle_attention(dev, cmd_type);
				break;

			default:
				if (config->svids[dev->svid_index] == VDM_DISPLAYPORT_VID)
				{
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
					if (cmd == VDM_CMD_DP_STATUS_UPDATE)
					{
						vdm_handle_dp_status_update(dev, cmd_type);
					}
					else if (cmd == VDM_CMD_DP_CONFIG)
					{
						vdm_handle_dp_config(dev, cmd_type);
					}
#endif
				}
				else if (config->svids[dev->svid_index] == VDM_HDMI_VID)
				{
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
					if (cmd == VDM_CMD_HDMI_STATUS_UPDATE)
					{
						vdm_handle_hdmi_status_update(dev, cmd_type);
					}
					else if (cmd == VDM_CMD_HDMI_CEC_WRITE)
					{
						// Not supported.
						pe_set_state(dev, PE_RESP_VDM_NAK);
					}
					else if (cmd == VDM_CMD_HDMI_CEC_READ)
					{
						// Not supported.
						pe_set_state(dev, PE_RESP_VDM_NAK);
					}
					else if (cmd == VDM_CMD_HDMI_DDC_WRITE)
					{
						vdm_handle_hdmi_ddc_write(dev, cmd_type);
					}
					else if (cmd == VDM_CMD_HDMI_DDC_READ)
					{
						vdm_handle_hdmi_ddc_read(dev, cmd_type);
					}
#endif
				}
				break;
		}

		// Return to READY state if we haven't transitioned to new state.
		if (!dev->state_change)
		{
#ifdef CABLE_PLUG
			pe_set_state(dev, PE_CBL_READY);
#else
			if (dev->power_role == PD_PWR_ROLE_SNK)
			{
				pe_set_state(dev, PE_SNK_READY);
			}
			else
			{
				pe_set_state(dev, PE_SRC_READY);
			}
#endif
			// VDM is complete.
			dev->vdm_in_progress = false;
		}
	}
	else /* Invalid SVID or command */
	{
		// Attention command should never have a response.
		if (cmd != VDM_CMD_ATTENTION)
		{
			// Send NAK with original SVID, object position, and cmd if invalid SVID or invalid command.
			pe_set_state(dev, PE_RESP_VDM_NAK);
		}
	}

	return;
}

static void usb_pd_pe_unstructured_vdm_handler(usb_pd_port_t *dev)
{
	// Prior to establishing an Explicit Contract, Unstructured VDMs 
	// shall not be sent and shall be ignored if received.
	if (!dev->explicit_contract)
	{
		return;
	}

	// Only a DFP shall be an initiator.  
	// Only a UFP or cable plug shall be a responder.

	// Unstructured VDM are currently unsupported.
	// For PD r2.0, DFP or UFP should ignore the msg if not supported.
	// For PD r3.0, DFP or UFP should return Not_Supported msg if not supported.
#if defined ENABLE_USB_PD_REV3_SUPPORT && !defined CABLE_PLUG
	if (!dev->state_change && (dev->p2p_spec_rev > PD_REV20))
	{
		pe_set_state(dev, PE_SEND_NOT_SUPPORTED);

		// VDM is complete.
		dev->vdm_in_progress = false;
	}
#endif

	return;
}


static void usb_pd_pe_vdm_handler(usb_pd_port_t *dev)
{
	uint32_t vdm_hdr = get_data_object(dev->msg_buf);

	if (USB_PD_VDM_IS_STRUCT_TYPE(vdm_hdr))
	{
		usb_pd_pe_structured_vdm_handler(dev);
	}
	else /* Unstructured VDM */
	{
		usb_pd_pe_unstructured_vdm_handler(dev);
	}

	return;
}

static void timeout_vconn_stable(unsigned int port)
{
	pe_set_state(&pd[port], PE_SRC_VDM_IDENTITY_REQUEST);
	return;
}


static void timeout_vdm_mode_entry(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	// Restore USB operation. 

	// Notify DPM.

	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_READY);
	}
	else
	{
		pe_set_state(dev, PE_SRC_READY);
	}

	return;
}

#endif /* ENABLE_VDM_SUPPORT */

#if !defined ENABLE_VDM_SUPPORT || (!defined DP_DONGLE && !defined HDMI_DONGLE)
/* Dummy function to prevent compiler error when VDM or DP alt mode support disabled. */
void usb_pd_pe_hpd_in_event(unsigned int port, uint8_t val)
{
	return;
}
#endif

static void timeout_vbus_5v_stabilize(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if (*dev->current_state == PE_SRC_STARTUP)
	{
		pe_set_state(dev, PE_SRC_SEND_CAPS);
	}
	else if (*dev->current_state == PE_PRS_SOURCE_ON)
	{
		pe_set_state(dev, PE_PRS_SOURCE_ON_EXIT);
	}

	return;
}

static void timeout_swap_source_start(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

#ifdef ENABLE_VDM_SUPPORT
	if (!dev->cable_id_req_complete && tcpm_is_vconn_enabled(dev->port))
	{
		pe_set_state(dev, PE_SRC_VDM_IDENTITY_REQUEST);
	}
	else
#endif
	{
		pe_set_state(dev, PE_SRC_SEND_CAPS);
	}

	return;
}

static void timeout_typec_send_source_cap(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	INFO("CapsCnt = %u\n", dev->caps_cnt);

	if ((dev->caps_cnt > N_CAPS_COUNT) &&
		!dev->pd_connected)
	{
		pe_set_state(dev, PE_SRC_DISABLED);
	//	pb_pd_cc_cb(true, true);
	}
	else
	{
		pe_set_state(dev, PE_SRC_SEND_CAPS);
	}

	return;
}


static void timeout_ps_hard_reset(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if ((*dev->current_state == PE_SRC_HARD_RESET) ||
		(*dev->current_state == PE_SRC_HARD_RESET_RECEIVED))
	{
		pe_set_state(dev, PE_SRC_TRANSITION_TO_DEFAULT);
	}

	return;
}

static void pe_src_startup_entry(usb_pd_port_t *dev)
{
	DUAL_ROLE_CHANGED();

	usb_pd_prl_reset(dev->port);
	dev->non_interruptable_ams = false;
	dev->power_role_swap_in_progress = false;

	// Reset source caps count.
	dev->caps_cnt = 0;

	if (tcpm_is_vconn_enabled(dev->port))
	{
		// Enable PD receive for SOP/SOP'.
		tcpm_enable_pd_receive(dev->port, true, false);
	}
	else
	{
		// Enable PD receive for SOP.
		tcpm_enable_pd_receive(dev->port, false, false);
	}

	if (dev->swap_source_start)
	{
		// Wait tSwapSourceStart before transitioning to SRC_SEND_CAPS.
		timer_start(&dev->timer, T_SWAP_SOURCE_START_MS, timeout_swap_source_start);
		dev->swap_source_start = false;
	}
	else
	{
#ifdef ENABLE_VDM_SUPPORT
		if (!dev->cable_id_req_complete && tcpm_is_vconn_enabled(dev->port))
		{
			dev->send_src_cap_start_once = true;
			timer_start(&dev->timer, T_VCONN_STABLE_MS, timeout_vconn_stable);
		}
		else
#endif
		{
			// Wait for VBUS to stablize before transitioning to SRC_SEND_CAPS.
			timer_start(&dev->timer, T_VBUS_5V_STABLIZE_MS, timeout_vbus_5v_stabilize);
		}
	}

	return;
}

#ifdef ENABLE_VDM_SUPPORT

static void timeout_vdm_sender_response(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if (*dev->current_state == PE_SRC_VDM_IDENTITY_REQUEST)
	{
		pe_set_state(dev, PE_SRC_DISCOVERY);
	}
	else
	{
		if (*dev->current_state == PE_INIT_PORT_VDM_IDENTITY_REQUEST)
		{
			dev->port_partner_id_req_complete = true;
		}
		else
		{
			dev->vdm_pending = true;
			dev->vdm_pending_state = *dev->current_state;
		}

		if (dev->power_role == PD_PWR_ROLE_SNK)
		{
			pe_set_state(dev, PE_SNK_READY);
		}
		else
		{
			pe_set_state(dev, PE_SRC_READY);
		}
	}

	return;
}


static void pe_src_vdm_identity_request_entry(usb_pd_port_t *dev)
{
	dev->discover_identity_cnt++;

	// Send Discover Identity VDM to local cable plug.
	vdm_send_structured_cmd(dev->port, VDM_PD_SID, VDM_CMD_DISCOVER_IDENTITY, 0, TCPC_TX_SOP_P);

	return;
}

static void pe_init_port_vdm_identity_request_entry(usb_pd_port_t *dev)
{
	// Send Discover Identity VDM to port partner.
	vdm_send_structured_cmd(dev->port, VDM_PD_SID, VDM_CMD_DISCOVER_IDENTITY, 0, TCPC_TX_SOP);

	return;
}

#endif


#define VDM_SEND_CAP_MS  (T_TYPEC_SEND_SOURCE_CAP_MS - T_VCONN_STABLE_MS)

static void pe_src_discovery_entry(usb_pd_port_t *dev)
{
#ifdef ENABLE_VDM_SUPPORT
	if (tcpm_is_vconn_enabled(dev->port))
	{
		// Check if cable identification is not complete and the count is not exceeded.
		if (!dev->cable_id_req_complete &&
			(dev->discover_identity_cnt < N_DISCOVER_IDENTITY_COUNT))
		{
			// Start timer to give up trying VDM identity request and begin sending 
			// Source Capabilities message within spec limit.
			if (dev->send_src_cap_start_once)
			{
				dev->send_src_cap_start_once = false;
				dev->send_src_cap_timeout = false;
				tusb422_lfo_timer_start(&dev->timer2, VDM_SEND_CAP_MS, timeout_send_src_cap);
			}

			if (!dev->send_src_cap_timeout)
			{
				pe_set_state(dev, PE_SRC_VDM_IDENTITY_REQUEST);
			}
		}

		if (!dev->state_change)
		{
			if (dev->caps_cnt == 0)
			{
				// Send SRC CAPS immediately.
				pe_set_state(dev, PE_SRC_SEND_CAPS);
			}
			else
			{
				// Start SourceCapabilityTimer.
				timer_start(&dev->timer, T_TYPEC_SEND_SOURCE_CAP_MS, timeout_typec_send_source_cap);
			}
		}
	}
	else
#endif
	{
		// Start SourceCapabilityTimer.
		timer_start(&dev->timer, T_TYPEC_SEND_SOURCE_CAP_MS, timeout_typec_send_source_cap);
	}

	return;
}

static void usb_pd_pe_handle_snk_src_default_state(usb_pd_port_t *dev)
{
	// Disable VCONN.
	tcpm_set_vconn_enable(dev->port, false);

	dev->pd_connected = false;
	dev->explicit_contract = false;
	dev->active_alt_modes = 0;

#ifdef ENABLE_VDM_SUPPORT
	dev->port_partner_id_req_complete = false;
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	tcpm_hpd_out_control(dev->port, HPD_LOW);
#endif
#endif

	return;
}

static void pe_src_transition_to_default_entry(usb_pd_port_t *dev)
{
	if (dev->active_alt_modes)
	{
		// Isolate any pins (including SBUs) previously configured for alt mode.
		tcpm_mux_control(dev->port, dev->data_role, MUX_DISABLE, dev->tcpc_dev->plug_polarity);
	}

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	if (dev->timer2.function == timeout_pps_timeout)
	{
		// Stop the SourcePPSCommTimer if there is a Request msg, PR swap, or Hard Reset.
		tusb422_lfo_timer_cancel(&dev->timer2);
	}
#endif

	// Disable VCONN and set default flags.
	usb_pd_pe_handle_snk_src_default_state(dev);

	// Remove Rp from VCONN pin. (Also restores original Rp value)
	tcpm_remove_rp_from_vconn_pin(dev->port);

	// Disable VBUS.
	tcpm_src_vbus_disable(dev->port);

	// Force VBUS discharge.
	tcpm_force_discharge(dev->port, VSTOP_DISCHRG);

	// Wait until VBUS drops to vSafe0V before checking data role and starting source recover timer.
	tcpm_set_voltage_alarm_lo(dev->port, VSAFE0V_MAX);

	return;
}

static void pe_src_transition_to_default_exit(usb_pd_port_t *dev)
{

	// Configure pins for USB mode.
	tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);

	// Enable VCONN.
	tcpm_set_vconn_enable(dev->port, true);
	dev->vconn_source = true;

	// Restore vSafe5V.
	tcpm_source_vbus(dev->port, false, 5000);

	// After a Hard Reset, the sink must respond to SRC_CAPS within tNoResponse.
	dev->start_src_no_response_timer = true;

	// Wait for VBUS to reach vSafe5V before going to PE_SRC_STARTUP state.
	tcpm_set_voltage_alarm_hi(dev->port, VSAFE5V_MIN);

	return;
}

static void timeout_src_recover(unsigned int port)
{
	pe_src_transition_to_default_exit(&pd[port]);
	return;
}

static void pe_src_send_caps_entry(usb_pd_port_t *dev)
{
	if (dev->start_src_no_response_timer)
	{
		// Starting no response timer here instead of PE_SRC_TRANSITION_TO_DEFAULT 
		// so we can use the LFO timer during PE_SRC_VDM_IDENTITY_REQUEST.
		dev->start_src_no_response_timer = false;
		timer_start_no_response(dev);
	}

	// Increment source caps count.
	dev->caps_cnt++;

	// Send Source Capabilities.
	usb_pd_pe_tx_data_msg(dev->port, DATA_MSG_TYPE_SRC_CAPS, TCPC_TX_SOP);
	return;
}


static void pe_src_negotiate_capability_entry(usb_pd_port_t *dev)
{
	enum supply_type_t supply_type;
	uint16_t operating_current_or_power;
	uint16_t src_max_current_or_power;
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	uint16_t output_voltage;
	uint16_t src_min_voltage;
	uint16_t src_max_voltage;
#endif
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	// Save the RDO.
	dev->rdo = get_data_object(dev->msg_buf);

	// Save the spec revision from the sink's Request message for all subsequent communication.
	dev->p2p_spec_rev = dev->rx_msg_spec_rev;
	// If Port to port operating spec revision is lower than the port the cable revision,
	// downgrade the port to cable operating spec revision.
	if (dev->p2p_spec_rev < dev->p2c_spec_rev)
	{
		dev->p2c_spec_rev = dev->p2p_spec_rev;
	}

	dev->object_position = RDO_OBJECT_POSITION(dev->rdo);

	if ((dev->object_position > 0) &&
		(dev->object_position <= config->num_src_pdos) &&
		(dev->object_position <= PD_MAX_PDO_NUM))
	{
		supply_type = (enum supply_type_t)PDO_SUPPLY_TYPE(dev->src_pdo[dev->object_position - 1]);

		if (supply_type <= SUPPLY_TYPE_VARIABLE)
		{
			operating_current_or_power = RDO_OPERATIONAL_CURRENT_OR_POWER(dev->rdo); /* 10mA or 250mW units */
			src_max_current_or_power = PDO_MAX_CURRENT_OR_POWER(dev->src_pdo[dev->object_position - 1]); /* 10mA or 250mW units */

			if (supply_type == SUPPLY_TYPE_BATTERY)
			{
				DEBUG("PE_SRC_NEG_CAP: ObjPos = %u, req = %u mW, avail = %u mW\n",
					  dev->object_position, (uint32_t)operating_current_or_power * 250, (uint32_t)src_max_current_or_power * 250);
			}
			else /* Fixed or Variable */
			{
				DEBUG("PE_SRC_NEG_CAP: ObjPos = %u, req = %u mA, avail = %u mA\n",
					  dev->object_position, operating_current_or_power * 10, src_max_current_or_power * 10);
			}
	
			if (operating_current_or_power <= src_max_current_or_power)
			{
				pe_set_state(dev, PE_SRC_TRANSITION_SUPPLY);
			}
		}
#ifdef ENABLE_USB_PD_REV3_SUPPORT
		else /* SUPPLY_TYPE_PROGRAMMABLE */
		{
			output_voltage = PROG_RDO_OUTPUT_VOLTAGE(dev->rdo) * 20; /* x 20mV conversion */
			operating_current_or_power = PROG_RDO_OPERATIONAL_CURRENT(dev->rdo); /* 50mA */
			src_max_current_or_power = APDO_MAX_CURRENT(dev->src_pdo[dev->object_position - 1]); /* 50mA */
			src_min_voltage = APDO_MIN_VOLTAGE(dev->src_pdo[dev->object_position - 1]) * 100; /* x 100mV conversion */
			src_max_voltage = APDO_MAX_VOLTAGE(dev->src_pdo[dev->object_position - 1]) * 100; /* x 100mV conversion */

			DEBUG("PE_SRC_NEG_CAP: ObjPos = %u, req = %u mV %u mA, avail = %u-%u mV %u mA\n",
				  dev->object_position, output_voltage, operating_current_or_power * 50, 
				  src_min_voltage, src_max_voltage, src_max_current_or_power * 50);

			if ((operating_current_or_power <= src_max_current_or_power) &&
				(output_voltage >= src_min_voltage) &&
				(output_voltage <= src_max_voltage))
			{
				pe_set_state(dev, PE_SRC_TRANSITION_SUPPLY);
			}
		}
#endif
	}
	else
	{
		DEBUG("PE_SRC_NEG_CAP: ObjPos = %u is invalid!\n", dev->object_position);
	}

	if (!dev->state_change)
	{
		// Request cannot be met.
		pe_set_state(dev, PE_SRC_CAPABILITY_RESPONSE);
	}

	return;
}


static void pe_src_transition_supply_entry(usb_pd_port_t *dev)
{
	if (dev->request_goto_min)
	{
		dev->request_goto_min = false;

		// Send GotoMin message.
		usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_GOTO_MIN, TCPC_TX_SOP);
	}
	else
	{
		// Send Accept message.
		usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_ACCEPT, TCPC_TX_SOP);
	}

	return;
}

static void pe_src_transition_supply_exit(usb_pd_port_t *dev)
{
	// Send PS_RDY.
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_PS_RDY, TCPC_TX_SOP);
	return;
}


static void pd_transition_power_supply(usb_pd_port_t *dev)
{
	enum supply_type_t supply_type;
	uint16_t v_src_new_min;
	uint16_t v_src_new_max;
	uint16_t present_voltage;
	uint16_t requested_voltage;
	uint16_t req_volt_pps;

	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	supply_type = (enum supply_type_t)PDO_SUPPLY_TYPE(dev->src_pdo[dev->object_position - 1]);

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	if (supply_type == SUPPLY_TYPE_PROGRAMMABLE)
	{
		// Get requested VBUS voltage (20mV units) and convert to 25 millivolt units.
	    req_volt_pps = PROG_RDO_OUTPUT_VOLTAGE(dev->rdo);
		requested_voltage = (PROG_RDO_OUTPUT_VOLTAGE(dev->rdo) * 4) / 5;
	}
	else
#endif
	{
		// Get requested VBUS voltage and convert to 25 millivolt units.
		requested_voltage = PDO_MIN_VOLTAGE(dev->src_pdo[dev->object_position - 1]) << 1;
	}

	// Get present VBUS voltage (25 millivolt units).
	present_voltage = tcpm_get_vbus_voltage(dev->port);

	CRIT("VBUS is %u mV, %u mV requested\n", present_voltage * 25, requested_voltage * 25);

	// If fixed or programmable PDO, get min/max vSrcNew which is 0.95 and 1.05 times the target.
	if ((supply_type == SUPPLY_TYPE_FIXED) || 
		(supply_type == SUPPLY_TYPE_PROGRAMMABLE))
	{
		v_src_new_max = requested_voltage + ((requested_voltage * 5) / 100);
		v_src_new_min = requested_voltage - ((requested_voltage * 5) / 100);
	}
	else /* Variable or Battery */
	{
		v_src_new_max = PDO_MAX_VOLTAGE(dev->src_pdo[dev->object_position - 1]) << 1;
		v_src_new_min = requested_voltage;
	}

	INFO("v_src_new_max = %u mV\n", (v_src_new_max * 25));

	if (supply_type == SUPPLY_TYPE_PROGRAMMABLE)
	    tcpm_source_vbus(dev->port, true, req_volt_pps * 20);
	else
	    tcpm_source_vbus(dev->port, true, (requested_voltage * 25));

	// Check if negative voltage transition.
	if (v_src_new_max < present_voltage)
	{
		tcpm_set_voltage_alarm_lo(dev->port, v_src_new_max);

//		timer_start(&dev->timer, T_SRC_SETTLE_MS, timeout_src_settle);

#ifdef ENABLE_USB_PD_REV3_SUPPORT
		if (supply_type != SUPPLY_TYPE_PROGRAMMABLE)
#endif
		{
			// Force VBUS discharge to new voltage level.
			tcpm_force_discharge(dev->port, v_src_new_max);
		}
	}
	else
	{
		if (dev->object_position == 1)
		{
			// Default 5V. No voltage transition required.
			pe_set_state(dev, PE_SRC_TRANSITION_SUPPLY_EXIT);
		}
		else
		{
			// Use Hi voltage alarm to determine when power supply is ready. 
			tcpm_set_voltage_alarm_hi(dev->port, v_src_new_min);
		}
	}

	return;
}


static void timeout_src_transition(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if (*dev->current_state == PE_SRC_TRANSITION_SUPPLY)
	{
		pd_transition_power_supply(&pd[port]);
	}
	else if (*dev->current_state == PE_PRS_TRANSITION_TO_OFF)
	{
		// Disable source VBUS.
		tcpm_src_vbus_disable(dev->port);
	}
	return;
}

static void pe_src_capability_response_entry(usb_pd_port_t *dev)
{
	// Send Reject.
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_REJECT, TCPC_TX_SOP);

	// BQ - Send Wait not supported.
}

static void usb_pd_pe_snk_src_ready_initiate_ams(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	if (dev->data_role == PD_DATA_ROLE_DFP)
	{
#ifdef ENABLE_VDM_SUPPORT
		if (tcpm_is_vconn_enabled(dev->port) && dev->cable_plug_comm_required)
		{
			if (dev->send_cable_soft_reset)
			{
				dev->send_cable_soft_reset = false;

				//pe_set_state(dev, PE_DFP_CBL_SEND_SOFT_RESET);
			}
			else
			{
				// If this is a DFP which needs to establish communication with a Cable Plug,
				// then the Policy Engine shall initialize and run the DiscoverIdentityTimer. - BQ
				// timer_start(&dev->timer, T_DISCOVER_IDENTITY_MS, timeout_func);
			}
		}
//		else if ((dev->power_role == PD_PWR_ROLE_SRC) && !dev->get_sink_caps_complete)
//		{
//			pe_set_state(dev, PE_SRC_GET_SINK_CAP);
//		}
		else if (USB_PD_VDM_MODAL_OPERATION(config->id_header_vdo) &&
				 !dev->port_partner_id_req_complete)
		{
			// Give UFP arbitrary amount of time to settle before starting VDM discovery.
			dev->vdm_pending_state = PE_INIT_PORT_VDM_IDENTITY_REQUEST;
			timer_start(&dev->timer, 25, timeout_vdm_busy);
		}
		else if (dev->vdm_pending)
		{
			dev->vdm_pending = false;
			pe_set_state(dev, dev->vdm_pending_state);
		}
		else if (dev->mode_exit_pending)
		{
			dev->mode_exit_pending = false;
			pe_set_state(dev, PE_DFP_VDM_MODE_EXIT_REQUEST);
		}
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
		else if ((dev->active_alt_modes & DP_ALT_MODE) && 
				 (dev->hpd_in_queue[dev->hpd_in_send_idx] != NOTIFY_HPD_NONE))
		{
			usb_pd_pe_update_displayport_status(dev);

			// Send Status Update VDM to port partner.
			pe_set_state(dev, PE_INIT_VDM_DP_STATUS_UPDATE);
		}
#endif
#endif
	}
	else /* UFP */
	{
#ifdef ENABLE_VDM_SUPPORT
		if (dev->vdm_pending)
		{
			dev->vdm_pending = false;
			pe_set_state(dev, dev->vdm_pending_state);
		}
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
		else if ((dev->active_alt_modes & DP_ALT_MODE) && 
				 (dev->hpd_in_queue[dev->hpd_in_send_idx] != NOTIFY_HPD_NONE))
		{
			pe_set_state(dev, PE_INIT_VDM_ATTENTION_REQUEST);
		}
#endif
#endif
	}

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	usb_pd_pe_check_sink_tx_ok(dev);
#endif

	return;
}

static void usb_pd_pe_handle_snk_src_ready_state(usb_pd_port_t *dev)
{
#ifdef CONFIG_TUSB422_PAL
	usb_pd_pal_notify_pd_state(dev->port, *dev->current_state);
#endif

	if (tcpm_is_vconn_enabled(dev->port))
	{
		// Enable PD receive for SOP/SOP'/SOP".
		tcpm_enable_pd_receive(dev->port, true, true);
	}

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	if ((dev->p2p_spec_rev > PD_REV20) && 
		(*dev->current_state == PE_SNK_READY) && 
		!dev->sink_tx_ok)
	{
		// Sink cannot initiate an AMS at this time.
		// Return and wait for source current advertisement change to 3.0A
		// before attempting to initiate any AMS.
		return;
	}
#endif

	usb_pd_pe_snk_src_ready_initiate_ams(dev);

	return;
}

#ifdef ENABLE_USB_PD_REV3_SUPPORT
static void timeout_pps_timeout(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	pe_set_state(dev, PE_SRC_HARD_RESET);
	return;
}
#endif


static void pe_src_ready_entry(usb_pd_port_t *dev)
{
	// Notify PRL of end of AMS.
	dev->non_interruptable_ams = false;

	dev->explicit_contract = true;

	//Falong
//    pb.otg_active = true;
	pb_pd_explicit_contract_cb(true, true);

	// While an USB PD Explicit Contract is in place, the Source shall 
	// advertise a USB Type-C Current of either 1.5A or 3.0A per Type-C spec.
	// Set Rp value to 3.0A (SinkTxOk) for USB-PD rev3 collision avoidance.
	tcpm_set_rp_value(dev->port, RP_HIGH_CURRENT);

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	dev->sink_tx_ok = true;

	if (PDO_SUPPLY_TYPE(dev->src_pdo[dev->object_position - 1]) == SUPPLY_TYPE_PROGRAMMABLE)
	{
		// If a programmable power supply was selected, set a timeout to receive 
		// the periodic request.
		tusb422_lfo_timer_start(&dev->timer2, T_PPS_TIMEOUT_MS, timeout_pps_timeout);
	}
#endif

	usb_pd_pe_handle_snk_src_ready_state(dev);

	return;
}

static void pe_src_hard_reset_received_entry(usb_pd_port_t *dev)
{
	timer_start(&dev->timer, T_PS_HARD_RESET_MS, timeout_ps_hard_reset);
	return;
}

static void pe_src_hard_reset_entry(usb_pd_port_t *dev)
{
	dev->hard_reset_cnt++;
	tcpm_transmit(dev->port, NULL, TCPC_TX_HARD_RESET, 0);
	timer_start(&dev->timer, T_PS_HARD_RESET_MS, timeout_ps_hard_reset);
	return;
}

static void pe_src_get_sink_cap_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_GET_SNK_CAP, TCPC_TX_SOP);
	return;
}

static void pe_src_disabled_entry(usb_pd_port_t *dev)
{
	tcpm_disable_pd_receive(dev->port);
	return;
}

/* can only be entered from PE_SRC_READY state */
static void pe_src_ping_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_PING, TCPC_TX_SOP);
	return;
}

static void pe_src_wait_new_caps_entry(usb_pd_port_t *dev)
{
	// Do nothing. Wait for policy manager to provide new caps.
}

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT

#define PRESENT_INPUT_EXTERNAL_PWR_BIT      (1 << 1)
#define PRESENT_INPUT_EXTERNAL_PWR_AC_BIT   (1 << 2)
#define PRESENT_INPUT_INTERNAL_PWR_BATT_BIT (1 << 3)
#define PRESENT_INPUT_INTERNAL_PWR_NON_BATT_BIT (1 << 4)

static void pe_send_status_entry(usb_pd_port_t *dev)
{
	uint8_t *data_block = dev->extended_msg_buf;

	data_block[0] = 0; /* Internal Temp */
	data_block[1] = 0; /* Present Input */
	data_block[2] = 0; /* Present Batt Input */
	if (dev->externally_powered)
	{
		data_block[1] |= PRESENT_INPUT_EXTERNAL_PWR_BIT;
	}
	else
	{
		data_block[1] |= PRESENT_INPUT_INTERNAL_PWR_BATT_BIT;
		data_block[2] = 0x1; /* Assume single fixed battery */
	}
	data_block[3] = 0; /* Event Flags */
	data_block[4] = 0; /* Temp Status */

	usb_pd_prl_tx_data_msg_ext(dev->port, EXT_MSG_TYPE_STATUS, TCPC_TX_SOP, 5);
	return;
}

#define USB_VID 0x0451
#define USB_PID 0x0422

static void pe_give_manufacturer_info_entry(usb_pd_port_t *dev)
{
	uint8_t *data_block = dev->extended_msg_buf;

	data_block[0] = USB_VID & 0xFF; /* VID */
	data_block[1] = USB_VID >> 8;   /* VID */
	data_block[2] = USB_PID & 0xFF; /* PID */
	data_block[3] = USB_PID >> 8;   /* PID */

	// Optional manufacturer string here, up to 22 chars.

	usb_pd_prl_tx_data_msg_ext(dev->port, EXT_MSG_TYPE_MANUF_INFO, TCPC_TX_SOP, 4);
	return;
}


#endif

static void pe_drs_send_swap_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_DR_SWAP, TCPC_TX_SOP);
	return;
}

static void pe_drs_evaluate_swap_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	dev->non_interruptable_ams = true;

	if (config->dual_role_data)
	{
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			if (config->auto_accept_swap_to_dfp)
			{
				pe_set_state(dev, PE_DRS_ACCEPT_SWAP);
			}
			else
			{
				// Ask platform policy manager or Reject.
				pe_set_state(dev, PE_SEND_REJECT);
			}
		}
		else /* DFP */
		{
			if (config->auto_accept_swap_to_ufp)
			{
				pe_set_state(dev, PE_DRS_ACCEPT_SWAP);
			}
			else
			{
				// Ask platform policy manager or Reject.
				pe_set_state(dev, PE_SEND_REJECT);
			}
		}
	}
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	else if (dev->p2p_spec_rev > PD_REV20)
	{
		pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
	}
#endif
	else
	{
		pe_set_state(dev, PE_SEND_REJECT);
	}

	if (*dev->current_state == PE_DRS_ACCEPT_SWAP)
	{
		// Update the message info here in case we receive a msg immediately after
		// transmitting the ACCEPT.
		if (dev->data_role == PD_DATA_ROLE_UFP)
		{
			tcpm_update_msg_header_info(dev->port, PD_DATA_ROLE_DFP, dev->power_role);

#ifdef DP_DONGLE
		if (dev->timer2.function == timeout_alt_mode_entry)
		{
			// Cancel tAMETimeout.
			tusb422_lfo_timer_cancel(&dev->timer2);
		}
#endif
		}
		else /* DFP */
		{
			tcpm_update_msg_header_info(dev->port, PD_DATA_ROLE_UFP, dev->power_role);
		}
	}

	return;
}

static void pe_drs_change_role_entry(usb_pd_port_t *dev)
{
	// Update the data role.
	if (dev->data_role == PD_DATA_ROLE_UFP)
	{
		dev->data_role = PD_DATA_ROLE_DFP;
	}
	else /* DFP */
	{
		dev->data_role = PD_DATA_ROLE_UFP;
	}

	// Notify system of new data role.
#ifdef CONFIG_TUSB422_PAL
	usb_pd_pal_data_role_swap(dev->port, dev->data_role);
#endif

	DUAL_ROLE_CHANGED();

	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_READY);
	}
	else /* SRC */
	{
		pe_set_state(dev, PE_SRC_READY);
	}

#ifdef ENABLE_VDM_SUPPORT
	// Data Role Swap shall reset the DiscoverIdentityCounter to zero.
	dev->discover_identity_cnt = 0;
#endif

	return;
}


static void pe_prs_send_swap_entry(usb_pd_port_t *dev)
{
	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		tcpm_snk_swap_standby(dev->port);
	}
	else if (dev->active_alt_modes &&
			 tcpm_is_vconn_enabled(dev->port) &&
			 (dev->tcpc_dev->silicon_revision <= 2))
	{
		/*** TUSB422 PG1.x workaround for VCONN disabled during PR swap (CDDS #77) ***/
		// If a VCONN-powered Alt-mode adapter is active, schedule a
		// Type-C error recovery after the PR swap is completed to ensure
		// proper alt-mode operation after the momentary loss of VCONN.
		dev->error_recovery_pending = true;
	}

	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_PR_SWAP, TCPC_TX_SOP);
	return;
}

static void pe_prs_evaluate_swap_entry(usb_pd_port_t *dev)
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	dev->non_interruptable_ams = true;

	if (dev->tcpc_dev->role == ROLE_DRP)
	{
		if (dev->power_role == PD_PWR_ROLE_SNK)
		{
			if (dev->tcpc_dev->silicon_revision == 0)
			{
				/*** TUSB422 PG1.0 cannot support SNK->SRC swap because PD will be disabled (CDDS #60) ***/
				pe_set_state(dev, PE_SEND_REJECT);
			}
			else
			{
				if (config->auto_accept_swap_to_source)
				{
					pe_set_state(dev, PE_PRS_ACCEPT_SWAP);
				}
				else
				{
					// Ask platform policy manager or Reject.
					pe_set_state(dev, PE_SEND_REJECT);
				}
			}

			if (*dev->current_state == PE_PRS_ACCEPT_SWAP)
			{
				tcpm_snk_swap_standby(dev->port);
			}
		}
		else /* SRC */
		{
			if (config->auto_accept_swap_to_sink)
			{
//				if (dev->externally_powered)
//				{
//					pe_set_state(dev, PE_SEND_REJECT);
//					// Alternatively, Response can be Wait followed by GetSourceCap message.
//					//pe_set_state(dev, PE_PRS_WAIT_SWAP);
//				}
//				else
				{
#ifdef ENABLE_USB_PD_REV3_SUPPORT
					if (dev->timer2.function == timeout_pps_timeout)
					{
						// Stop the SourcePPSCommTimer if there is a Request msg, PR swap, or Hard Reset.
						tusb422_lfo_timer_cancel(&dev->timer2);
					}
#endif
					pe_set_state(dev, PE_PRS_ACCEPT_SWAP);

					if (dev->active_alt_modes &&
						tcpm_is_vconn_enabled(dev->port) &&
						(dev->tcpc_dev->silicon_revision <= 2))
					{
						/*** TUSB422 PG1.x workaround for VCONN disabled during PR swap (CDDS #77) ***/
						// If a VCONN-powered Alt-mode adapter is active, schedule a
						// Type-C error recovery after the PR swap is completed to ensure
						// proper alt-mode operation after the momentary loss of VCONN.
						dev->error_recovery_pending = true;
					}
				}
			}
			else
			{
				// Ask platform policy manager or Reject.
				pe_set_state(dev, PE_SEND_REJECT);
			}
		}
	}
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	else if (dev->p2p_spec_rev > PD_REV20)
	{
		pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
	}
#endif
	else
	{
		pe_set_state(dev, PE_SEND_REJECT);
	}

	return;
}

static void timeout_ps_source(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	dev->power_role_swap_in_progress = false;
	pe_set_state(dev, PE_ERROR_RECOVERY);
	return;
}

static void pe_prs_transition_to_off_entry(usb_pd_port_t *dev)
{
	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		// Start PSSourceOff timer.
		timer_start(&dev->timer, T_PS_SOURCE_OFF_MS, timeout_ps_source); 

		// Disable sink VBUS.
		tcpm_snk_vbus_disable(dev->port);
	}
	else /* SRC */
	{
		// Accept msg was ACKed, transition from Attached.SRC to Attached.SNK according to Type-C spec.
		// This will prevent transistion to Unattached state when Rd-Rd condition is detected. 
		tcpm_handle_power_role_swap(dev->port);

		// Wait tSrcTransition before disabling VBUS.
		timer_start(&dev->timer, T_SRC_TRANSITION_MS, timeout_src_transition);

		// Wait until VBUS drops to vSafe0V before changing CC to Rd.
		tcpm_set_voltage_alarm_lo(dev->port, VSAFE0V_MAX);
	}

	dev->explicit_contract = false;
//	pb.otg_active = false;

	return;
}

static void pe_prs_assert_rd_entry(usb_pd_port_t *dev)
{
	// Disable AutoDischargeDisconnect to prevent VCONN from being disabled
	// when CC open condition is detected.
	tcpm_set_autodischarge_disconnect(dev->port, false);

	// Rp -> Rd.
	tcpm_cc_pin_control(dev->port, ROLE_SNK);

	pe_set_state(dev, PE_PRS_WAIT_SOURCE_ON);

	return;
}

static void pe_prs_wait_source_on_entry(usb_pd_port_t *dev)
{
	/*** TUSB422 PG1.x workaround for VCONN disabled during PR swap (CDDS #77) ***/
	if ((dev->tcpc_dev->silicon_revision <= 2) && dev->vconn_source)
	{
		tcpm_set_vconn_enable(dev->port, true);
	}

	// Set new power role to sink.
	dev->power_role = PD_PWR_ROLE_SNK;

	DUAL_ROLE_CHANGED();

	tcpm_update_msg_header_info(dev->port, dev->data_role, dev->power_role);

	// Disable PD receive for SOP' and SOP".
	tcpm_enable_pd_receive(dev->port, false, false);

	// Send PS_RDY.
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_PS_RDY, TCPC_TX_SOP);

	// Start PSSourceOn timer.
	timer_start(&dev->timer, T_PS_SOURCE_ON_MS, timeout_ps_source); 

	return;
}

static void pe_prs_assert_rp_entry(usb_pd_port_t *dev)
{
	/*** TUSB422 PG1.x workaround for VCONN disabled during PR swap (CDDS #77) ***/
	if ((dev->tcpc_dev->silicon_revision <= 2) && dev->vconn_source)
	{
		tcpm_set_vconn_enable(dev->port, true);
	}

	// PS_RDY was received from original source, transition from Attached.SNK 
	// to Attached.SRC according to Type-C spec.
	tcpm_handle_power_role_swap(dev->port);

	// Rd -> Rp.
	tcpm_cc_pin_control(dev->port, ROLE_SRC);

	pe_set_state(dev, PE_PRS_SOURCE_ON);

	return;
}

static void pe_prs_source_on_entry(usb_pd_port_t *dev)
{
	// Enable source VBUS vSafe5V.
	tcpm_source_vbus(dev->port, false, 5000);

	// Wait until VBUS rises to vSafe5V before sending PS_RDY.
	tcpm_set_voltage_alarm_hi(dev->port, VSAFE5V_MIN);

	return;
}

static void pe_prs_source_on_exit(usb_pd_port_t *dev)
{
	// Set new power role to source.
	dev->power_role = PD_PWR_ROLE_SRC;

	DUAL_ROLE_CHANGED();

	tcpm_update_msg_header_info(dev->port, dev->data_role, dev->power_role);

	// Re-enable AutoDischargeDisconnect.
	tcpm_set_autodischarge_disconnect(dev->port, true);

	// Disable bleed discharge to save power.
	tcpm_set_bleed_discharge(dev->port, false);

	// Send PS_RDY.
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_PS_RDY, TCPC_TX_SOP);

	dev->swap_source_start = true;

	return;
}

static void pe_vcs_send_swap_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_VCONN_SWAP, TCPC_TX_SOP);
	return;
}

static void pe_vcs_evaluate_swap_entry(usb_pd_port_t *dev) 
{
	usb_pd_port_config_t *config = usb_pd_pm_get_config(dev->port);

	dev->non_interruptable_ams = true;

	if (dev->p2p_spec_rev > PD_REV20 && !config->vconn_source_capable)
	{
#ifdef ENABLE_USB_PD_REV3_SUPPORT
		pe_set_state(dev, PE_SEND_NOT_SUPPORTED);
#endif
	}
	else if (dev->vconn_source || config->auto_accept_vconn_swap)
	{
		// Current VCONN source must accept swap request.
		pe_set_state(dev, PE_VCS_ACCEPT_SWAP);
	}
	else
	{
		pe_set_state(dev, PE_SEND_REJECT);
	}

	return;
}

static void timeout_vconn_source_on(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	// Hard reset based on CC state.
	if (dev->tcpc_dev->cc_status & CC_STATUS_CONNECT_RESULT)
	{
		// Rd asserted (Consumer/Provider)
		pe_set_state(dev, PE_SNK_HARD_RESET);
	}
	else
	{
		// Rp asserted (Provider/Consumer)
		pe_set_state(dev, PE_SRC_HARD_RESET);
	}

	return;
}


static void pe_vcs_wait_for_vconn_entry(usb_pd_port_t *dev)
{
	// Start VCONNOn timer.
	timer_start(&dev->timer, T_VCONN_SOURCE_ON_MS, timeout_vconn_source_on); 
	return;
}

static void timeout_snk_error_recovery_pending(unsigned int port)
{
	tcpm_change_role(port, ROLE_SNK, true);
	return;
}

static void pe_vcs_turn_off_vconn_entry(usb_pd_port_t *dev)
{
	// Disable VCONN.
	tcpm_set_vconn_enable(dev->port, false);
	dev->vconn_source = false;

	// Cancel any error recovery due to VCONN loss during Power Role swap.
	dev->error_recovery_pending = false;
	if (dev->timer2.function == timeout_snk_error_recovery_pending)
	{
		tusb422_lfo_timer_cancel(&dev->timer2);
	}

	DUAL_ROLE_CHANGED();

	// Disable PD receive for SOP'/SOP".
	tcpm_enable_pd_receive(dev->port, false, false);

	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_READY);
	}
	else
	{
		pe_set_state(dev, PE_SRC_READY);
	}

	return;
}

static void timeout_vconn_enable(unsigned int port)
{
	pe_set_state(&pd[port], PE_VCS_SEND_PS_RDY);
	return;
}

static void pe_vcs_turn_on_vconn_entry(usb_pd_port_t *dev) 
{
	// Enable VCONN.
	tcpm_set_vconn_enable(dev->port, true);
	dev->vconn_source = true;

	DUAL_ROLE_CHANGED();

	// Short delay for VCONN stabilization before sending PS_RDY.
	timer_start(&dev->timer, 10, timeout_vconn_enable);

	return;
}

static void pe_vcs_send_ps_rdy_entry(usb_pd_port_t *dev)
{
	// Send PS_RDY.
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_PS_RDY, TCPC_TX_SOP);

#ifdef ENABLE_VDM_SUPPORT
	// After VCONN swap, VCONN source must issue SOP' or SOP" Soft Reset 
	// to cable plug before communication.
	dev->send_cable_soft_reset = true;
#endif
	return;
}


static void timeout_sink_wait_cap(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	pb_pd_cc_cb(true, false);

	DEBUG("%s: HR cnt = %u.\n", __func__, dev->hard_reset_cnt);

	if (dev->hard_reset_cnt <= N_HARD_RESET_COUNT)
	{
		pe_set_state(dev, PE_SNK_HARD_RESET);
	}

	return;
}

static void timeout_ps_transition(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if (dev->hard_reset_cnt <= N_HARD_RESET_COUNT)
	{
		pe_set_state(dev, PE_SNK_HARD_RESET);
	}

	return;
}

static void timeout_sink_request(unsigned int port)
{
	pe_set_state(&pd[port], PE_SNK_SELECT_CAPABILITY);
	return;
}

static void pe_snk_startup_entry(usb_pd_port_t *dev)
{
	DUAL_ROLE_CHANGED();

	usb_pd_prl_reset(dev->port);
	dev->non_interruptable_ams = false;
	dev->power_role_swap_in_progress = false;

	pe_set_state(dev, PE_SNK_DISCOVERY);

	return;
}

static void pe_snk_discovery_entry(usb_pd_port_t *dev)
{
	if (dev->vbus_present)
	{
		pe_set_state(dev, PE_SNK_WAIT_FOR_CAPS);
	}

	return;
}

static void pe_snk_wait_for_caps_entry(usb_pd_port_t *dev)
{
	// VBUS = 5V.
	// Re-enable sink VBUS if it was previously disabled due to hard reset.
	if (dev->hard_reset_complete)
	{
		dev->hard_reset_complete = false;
//		tcpm_sink_vbus(dev->port, false, 5000, GET_SRC_CURRENT_MA(dev->tcpc_dev->src_current_adv));
	}
    tcpm_sink_vbus(dev->port, false, 5000, GET_SRC_CURRENT_MA(dev->tcpc_dev->src_current_adv));

	// Re-enable AutoDischargeDisconnect and VBUS detection.
	// (May have been disabled due to hard reset or power role swap)
	tcpm_set_autodischarge_disconnect(dev->port, true);
	tcpm_enable_vbus_detect(dev->port);

	// Enable PD receive for SOP.
	tcpm_enable_pd_receive(dev->port, false, false);

	timer_start(&dev->timer, T_TYPEC_SINK_WAIT_CAP_MS, timeout_sink_wait_cap);

	return;
}

static void pe_snk_evaluate_capability_entry(usb_pd_port_t *dev)
{
	// Reset HardResetCounter to zero.
	dev->hard_reset_cnt = 0;

	dev->non_interruptable_ams = true;

	// Check the spec revision from the Source Caps message and set the 
	// port to port spec revision to the highest value supported by both ports.
	if (dev->p2p_spec_rev > dev->rx_msg_spec_rev)
	{
		dev->p2p_spec_rev = dev->rx_msg_spec_rev;
	}

	if (dev->explicit_contract)
	{
		// Save current PDO in case request gets Reject or Wait response.
		dev->prev_selected_pdo = dev->selected_pdo;
	}

	// Ask policy manager to evaluate options based on supplied capabilities.
	usb_pd_pm_evaluate_src_caps(dev->port);

	pe_set_state(dev, PE_SNK_SELECT_CAPABILITY);

	return;
}

static void pe_snk_select_capability_entry(usb_pd_port_t *dev)
{
	uint16_t threshold_25mv;

	if (dev->explicit_contract)
	{
		if (PDO_MIN_VOLTAGE(dev->selected_pdo) < PDO_MIN_VOLTAGE(dev->prev_selected_pdo))
		{
			// Calculate threshold equal to 80% of new min voltage.
			threshold_25mv = (PDO_MIN_VOLTAGE(dev->selected_pdo) * 16) / 10;

			// Set new lower threshold to detect VBUS sink disconnect and
			// start discharge when AUTO_DISCHARGE_DISCONNECT is enabled.
			tcpm_set_sink_disconnect_threshold(dev->port, threshold_25mv);
		}
	}

	// Clear sink Wait and GotoMin flags.
	dev->snk_wait = false;
	dev->snk_goto_min = false;

	// Build RDO based on policy manager response.
	build_rdo(dev->port);

	// Send RDO.
	usb_pd_pe_tx_data_msg(dev->port, DATA_MSG_TYPE_REQUEST, TCPC_TX_SOP);

	return;
}

static void pe_snk_transition_sink_entry(usb_pd_port_t *dev)
{
	uint16_t rdo_operational_curr_or_pwr;

	timer_start(&dev->timer, T_PS_TRANSITION_MS, timeout_ps_transition);

	// Request policy manager to transition to new power level and wait for PS_RDY from source.

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	if (PDO_SUPPLY_TYPE(dev->selected_pdo) == SUPPLY_TYPE_PROGRAMMABLE)
	{
		rdo_operational_curr_or_pwr = PROG_RDO_OPERATIONAL_CURRENT(dev->rdo);
	}
	else /* Fixed, variable, or battery */
#endif
	{
		if (dev->snk_goto_min)
		{
			rdo_operational_curr_or_pwr = RDO_MIN_OPERATIONAL_CURRENT_OR_POWER(dev->rdo);
		}
		else
		{
			rdo_operational_curr_or_pwr = RDO_OPERATIONAL_CURRENT_OR_POWER(dev->rdo);
		}
	}

	if (PDO_SUPPLY_TYPE(dev->selected_pdo) == SUPPLY_TYPE_FIXED)
	{
		tcpm_sink_vbus(dev->port, true, 
					   PDO_VOLT_TO_MV(PDO_MIN_VOLTAGE(dev->selected_pdo)),
					   PDO_CURR_TO_MA(rdo_operational_curr_or_pwr));
	}
	else if (PDO_SUPPLY_TYPE(dev->selected_pdo) == SUPPLY_TYPE_VARIABLE)
	{
		tcpm_sink_vbus_vari(dev->port,
							PDO_VOLT_TO_MV(PDO_MIN_VOLTAGE(dev->selected_pdo)),
							PDO_VOLT_TO_MV(PDO_MAX_VOLTAGE(dev->selected_pdo)),
							PDO_CURR_TO_MA(rdo_operational_curr_or_pwr));
	}
	else if (PDO_SUPPLY_TYPE(dev->selected_pdo) == SUPPLY_TYPE_BATTERY)
	{
		tcpm_sink_vbus_batt(dev->port,
							PDO_VOLT_TO_MV(PDO_MIN_VOLTAGE(dev->selected_pdo)),
							PDO_VOLT_TO_MV(PDO_MAX_VOLTAGE(dev->selected_pdo)),
							PDO_PWR_TO_MW(rdo_operational_curr_or_pwr));
	}
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	else /* SUPPLY_TYPE_PROGRAMMABLE */
	{
		tcpm_sink_vbus(dev->port, true, 
					   PROG_RDO_OUTPUT_VOLTAGE(dev->rdo) * 20,
					   rdo_operational_curr_or_pwr * 50);
	}
#endif

	return;
}

#ifdef ENABLE_USB_PD_REV3_SUPPORT
static void timeout_pps_request(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	pe_set_state(dev, PE_SNK_SELECT_CAPABILITY);
	return;
}
#endif

#define DEFAULT_5V  (5000 / 25)  /* 25mV LSB */

static void pe_snk_ready_entry(usb_pd_port_t *dev)
{
	uint16_t min_voltage;
	uint16_t threshold;

	// Notify PRL of end of AMS.
	dev->non_interruptable_ams = false;

	dev->explicit_contract = true;

	//Falong
	pb_pd_explicit_contract_cb(false, true);

	if (dev->snk_wait)
	{
		// Start SinkRequest timer.
		timer_start(&dev->timer, T_SINK_REQUEST_MS, timeout_sink_request);
	}

	// Get PDO min voltage (multiply by 2 to convert to 25mV LSB) so we can set discharge voltage threshold. 
	min_voltage = PDO_MIN_VOLTAGE(dev->selected_pdo) << 1;

	if (min_voltage == DEFAULT_5V)
	{
		// Set Sink Disconnect Threshold to zero.
		threshold = 0;
	}
	else
	{
		// Set Sink Disconnect Threshold to 80% of min voltage.
		threshold = (min_voltage * 8) / 10;
		DEBUG("SNK VBUS Disconn Thres = %u mV.\n", threshold * 25);
	}

	// Set non-zero value to specify VBUS threshold for sink disconnect detection.
	// Otherwise, the default threshold (3.75V per TUSB422 design) for VBUS present will be used.
	tcpm_set_sink_disconnect_threshold(dev->port, threshold);

#ifdef DP_DONGLE
	// Weaken Ra.
	tcpm_ra_ctrl(dev->port, true);

	// Enable power to TUSB544. The TUSB544 will not draw significant power 
	// until it is configured and HPD pin is asserted.
	if (tcpm_redriver_pwr_ctrl(dev->port, true) != 0)
	{
		usb_pd_pe_update_dp_status(0, 0, ADAPTER_POWER_LOW);
	}
#endif

	usb_pd_pe_handle_snk_src_ready_state(dev);

	if (dev->error_recovery_pending)
	{
		dev->error_recovery_pending = false;
		tusb422_lfo_timer_start(&dev->timer2, 1000, timeout_snk_error_recovery_pending);
	}
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	else if (PDO_SUPPLY_TYPE(dev->selected_pdo) == SUPPLY_TYPE_PROGRAMMABLE)
	{
		// If a programmable power supply is selected, the request must be 
		// periodically resent every tPPSRequest (10sec).
		tusb422_lfo_timer_start(&dev->timer2, T_PPS_REQUEST_MS, timeout_pps_request);
	}
#endif

	return;
}

static void pe_snk_hard_reset_entry(usb_pd_port_t *dev)
{
	dev->hard_reset_cnt++;
	tcpm_transmit(dev->port, NULL, TCPC_TX_HARD_RESET, 0);

	return;
}

static void timeout_sink_vbus_off(unsigned int port)
{
	// Re-enable AutoDischargeDisconnect.
	// (May have been disabled due to hard reset or power role swap)
	tcpm_set_autodischarge_disconnect(port, true);

	// Re-enable VBUS detect.
	tcpm_enable_vbus_detect(port);

	// Re-enable sink VBUS.
	tcpm_snk_vbus_enable(port);

	return;
}

static void pe_snk_transition_to_default_entry(usb_pd_port_t *dev)
{
	if (dev->active_alt_modes)
	{
		// Isolate any pins (including SBUs) previously configured for alt mode.
		// Pins will be configured for USB after VBUS reaches vSafe0V.
		tcpm_mux_control(dev->port, dev->data_role, MUX_DISABLE, dev->tcpc_dev->plug_polarity);
	}

	usb_pd_pe_handle_snk_src_default_state(dev);

	// Disable AutoDischargeDisconnect and VBUS sink.
	tcpm_snk_swap_standby(dev->port);

	// Request policy manager to set data role to UFP.
	if (dev->data_role != PD_DATA_ROLE_UFP)
	{
		dev->data_role = PD_DATA_ROLE_UFP;

		tcpm_update_msg_header_info(dev->port, dev->data_role, dev->power_role);

#ifdef CONFIG_TUSB422_PAL
		// BQ - should we notify in PE_SNK_Wait_for_Capabilities when VBUS is present instead? 
		usb_pd_pal_notify_connect_state(dev->port, dev->tcpc_dev->state, dev->tcpc_dev->plug_polarity);
//        usb_pd_pal_data_role_swap(dev->port, PD_DATA_ROLE_UFP);
#endif
	}

	// Source must reach vSafe0V within 685ms (tPSHardReset + tSafe0V)
	timer_start(&dev->timer, T_SINK_VBUS_OFF_MS, timeout_sink_vbus_off);

	// During a Hard Reset the Source voltage will transition to vSafe0V and then 
	// transition to vSafe5V. Sinks need to ensure that VBUS present is not indicated 
	// until after the Source has completed the Hard Reset process by detecting both of these transitions.
	dev->vbus_present = false;
	tcpm_set_voltage_alarm_lo(dev->port, VSAFE0V_MAX);

	// Set flag to re-enable sink VBUS in PE_SNK_WAIT_FOR_CAPS.
	dev->hard_reset_complete = true;

	// Assume sink has reached default level and transition to SNK_STARTUP.
	pe_set_state(dev, PE_SNK_STARTUP);

	return;
}

static void pe_snk_give_sink_cap_entry(usb_pd_port_t *dev)
{
	// Request Sink Caps from policy manager.

	usb_pd_pe_tx_data_msg(dev->port, DATA_MSG_TYPE_SNK_CAPS, TCPC_TX_SOP);
	return;
}

static void pe_snk_get_source_cap_entry(usb_pd_port_t *dev)
{
	usb_pd_prl_tx_ctrl_msg(dev->port, CTRL_MSG_TYPE_GET_SRC_CAP, TCPC_TX_SOP);
	return;
}

static void timeout_bist_cont_mode(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

	if (dev->power_role == PD_PWR_ROLE_SNK)
	{
		pe_set_state(dev, PE_SNK_TRANSITION_TO_DEFAULT);
	}
	else
	{
		pe_set_state(dev, PE_SRC_TRANSITION_TO_DEFAULT);
	}

	return;
}


static void pe_bist_carrier_mode_entry(usb_pd_port_t *dev)
{
	// Start BIST transmit.  TCPC HW will automatically stop transmit after tBISTContMode.
	tcpm_transmit(dev->port, NULL, TCPC_TX_BIST_MODE2, 0);

	timer_start(&dev->timer, T_BIST_CONT_MODE_MS, timeout_bist_cont_mode);

	return;
}

static void pe_bist_test_mode_entry(usb_pd_port_t *dev)
{
	return;
}

static void pe_error_recovery_entry(usb_pd_port_t *dev)
{
	tcpm_execute_error_recovery(dev->port);
	return;
}

static void pe_dr_src_give_sink_caps_entry(usb_pd_port_t *dev)
{
	usb_pd_pe_tx_data_msg(dev->port, DATA_MSG_TYPE_SNK_CAPS, TCPC_TX_SOP);
	return;
}


static void pe_dr_snk_give_source_caps_entry(usb_pd_port_t *dev)
{
	usb_pd_pe_tx_data_msg(dev->port, DATA_MSG_TYPE_SRC_CAPS, TCPC_TX_SOP);
	return;
}

static void pe_dummy_state_entry(usb_pd_port_t *dev)
{
	INFO("%s\n", __func__);
	return;
}

typedef void (*state_entry_fptr)(usb_pd_port_t *dev);

static const state_entry_fptr pe_state_entry[PE_NUM_STATES] =
{
	pe_dummy_state_entry,                 /* PE_UNATTACHED                 */
	pe_src_startup_entry,                 /* PE_SRC_STARTUP                */
	pe_src_discovery_entry,               /* PE_SRC_DISCOVERY              */
	pe_src_send_caps_entry,               /* PE_SRC_SEND_CAPS              */
	pe_src_negotiate_capability_entry,    /* PE_SRC_NEGOTIATE_CAPABILITY   */
	pe_src_transition_supply_entry,       /* PE_SRC_TRANSITION_SUPPLY      */
	pe_src_transition_supply_exit,        /* PE_SRC_TRANSITION_SUPPLY_EXIT */
	pe_src_ready_entry,                   /* PE_SRC_READY                  */
	pe_src_disabled_entry,                /* PE_SRC_DISABLED               */
	pe_src_capability_response_entry,     /* PE_SRC_CAPABILITY_RESPONSE    */
	pe_src_hard_reset_entry,              /* PE_SRC_HARD_RESET             */
	pe_src_hard_reset_received_entry,     /* PE_SRC_HARD_RESET_RECEIVED    */
	pe_src_transition_to_default_entry,   /* PE_SRC_TRANSITION_TO_DEFAULT  */
	pe_src_get_sink_cap_entry,            /* PE_SRC_GET_SINK_CAP           */
	pe_src_wait_new_caps_entry,           /* PE_SRC_WAIT_NEW_CAPS          */
	pe_send_soft_reset_entry,             /* PE_SRC_SEND_SOFT_RESET        */
	pe_send_accept_entry,                 /* PE_SRC_SOFT_RESET             */
	pe_src_ping_entry,                    /* PE_SRC_PING                   */
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	pe_dummy_state_entry,                 /* PE_SRC_SEND_SOURCE_ALERT      */
	pe_dummy_state_entry,                 /* PE_SRC_SINK_ALERT_RECEIVED    */
	pe_send_not_supported_entry,          /* PE_SRC_GIVE_SOURCE_CAP_EXT    */
	pe_send_status_entry,                 /* PE_SRC_GIVE_SOURCE_STATUS     */
	pe_dummy_state_entry,                 /* PE_SRC_GET_SINK_STATUS        */
#endif

	pe_snk_startup_entry,                 /* PE_SNK_STARTUP                */
	pe_snk_discovery_entry,               /* PE_SNK_DISCOVERY              */
	pe_snk_wait_for_caps_entry,           /* PE_SNK_WAIT_FOR_CAPS          */
	pe_snk_evaluate_capability_entry,     /* PE_SNK_EVALUATE_CAPABILITY    */
	pe_snk_select_capability_entry,       /* PE_SNK_SELECT_CAPABILITY      */
	pe_snk_transition_sink_entry,         /* PE_SNK_TRANSITION_SINK        */
	pe_snk_ready_entry,                   /* PE_SNK_READY                  */
	pe_snk_hard_reset_entry,              /* PE_SNK_HARD_RESET             */
	pe_snk_transition_to_default_entry,   /* PE_SNK_TRANSITION_TO_DEFAULT  */
	pe_snk_give_sink_cap_entry,           /* PE_SNK_GIVE_SINK_CAP          */
	pe_snk_get_source_cap_entry,          /* PE_SNK_GET_SOURCE_CAP         */
	pe_send_soft_reset_entry,             /* PE_SNK_SEND_SOFT_RESET        */
	pe_send_accept_entry,                 /* PE_SNK_SOFT_RESET             */
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	pe_dummy_state_entry,                 /* PE_SNK_SOURCE_ALERT_RECEIVED  */
	pe_dummy_state_entry,                 /* PE_SNK_SEND_SINK_ALERT        */
	pe_send_not_supported_entry,          /* PE_SNK_GET_SOURCE_CAP_EXT     */
	pe_dummy_state_entry,                 /* PE_SNK_GET_SOURCE_STATUS      */
	pe_send_status_entry,                 /* PE_SNK_GIVE_SINK_STATUS       */
#endif

	pe_send_reject_entry,                 /* PE_SEND_REJECT                */
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	pe_send_not_supported_entry,          /* PE_SEND_NOT_SUPPORTED         */
#endif

	pe_dr_src_give_sink_caps_entry,       /* PE_DR_SRC_GIVE_SINK_CAP       */
	pe_dr_snk_give_source_caps_entry,     /* PE_DR_SNK_GIVE_SOURCE_CAP     */
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	pe_dummy_state_entry,                 /* PE_DR_SRC_GET_SOURCE_CAP_EXT  */
	pe_dummy_state_entry,                 /* PE_DR_SNK_GIVE_SOURCE_CAP_EXT */
#endif

	pe_bist_carrier_mode_entry,           /* PE_BIST_CARRIER_MODE          */
	pe_bist_test_mode_entry,              /* PE_BIST_TEST_MODE             */

	pe_error_recovery_entry,              /* PE_ERROR_RECOVERY             */

	pe_drs_send_swap_entry,               /* PE_DRS_SEND_SWAP              */
	pe_drs_evaluate_swap_entry,           /* PE_DRS_EVALUATE_SWAP          */
	pe_send_accept_entry,                 /* PE_DRS_ACCEPT_SWAP            */
	pe_drs_change_role_entry,             /* PE_DRS_CHANGE_ROLE            */

	pe_prs_send_swap_entry,               /* PE_PRS_SEND_SWAP              */
	pe_prs_evaluate_swap_entry,           /* PE_PRS_EVALUATE_SWAP          */
	pe_send_wait_entry,                   /* PE_PRS_WAIT_SWAP              */
	pe_send_accept_entry,                 /* PE_PRS_ACCEPT_SWAP            */
	pe_prs_transition_to_off_entry,       /* PE_PRS_TRANSITION_TO_OFF      */
	pe_prs_assert_rd_entry,               /* PE_PRS_ASSERT_RD              */
	pe_prs_wait_source_on_entry,          /* PE_PRS_WAIT_SOURCE_ON         */
	pe_prs_assert_rp_entry,               /* PE_PRS_ASSERT_RP              */
	pe_prs_source_on_entry,               /* PE_PRS_SOURCE_ON              */
	pe_prs_source_on_exit,                /* PE_PRS_SOURCE_ON_EXIT         */

	pe_vcs_send_swap_entry,               /* PE_VCS_SEND_SWAP              */
	pe_vcs_evaluate_swap_entry,           /* PE_VCS_EVALUATE_SWAP          */
	pe_send_accept_entry,                 /* PE_VCS_ACCEPT_SWAP            */
	pe_vcs_wait_for_vconn_entry,          /* PE_VCS_WAIT_FOR_VCONN         */
	pe_vcs_turn_off_vconn_entry,          /* PE_VCS_TURN_OFF_VCONN         */
	pe_vcs_turn_on_vconn_entry,           /* PE_VCS_TURN_ON_VCONN          */
	pe_vcs_send_ps_rdy_entry,             /* PE_VCS_SEND_PS_RDY            */

#ifdef ENABLE_VDM_SUPPORT
	pe_src_vdm_identity_request_entry,    /* PE_SRC_VDM_IDENTITY_REQUEST   */  
	pe_init_port_vdm_identity_request_entry,  /* PE_INIT_PORT_VDM_IDENTITY_REQUEST  */
	pe_init_vdm_svids_request_entry,      /* PE_INIT_VDM_SVIDS_REQUEST     */
	pe_init_vdm_modes_request_entry,      /* PE_INIT_VDM_MODES_REQUEST     */
	pe_init_vdm_attention_request_entry,  /* PE_INIT_VDM_ATTENTION_REQUEST */
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	pe_init_vdm_dp_status_update_entry,   /* PE_INIT_VDM_DP_STATUS_UPDATE  */
	pe_init_vdm_dp_config_entry,          /* PE_INIT_VDM_DP_CONFIG         */
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	pe_init_vdm_hdmi_status_update_entry, /* PE_INIT_VDM_HDMI_STATUS_UPDATE */
	/* PE_INIT_VDM_HDMI_CEC_WRITE    */
	/* PE_INIT_VDM_HDMI_CEC_READ     */
	pe_init_vdm_hdmi_ddc_write_entry,     /* PE_INIT_VDM_HDMI_DDC_WRITE     */
	pe_init_vdm_hdmi_ddc_read_entry,      /* PE_INIT_VDM_HDMI_DDC_READ      */
#endif

	pe_resp_vdm_nak_entry,                /* PE_RESP_VDM_NAK               */
	pe_resp_vdm_get_identity_entry,       /* PE_RESP_VDM_GET_IDENTITY      */
	pe_resp_vdm_get_svids_entry,          /* PE_RESP_VDM_GET_SVIDS         */
	pe_resp_vdm_get_modes_entry,          /* PE_RESP_VDM_GET_MODES         */
	pe_dummy_state_entry,                 /* PE_RCV_VDM_ATTENTION_REQUEST  */
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	pe_resp_vdm_dp_status_update_entry,   /* PE_RESP_VDM_DP_STATUS_UPDATE  */
	pe_resp_vdm_dp_config_entry,          /* PE_RESP_VDM_DP_CONFIG         */
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	pe_resp_vdm_hdmi_status_update_entry, /* PE_RESP_VDM_HDMI_STATUS_UPDATE */
	/* PE_RESP_VDM_HDMI_CEC_WRITE     */
	/* PE_RESP_VDM_HDMI_CEC_READ      */
	pe_resp_vdm_hdmi_ddc_write_entry,     /* PE_RESP_VDM_HDMI_DDC_WRITE     */
	pe_resp_vdm_hdmi_ddc_read_entry,      /* PE_RESP_VDM_HDMI_DDC_READ      */
#endif 

	pe_dfp_vdm_mode_entry_request,        /* PE_DFP_VDM_MODE_ENTRY_REQUEST */
	pe_dfp_vdm_mode_exit_request,         /* PE_DFP_VDM_MODE_EXIT_REQUEST  */

	pe_ufp_vdm_eval_mode_entry,           /* PE_UFP_VDM_EVAL_MODE_ENTRY    */
	pe_ufp_vdm_mode_exit_entry,           /* PE_UFP_VDM_MODE_EXIT          */
#endif

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	pe_dummy_state_entry,                 /* PE_GET_BATTERY_CAP            */
	pe_send_not_supported_entry,          /* PE_GIVE_BATTERY_CAP           */
	pe_dummy_state_entry,                 /* PE_GET_BATTERY_STATUS         */
	pe_send_not_supported_entry,          /* PE_GIVE_BATTERY_STATUS        */
	pe_dummy_state_entry,                 /* PE_GET_MANUFACTURER_INFO      */
	pe_give_manufacturer_info_entry,      /* PE_GIVE_MANUFACTURER_INFO     */
	pe_dummy_state_entry,                 /* PE_SEND_SECURITY_REQUEST      */
	pe_dummy_state_entry,                 /* PE_SEND_SECURITY_RESPONSE     */
	pe_dummy_state_entry,                 /* PE_SECURITY_RESP_RECEIVED     */
	pe_dummy_state_entry,                 /* PE_SEND_FW_UPDATE_REQUEST     */
	pe_dummy_state_entry,                 /* PE_SEND_FW_UPDATE_RESPONSE    */
	pe_dummy_state_entry,                 /* PE_FW_UPDATE_RESP_RECEIVED    */
#endif
};


void usb_pd_pe_state_machine(unsigned int port)
{
	usb_pd_port_t *dev = &pd[port];

#if defined ENABLE_VDM_SUPPORT && defined ENABLE_DP_ALT_MODE_SUPPORT && defined DP_DONGLE
	if ((dev->active_alt_modes & DP_ALT_MODE) && dev->hpd_in_changed)
	{
		dev->hpd_in_changed = false;

		if (dev->hpd_in_value == 0)
		{
			// Start timer to see if this is an HPD IRQ.
			tusb422_lfo_timer_start(&dev->timer2, T_IRQ_HPD_MAX_MS, timeout_irq_hpd);
		}
		else /* HPD high */
		{
			if (dev->timer2.function == timeout_irq_hpd)
			{
				// Timer did not expire. IRQ_HPD occured.
				tusb422_lfo_timer_cancel(&dev->timer2);
				dev->hpd_in_queue[dev->hpd_in_queue_idx++] = NOTIFY_HPD_IRQ; 
				PRINT("-> HPD IRQ\n");
			}
			else
			{
				// Drive HPD to redriver high.
				tcpm_redriver_hpd_ctrl(port, 1);

				dev->displayport_status |= UFP_D_CONNECTED;
				dev->hpd_in_queue[dev->hpd_in_queue_idx++] = NOTIFY_HPD_HIGH; 
			}

			if (dev->hpd_in_queue_idx >= HPD_IN_QUEUE_SIZE)
			{
				dev->hpd_in_queue_idx = 0;
			}

			dev->attention_pending = true;
		}
	}
#endif

#if defined ENABLE_VDM_SUPPORT && defined ENABLE_HDMI_ALT_MODE_SUPPORT && defined HDMI_DONGLE
	uint16_t i, j, offset;
	uint8_t ext_blocks, segment;

	// To avoid potential PD negotiation timeouts, do not read EDID until after
	// HDMI Alt-Mode is active and the initial status update has been completed.
	if (dev->hdmi_initial_status_update_complete && dev->hpd_in_changed)
	{
		dev->hpd_in_changed = false;

		if (dev->hpd_in_value)
		{
			// Read EDID from monitor.
			for (i = 0; i < 128; i += 32)
			{
				hdmi_read(port, (EDID_SLAVE_ADDR >> 1), 0, i, &dev->hdmi_edid[i], 32);
			}

			// Get the number of (optional) 128-byte EDID extension blocks to follow.
			ext_blocks = dev->hdmi_edid[0x7E];
			INFO("ext_blocks = %u\n", ext_blocks);

			if (ext_blocks)
			{
				// Read the remaining EDID blocks if any.
				ext_blocks = MIN(ext_blocks, NUM_EDID_BLOCKS - 1);
				segment = 0;

				for (i = 0; i < ext_blocks; i++)
				{
					// Each segment contains two blocks of EDID data (256-bytes total).  
					// Segment 0 has EDID blocks 1 & 2, Segment 1 has EDID blocks 3 & 4, etc.
					if (i & 0x1)
					{
						offset = 0;
						segment++;
					}
					else
					{
						offset = 0x80;
					}

					// Read each EDID block with four 32-byte reads.
					for (j = 0; j < 128; j+=32)
					{
						hdmi_read(port, (EDID_SLAVE_ADDR >> 1), segment, offset, 
								  &dev->hdmi_edid[(segment * 0x100) + offset], 32);
						offset += 32;
					}
				}
			}

			for (i = 0; i <= ext_blocks; i++)
			{
				CRIT("EDID[%u] = 0x", i + 1);
				for (j = 0; j < 128; j++)
				{
					CRIT("%02x ", dev->hdmi_edid[j + (i * 0x80)]);
				}
				CRIT("\n");
			}

			// Set HPD status and EDID availability.
			dev->hdmi_status |= EDID_AVAILABLE | HPD_STATUS;
		}
		else /* HPD low */
		{
			// Clear HPD status and EDID availability.
			dev->hdmi_status &= ~(EDID_AVAILABLE | HPD_STATUS);
		}

		dev->attention_pending = true;
	}
#endif

	while (dev->state_change)
	{
		dev->state_change = false;

		// Use branch table to execute "actions on entry or exit" for the current state.
		if (*dev->current_state < PE_NUM_STATES)
		{
			pe_state_entry[*dev->current_state](dev);
		}
	}

#ifdef ENABLE_VDM_SUPPORT
	if (dev->attention_pending && dev->active_alt_modes &&
		((*dev->current_state == PE_SNK_READY) || 
		 (*dev->current_state == PE_SRC_READY)))
	{
#ifdef HDMI_DONGLE
		// If HDMI alt-mode is active, check if the initial status update request has been completed.
		if (((dev->active_alt_modes & HDMI_ALT_MODE) && dev->hdmi_initial_status_update_complete) ||
			!(dev->active_alt_modes & HDMI_ALT_MODE))
#endif
		{
			dev->attention_pending = false;
			pe_set_state(dev, PE_INIT_VDM_ATTENTION_REQUEST);
			dev->state_change = false;
			pe_state_entry[*dev->current_state](dev);
		}
	}
#endif

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	if (*dev->current_state != PE_UNATTACHED)
	{
		usb_pd_prl_chunking_state_machine(port);
	}
#endif

	return;
}


void usb_pd_pe_notify(unsigned int port, usb_pd_prl_alert_t prl_alert)
{
	usb_pd_port_t *dev = &pd[port];

	switch (prl_alert)
	{
		case PRL_ALERT_HARD_RESET_RECEIVED:
			// PD message passing is enabled in protocol layer.

			if (dev->power_role == PD_PWR_ROLE_SNK)
			{
				// Stop sender response timer.
				timer_cancel(&dev->timer);

				pe_set_state(dev, PE_SNK_TRANSITION_TO_DEFAULT);
			}
			else /* SRC */
			{
				pe_set_state(dev, PE_SRC_HARD_RESET_RECEIVED);
			}
			break;

		case PRL_ALERT_MSG_TX_SUCCESS:	 /* GoodCRC received */
			switch (*dev->current_state)
			{
				case PE_SRC_SEND_CAPS:
					dev->caps_cnt = 0;
					dev->hard_reset_cnt = 0;
					timer_cancel_no_response(dev);
					dev->non_interruptable_ams = true;
					dev->pd_connected = true;
					dev->pd_connected_since_attach = true;
					timer_start(&dev->timer, T_SENDER_RESPONSE_MS, timeout_sender_response);
					break;

				case PE_SRC_HARD_RESET:
					// Do nothing.  Wait for PSHardReset timer to expire to transition to default.
					break;

				case PE_SNK_HARD_RESET:
					pe_set_state(dev, PE_SNK_TRANSITION_TO_DEFAULT);
					break;

				case PE_SRC_GET_SINK_CAP:
				case PE_DRS_SEND_SWAP:
				case PE_PRS_SEND_SWAP:
				case PE_VCS_SEND_SWAP:
				case PE_SNK_SEND_SOFT_RESET:
				case PE_SRC_SEND_SOFT_RESET:
				case PE_SNK_SELECT_CAPABILITY:
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
				case PE_GET_BATTERY_CAP:
				case PE_GET_BATTERY_STATUS:
				case PE_GET_MANUFACTURER_INFO:
				case PE_SRC_GET_SINK_STATUS:
				case PE_SNK_GET_SOURCE_CAP_EXT:
				case PE_SNK_GET_SOURCE_STATUS:
				case PE_DR_SRC_GET_SOURCE_CAP_EXT:
#endif
					dev->non_interruptable_ams = true;
					timer_start(&dev->timer, T_SENDER_RESPONSE_MS, timeout_sender_response);
					break;

				case PE_SNK_SOFT_RESET:
					// Accept msg was successfully sent.
					dev->non_interruptable_ams = false;
					pe_set_state(dev, PE_SNK_WAIT_FOR_CAPS);
					break;

				case PE_SRC_SOFT_RESET:
					// Accept msg was successfully sent.
					dev->non_interruptable_ams = false;
					pe_set_state(dev, PE_SRC_SEND_CAPS);
					break;

				case PE_SRC_TRANSITION_SUPPLY:
					timer_start(&dev->timer, T_SRC_TRANSITION_MS, timeout_src_transition);
					break;

				case PE_SRC_CAPABILITY_RESPONSE:
					if (dev->explicit_contract /* || send_wait*/)
					{
						// Reject and Current contract is still valid or wait sent.
						pe_set_state(dev, PE_SRC_READY);

						// If explicit contract and current contract is invalid go to hard reset. - BQ
					}
					else /* No explicit contract */
					{
						pe_set_state(dev, PE_SRC_WAIT_NEW_CAPS);
					}
					break;

				case PE_SRC_TRANSITION_SUPPLY_EXIT:
				case PE_DR_SRC_GIVE_SINK_CAP:
				case PE_SRC_PING:
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
				case PE_SRC_SEND_SOURCE_ALERT:
				case PE_SRC_GIVE_SOURCE_CAP_EXT:
				case PE_SRC_GIVE_SOURCE_STATUS:
#endif
					pe_set_state(dev, PE_SRC_READY);
					break;

				case PE_SNK_GET_SOURCE_CAP:
				case PE_SNK_GIVE_SINK_CAP:
				case PE_DR_SNK_GIVE_SOURCE_CAP:
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
				case PE_SNK_SEND_SINK_ALERT:
				case PE_SNK_GIVE_SINK_STATUS:
#endif
					pe_set_state(dev, PE_SNK_READY);
					break;

				case PE_PRS_SOURCE_ON_EXIT:
					pe_set_state(dev, PE_SRC_STARTUP);
					break;

				case PE_DRS_ACCEPT_SWAP:
					pe_set_state(dev, PE_DRS_CHANGE_ROLE);
					break;

				case PE_PRS_ACCEPT_SWAP:
					dev->power_role_swap_in_progress = true;
					pe_set_state(dev, PE_PRS_TRANSITION_TO_OFF);
					break;

				case PE_VCS_ACCEPT_SWAP:
					if (dev->vconn_source)
					{
						pe_set_state(dev, PE_VCS_WAIT_FOR_VCONN);
					}
					else
					{
						pe_set_state(dev, PE_VCS_TURN_ON_VCONN);
					}
					break;

#ifdef ENABLE_VDM_SUPPORT
				case PE_SRC_VDM_IDENTITY_REQUEST:
				case PE_INIT_PORT_VDM_IDENTITY_REQUEST:
				case PE_INIT_VDM_SVIDS_REQUEST:
				case PE_INIT_VDM_MODES_REQUEST:
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
				case PE_INIT_VDM_DP_STATUS_UPDATE:
				case PE_INIT_VDM_DP_CONFIG:
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
				case PE_INIT_VDM_HDMI_STATUS_UPDATE:
//				case PE_INIT_VDM_HDMI_CEC_WRITE:
//				case PE_INIT_VDM_HDMI_CEC_READ:
				case PE_INIT_VDM_HDMI_DDC_WRITE:
				case PE_INIT_VDM_HDMI_DDC_READ:
#endif
					// Start VDMResponseTimer.
					timer_start(&dev->timer, T_VDM_SENDER_RESPONSE_MS, timeout_vdm_sender_response);
					break;

				case PE_DFP_VDM_MODE_ENTRY_REQUEST:
					// Start VDMModeEntryTimer.
					timer_start(&dev->timer, T_VDM_WAIT_MODE_ENTRY_MS, timeout_vdm_mode_entry);
					break;

				case PE_DFP_VDM_MODE_EXIT_REQUEST:
					// Start VDMModeExitTimer.
					timer_start(&dev->timer, T_VDM_WAIT_MODE_EXIT_MS, timeout_vdm_mode_exit);
					break;

				case PE_INIT_VDM_ATTENTION_REQUEST:
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
					// IRQ_HPD is has been sent, clear the status. 
					dev->displayport_status &= ~IRQ_HPD;
					// Fall through...
				case PE_RESP_VDM_DP_STATUS_UPDATE:
				case PE_RESP_VDM_DP_CONFIG:
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
				case PE_RESP_VDM_HDMI_STATUS_UPDATE:
#ifdef HDMI_DONGLE
					dev->hdmi_initial_status_update_complete = true;
					// Fall through...
#endif
//				case PE_RESP_VDM_HDMI_CEC_WRITE:
//				case PE_RESP_VDM_HDMI_CEC_READ:
				case PE_RESP_VDM_HDMI_DDC_WRITE:
				case PE_RESP_VDM_HDMI_DDC_READ:
#endif
				case PE_RESP_VDM_GET_IDENTITY:
				case PE_RESP_VDM_GET_SVIDS:
				case PE_RESP_VDM_GET_MODES:
				case PE_RESP_VDM_NAK:
				case PE_UFP_VDM_EVAL_MODE_ENTRY: /* for ACK Resp */
				case PE_UFP_VDM_MODE_EXIT:       /* for ACK Resp */
					dev->vdm_in_progress = false;
					// Fall through...
#endif /* ENABLE_VDM_SUPPORT */
				case PE_PRS_WAIT_SWAP:
				case PE_VCS_SEND_PS_RDY:
				case PE_SEND_REJECT:
#ifdef ENABLE_USB_PD_REV3_SUPPORT
				case PE_SEND_NOT_SUPPORTED:
#endif
					if (dev->power_role == PD_PWR_ROLE_SNK)
					{
						pe_set_state(dev, PE_SNK_READY);
					}
					else
					{
						pe_set_state(dev, PE_SRC_READY);
					}
					break;

				default:
					break;
			}
			break;

		case PRL_ALERT_MSG_TX_DISCARDED: /* Msg was received or Hard Reset issued before Tx */
			// Do nothing. Proceed to handle the received message or Hard Reset.
			INFO("TX discarded\n");
			break;

		case PRL_ALERT_MSG_TX_FAILED: /* No GoodCRC response */
#ifdef ENABLE_VDM_SUPPORT
			dev->vdm_in_progress = false;
#endif
			if (*dev->current_state == PE_PRS_WAIT_SOURCE_ON)
			{
				// Cancel PSSourceOn timer.
				timer_cancel(&dev->timer);

				// PS_RDY msg failed.
				pe_set_state(dev, PE_ERROR_RECOVERY);
			}
			else if (*dev->current_state == PE_PRS_SOURCE_ON_EXIT)
			{
				// PS_RDY msg failed.
				pe_set_state(dev, PE_ERROR_RECOVERY);
			}
			else if (dev->power_role == PD_PWR_ROLE_SNK)
			{
				if (*dev->current_state == PE_SNK_SEND_SOFT_RESET)
				{
					// Any failure in the Soft Reset process will trigger a Hard Reset.
					pe_set_state(dev, PE_SNK_HARD_RESET);
				}
				else if (*dev->current_state != PE_UNATTACHED)
				{
					// Failure to see a GoodCRC when a port pair 
					// is connected will result in a Soft Reset.
					pe_set_state(dev, PE_SNK_SEND_SOFT_RESET);
				}
			}
			else /* SRC */
			{
				if (*dev->current_state == PE_SRC_SEND_SOFT_RESET)
				{
					// Any failure in the Soft Reset process will trigger a Hard Reset.
					pe_set_state(dev, PE_SRC_HARD_RESET);
				}
				else if ((*dev->current_state == PE_SRC_SEND_CAPS) &&
						 (!dev->pd_connected))
				{
					pe_set_state(dev, PE_SRC_DISCOVERY);
				}
#ifdef ENABLE_VDM_SUPPORT
				else if (*dev->current_state == PE_SRC_VDM_IDENTITY_REQUEST)
				{
					// VDM Discover Identity msg to cable failed.

#ifdef ENABLE_USB_PD_REV3_SUPPORT
					// If there is no ACK after half the attempts are exhausted,
					// try the remainder of the attempts with PD spec revision 2.0. 
					if (dev->discover_identity_cnt == (N_DISCOVER_IDENTITY_COUNT >> 1))
					{
						dev->p2c_spec_rev = PD_REV20;
						dev->p2c_vdm_version = STRUCT_VDM_VER_1P0;
					}
#endif
					pe_set_state(dev, PE_SRC_DISCOVERY);
				}
#endif
				else if (*dev->current_state != PE_UNATTACHED)
				{
					// Failure to see a GoodCRC when a port pair 
					// is connected will result in a Soft Reset.
					pe_set_state(dev, PE_SRC_SEND_SOFT_RESET);
				}
			}
			break;

		case PRL_ALERT_MSG_RECEIVED:
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
			if (dev->rx_msg_extended)
			{
				usb_pd_pe_ext_msg_rx_handler(dev);
			}
			else
#endif			
			{
				if (dev->rx_msg_data_len)
				{
					usb_pd_pe_data_msg_rx_handler(dev);
				}
				else
				{
					usb_pd_pe_ctrl_msg_rx_handler(dev);
				}
			}
			break;

		default:
			break;
	}

	return;
}


int usb_pd_policy_manager_request(unsigned int port, pd_policy_manager_request_t req)
{
	usb_pd_port_t *dev = &pd[port];
	usb_pd_pe_status_t status = STATUS_OK;

	if (*dev->current_state == PE_SRC_READY)
	{
		if (req == PD_POLICY_MNGR_REQ_GET_SINK_CAPS)
		{
			pe_set_state(dev, PE_SRC_GET_SINK_CAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_SRC_CAPS_CHANGE)
		{
			pe_set_state(dev, PE_SRC_SEND_CAPS);
		}
		else if (req == PD_POLICY_MNGR_REQ_GOTO_MIN)
		{
			dev->request_goto_min = true;
			pe_set_state(dev, PE_SRC_TRANSITION_SUPPLY);
		}
		else if (req == PD_POLICY_MNGR_REQ_PR_SWAP)
		{
			pe_set_state(dev, PE_PRS_SEND_SWAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_DR_SWAP)
		{
			pe_set_state(dev, PE_DRS_SEND_SWAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_VCONN_SWAP)
		{
			pe_set_state(dev, PE_VCS_SEND_SWAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_HARD_RESET)
		{
			pe_set_state(dev, PE_SRC_HARD_RESET);
		}
		else
		{
			status = STATUS_REQUEST_NOT_SUPPORTED_IN_CURRENT_STATE;
		}
	}
	else if (*dev->current_state == PE_SRC_WAIT_NEW_CAPS)
	{
		if (req == PD_POLICY_MNGR_REQ_SRC_CAPS_CHANGE)
		{
			pe_set_state(dev, PE_SRC_SEND_CAPS);
		}
		else
		{
			status = STATUS_REQUEST_NOT_SUPPORTED_IN_CURRENT_STATE;
		}
	}
	else if (*dev->current_state == PE_SNK_READY)
	{
		if (req == PD_POLICY_MNGR_REQ_UPDATE_REMOTE_CAPS)
		{
			pe_set_state(dev, PE_SNK_GET_SOURCE_CAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_PR_SWAP)
		{
			pe_set_state(dev, PE_PRS_SEND_SWAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_DR_SWAP)
		{
			pe_set_state(dev, PE_DRS_SEND_SWAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_VCONN_SWAP)
		{
			pe_set_state(dev, PE_VCS_SEND_SWAP);
		}
		else if (req == PD_POLICY_MNGR_REQ_HARD_RESET)
		{
			pe_set_state(dev, PE_SNK_HARD_RESET);
		}
		else
		{
			status = STATUS_REQUEST_NOT_SUPPORTED_IN_CURRENT_STATE;
		}
	}
	else if (req == PD_POLICY_MNGR_REQ_STOP)
	{
		// Unattach is pending. Stop all timers.
		timer_cancel(&dev->timer);
		tusb422_lfo_timer_cancel(&dev->timer2);
		// Stop state machine.
		dev->state_change = false;
	}
	else
	{
		status = STATUS_REQUEST_NOT_SUPPORTED_IN_CURRENT_STATE;
	}

	usb_pd_pe_state_machine(0);

	return status;
}

bool usb_pd_pe_is_remote_externally_powered(unsigned int port)
{
	return pd[port].remote_externally_powered;
}

static void timeout_src_settling(unsigned int port)
{
	pe_set_state(&pd[port], PE_SRC_TRANSITION_SUPPLY_EXIT);
	return;
}

static void timeout_sink_vbus_recovery(unsigned int port)
{
	// Re-enable VBUS detect.
	tcpm_enable_vbus_detect(port);
	return;
}

void usb_pd_pe_voltage_alarm_handler(unsigned int port, bool hi_voltage)
{
	usb_pd_port_t *dev = &pd[port];
	usb_pd_port_config_t *config = usb_pd_pm_get_config(port);

	if (hi_voltage)
	{
		if (*dev->current_state == PE_SRC_TRANSITION_SUPPLY)
		{
			// Positive voltage transition complete.  
			// Start power supply settling timeout.
			timer_start(&dev->timer, config->src_settling_time_ms, timeout_src_settling);
		}
		else if (*dev->current_state == PE_PRS_SOURCE_ON)
		{
			timer_start(&dev->timer, T_VBUS_5V_STABLIZE_MS, timeout_vbus_5v_stabilize);
		}
		else if (*dev->current_state == PE_SRC_TRANSITION_TO_DEFAULT)
		{
			pe_set_state(dev, PE_SRC_STARTUP);
		}
		else if ((*dev->current_state == PE_SNK_STARTUP) ||
				 (*dev->current_state == PE_SNK_DISCOVERY))
		{
			// Cancel T_SINK_VBUS_RECOVERY_MS timer.
			timer_cancel(&dev->timer);

			dev->vbus_present = true;

			if (*dev->current_state == PE_SNK_DISCOVERY)
			{
				pe_set_state(dev, PE_SNK_WAIT_FOR_CAPS);
			}
		}
	}
	else /* Low voltage */
	{
		if (*dev->current_state == PE_SRC_TRANSITION_TO_DEFAULT)
		{
			// Request policy manager to set the Port Data Role to DFP.
			if (dev->data_role != PD_DATA_ROLE_DFP)
			{
				dev->data_role = PD_DATA_ROLE_DFP;

				tcpm_update_msg_header_info(dev->port, dev->data_role, dev->power_role);

#ifdef CONFIG_TUSB422_PAL
				usb_pd_pal_notify_connect_state(dev->port, dev->tcpc_dev->state, dev->tcpc_dev->plug_polarity);
//                usb_pd_pal_data_role_swap(dev->port, PD_DATA_ROLE_DFP);
#endif
			}

			// Start source recovery timer to enable VBUS 5V.
			timer_start(&dev->timer, T_SRC_RECOVER_MS, timeout_src_recover);
		}
		else if (*dev->current_state == PE_PRS_TRANSITION_TO_OFF)
		{
			if (dev->power_role == PD_PWR_ROLE_SRC)
			{
				pe_set_state(dev, PE_PRS_ASSERT_RD);
			}
		}
		else if (*dev->current_state == PE_SRC_TRANSITION_SUPPLY)
		{
			// Negative voltage transition complete.  
			// Start power supply settling timeout.
			timer_start(&dev->timer, config->src_settling_time_ms, timeout_src_settling);
		}
		else if ((*dev->current_state == PE_SNK_STARTUP) ||
				 (*dev->current_state == PE_SNK_DISCOVERY))
		{
			// Cancel T_SINK_VBUS_OFF_MS timer.
			timer_cancel(&dev->timer);

			// Start timeout for source to provide vSafe5V.
			timer_start(&dev->timer, T_SINK_VBUS_RECOVERY_MS, timeout_sink_vbus_recovery);

			if ((dev->tcpc_dev->cc_status & CC_STATUS_CC1_CC2_STATE_MASK) == CC_STATE_OPEN)
			{
				// Re-enable VBUS detect which may have been disabled during hard reset so disconnect will be detected.
				tcpm_enable_vbus_detect(dev->port);
			}
			else
			{
				// Configure pins for USB mode.
				tcpm_mux_control(dev->port, dev->data_role, MUX_USB, dev->tcpc_dev->plug_polarity);

				// Set Hi voltage alarm to determine when VBUS is present.
				tcpm_set_voltage_alarm_hi(dev->port, VSAFE5V_MIN);
			}
		}
	}

	return;
}


void usb_pd_pe_current_change_handler(unsigned int port, tcpc_cc_snk_state_t curr_adv)
{
	usb_pd_port_t *dev = &pd[port];

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	CRIT("Current Adv = %sA\n", (curr_adv == CC_SNK_STATE_POWER30) ? "3.0" :
		 (curr_adv == CC_SNK_STATE_POWER15) ? "1.5" : "500/900m");

	// Used by SNK for collision avoidance.
	if (curr_adv == CC_SNK_STATE_POWER30)
	{
		// Sink Tx OK.  Sink can initiate an AMS.
		dev->sink_tx_ok = true;

		if (*dev->current_state == PE_SNK_READY)
		{
			usb_pd_pe_snk_src_ready_initiate_ams(dev);
		}
	}
	else if (curr_adv == CC_SNK_STATE_POWER15)
	{
		// Sink Tx NG.
		dev->sink_tx_ok = false;
	}
#endif 

	if (!dev->explicit_contract && !dev->power_role_swap_in_progress &&
		(*dev->current_state != PE_UNATTACHED))
	{
		// Notify system of current change.
		tcpm_sink_vbus(port, false, 5000, GET_SRC_CURRENT_MA(dev->tcpc_dev->src_current_adv));
	}

	return;
}

void usb_pd_pe_init(unsigned int port, usb_pd_port_config_t *config)
{
	usb_pd_port_t *dev = &pd[port];
	unsigned int i;

	dev->tcpc_dev = tcpm_get_device(port);

	pd[port].state_idx = 0;

	for (i = 0; i < PD_STATE_HISTORY_LEN; i++)
	{
		pd[port].state[i] = (usb_pd_pe_state_t)0xEE;
	}

	dev->state_change = false;
	dev->current_state = &dev->state[0];

	dev->port = port;
	dev->timer.data = port;
	dev->timer2.data = port;
#if defined HDMI_DONGLE && defined ENABLE_HDMI_ALT_MODE_SUPPORT
	dev->hdmi_timer.data = port;
#endif
#ifdef DP_DONGLE
	dev->dp_timer.data = port;
#endif

	// Copy externally_powered from config as this parameter could change dynamically.
	dev->externally_powered = config->externally_powered;

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	dev->fast_role_swap_supported = (config->fast_role_swap_support) ? true : false;
	if (dev->fast_role_swap_supported)
	{
		tusb422_set_fast_role_swap_detect(port, true);
	}
#endif

	usb_pd_pe_handle_unattach(dev);

#ifdef HDMI_DONGLE
	// Check HPD in case HDMI was attached before Type-C.
	dev->hpd_in_value = tcpm_get_hpd_in(0);
	CRIT("HPD: %u\n", dev->hpd_in_value);
	if (dev->hpd_in_value)
	{
		// Start debounce. Use 750ms to make sure PD negotiation is completed 
		// before EDID read is attempted.
		timer_start(&dev->hdmi_timer, 750, timeout_hpd_debounce);
	}
#endif

	return;
}

